@@ -38,14 +38,7 @@
 
 #include <gdk/gdk.h>
 
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtklabel.h>
+#include <gtk/gtk.h>
 
 #include "brasero-utils.h"
 #include "brasero-disc.h"
@@ -503,61 +496,47 @@ brasero_burn_dialog_insert_disc_cb (BraseroBurn *burn,
 
 	if (error == BRASERO_BURN_WARNING_INSERT_AFTER_COPY) {
 		secondary_message = g_strdup (_("An image of the disc has been created on your hard drive."
-						"\nBurning will begin as soon as a recordable medium is inserted."));
+						"\nBurning will begin as soon as a recordable disc is inserted."));
 		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
 	}
 	else if (error == BRASERO_BURN_WARNING_CHECKSUM) {
-		secondary_message = g_strdup (_("A data integrity test will begin as soon as the medium is inserted."));
+		secondary_message = g_strdup (_("A data integrity test will begin as soon as the disc is inserted."));
 		main_message = g_strdup (_("Please, re-insert the disc in the CD/DVD burner."));
 	}
-	else if (error == BRASERO_BURN_WARNING_NEXT_COPY) {
-		secondary_message = g_strdup (_("A recording was successfully completed."
-						"\nThe next recording will begin as soon as a recordable medium is inserted."));
-		main_message = brasero_burn_dialog_get_media_type_string (burn, type, TRUE);
-	}
-	else if (error == BRASERO_BURN_ERROR_MEDIA_BUSY) {
-		main_message = g_strdup_printf (_("The disc in \"%s\" is busy."), drive_name);
-		secondary_message = g_strdup (_("make sure another application is not using it."));
+	else if (error == BRASERO_BURN_ERROR_DRIVE_BUSY) {
+		/* Translators: %s is the name of a drive */
+		main_message = g_strdup_printf (_("\"%s\" is busy."), drive_name);
+		secondary_message = g_strdup_printf ("%s.", _("Make sure another application is not using it"));
 	} 
-	else if (error == BRASERO_BURN_ERROR_MEDIA_NONE) {
+	else if (error == BRASERO_BURN_ERROR_MEDIUM_NONE) {
 		secondary_message = g_strdup_printf (_("There is no disc in \"%s\"."), drive_name);
 		main_message = brasero_burn_dialog_get_media_type_string (burn, type, TRUE);
 	}
-	else if (error == BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED) {
+	else if (error == BRASERO_BURN_ERROR_MEDIUM_INVALID) {
 		secondary_message = g_strdup_printf (_("The disc in \"%s\" is not supported."), drive_name);
 		main_message = brasero_burn_dialog_get_media_type_string (burn, type, TRUE);
 	}
-	else if (error == BRASERO_BURN_ERROR_MEDIA_NOT_REWRITABLE) {
+	else if (error == BRASERO_BURN_ERROR_MEDIUM_NOT_REWRITABLE) {
 		secondary_message = g_strdup_printf (_("The disc in \"%s\" is not rewritable."), drive_name);
 		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
 	}
-	else if (error == BRASERO_BURN_ERROR_MEDIA_BLANK) {
+	else if (error == BRASERO_BURN_ERROR_MEDIUM_NO_DATA) {
 		secondary_message = g_strdup_printf (_("The disc in \"%s\" is empty."), drive_name);
 		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
 	}
-	else if (error == BRASERO_BURN_ERROR_MEDIA_NOT_WRITABLE) {
+	else if (error == BRASERO_BURN_ERROR_MEDIUM_NOT_WRITABLE) {
 		secondary_message = g_strdup_printf (_("The disc in \"%s\" is not writable."), drive_name);
 		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
 	}
-	else if (error == BRASERO_BURN_ERROR_DVD_NOT_SUPPORTED) {
-		secondary_message = g_strdup_printf (_("The disc in \"%s\" is a DVD."), drive_name);
-		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
-	}
-	else if (error == BRASERO_BURN_ERROR_CD_NOT_SUPPORTED) {
-		secondary_message = g_strdup_printf (_("The disc in \"%s\" is a CD."), drive_name);
-		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
-	}
-	else if (error == BRASERO_BURN_ERROR_MEDIA_SPACE) {
-		secondary_message = g_strdup_printf (_("The disc in \"%s\" is not big enough."), drive_name);
+	else if (error == BRASERO_BURN_ERROR_MEDIUM_SPACE) {
+		secondary_message = g_strdup_printf (_("Not enough space available on the disc in \"%s\"."), drive_name);
 		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
 	}
 	else if (error == BRASERO_BURN_ERROR_NONE) {
-		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
-		main_message = g_strdup_printf ("<b><big>%s</big></b>", secondary_message);
-		g_free (secondary_message);
+		main_message = brasero_burn_dialog_get_media_type_string (burn, type, TRUE);
 		secondary_message = NULL;
 	}
-	else if (error == BRASERO_BURN_ERROR_RELOAD_MEDIA) {
+	else if (error == BRASERO_BURN_ERROR_MEDIUM_NEED_RELOADING) {
 		secondary_message = g_strdup_printf (_("The disc in \"%s\" needs to be reloaded."), drive_name);
 		main_message = g_strdup (_("Please, eject the disc and reload it."));
 	}
@@ -580,13 +559,20 @@ brasero_burn_dialog_insert_disc_cb (BraseroBurn *burn,
 			g_free (secondary_message);
 		}
 	}
-	else
+	else {
+		gchar *string;
+
 		message = gtk_message_dialog_new_with_markup (window,
 							      GTK_DIALOG_DESTROY_WITH_PARENT|
 							      GTK_DIALOG_MODAL,
 							      GTK_MESSAGE_WARNING,
 							      GTK_BUTTONS_CANCEL,
-							      "%s", main_message);
+							      NULL);
+
+		string = g_strdup_printf ("<b><big>%s</big></b>", main_message);
+		gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (message), string);
+		g_free (string);
+	}
 
 	g_free (main_message);
 
@@ -629,9 +615,152 @@ brasero_burn_dialog_insert_disc_cb (BraseroBurn *burn,
 	return BRASERO_BURN_OK;
 }
 
+static BraseroBurnResult
+brasero_burn_dialog_image_error (BraseroBurn *burn,
+				 GError *error,
+				 gboolean is_temporary,
+				 BraseroBurnDialog *dialog)
+{
+	gint result;
+	gchar *path;
+	gchar *string;
+	GtkWindow *window;
+	GtkWidget *message;
+	gboolean hide = FALSE;
+
+	if (!GTK_WIDGET_VISIBLE (dialog)) {
+		gtk_widget_show (GTK_WIDGET (dialog));
+		hide = TRUE;
+	}
+
+	g_timer_stop (BRASERO_BURN_DIALOG (dialog)->priv->total_time);
+
+	window = GTK_WINDOW (dialog);
+
+	string = g_strdup_printf ("%s. %s",
+				  is_temporary?
+				  _("A file could not be created at the location specified for temporary files"):
+				  _("The image could not be created at the specified location"),
+				  _("Do you want to specify another location for this session or retry with the current location?"));
+
+	message = gtk_message_dialog_new (window,
+					  GTK_DIALOG_DESTROY_WITH_PARENT|
+					  GTK_DIALOG_MODAL,
+					  GTK_MESSAGE_ERROR,
+					  GTK_BUTTONS_NONE,
+					  string);
+	g_free (string);
+
+	if (error && error->code == BRASERO_BURN_ERROR_DISK_SPACE)
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
+							 "%s.\n%s.",
+							  error->message,
+							  _("You may want to free some space on the disc and retry"));
+	else
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
+							 "%s.",
+							  error->message);
+
+	gtk_dialog_add_buttons (GTK_DIALOG (message),
+				_("_Keep Current Location"), GTK_RESPONSE_OK,
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				_("_Change Location"), GTK_RESPONSE_ACCEPT,
+				NULL);
+
+	result = gtk_dialog_run (GTK_DIALOG (message));
+	gtk_widget_destroy (message);
+
+	if (hide)
+		gtk_widget_hide (GTK_WIDGET (dialog));
+
+	if (result == GTK_RESPONSE_OK) {
+		g_timer_start (BRASERO_BURN_DIALOG (dialog)->priv->total_time);
+		return BRASERO_BURN_OK;
+	}
+
+	if (result != GTK_RESPONSE_ACCEPT) {
+		g_timer_start (BRASERO_BURN_DIALOG (dialog)->priv->total_time);
+		return BRASERO_BURN_CANCEL;
+	}
+
+	/* Show a GtkFileChooserDialog */
+	if (!is_temporary) {
+		gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (message), TRUE);
+		message = gtk_file_chooser_dialog_new (_("Location for Image File"),
+						       GTK_WINDOW (dialog),
+						       GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+						       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						       GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+						       NULL);
+	}
+	else
+		message = gtk_file_chooser_dialog_new (_("Location for Temporary Files"),
+						       GTK_WINDOW (dialog),
+						       GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+						       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						       GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+						       NULL);
+
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (message), TRUE);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (message), g_get_home_dir ());
+
+	result = gtk_dialog_run (GTK_DIALOG (message));
+	if (result != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (message);
+		g_timer_start (BRASERO_BURN_DIALOG (dialog)->priv->total_time);
+		return BRASERO_BURN_CANCEL;
+	}
+
+	path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (message));
+	gtk_widget_destroy (message);
+
+	if (!is_temporary) {
+		BraseroImageFormat format;
+		gchar *image = NULL;
+		gchar *toc = NULL;
+
+		format = brasero_burn_session_get_output_format (dialog->priv->session);
+		brasero_burn_session_get_output (dialog->priv->session,
+						 &image,
+						 &toc,
+						 NULL);
+
+		if (toc) {
+			gchar *name;
+
+			name = g_path_get_basename (toc);
+			g_free (toc);
+
+			toc = g_build_filename (path, name, NULL);
+			BRASERO_BURN_LOG ("New toc location %s", toc);
+		}
+
+		if (image) {
+			gchar *name;
+
+			name = g_path_get_basename (image);
+			g_free (image);
+
+			image = g_build_filename (path, name, NULL);
+			BRASERO_BURN_LOG ("New image location %s", toc);
+		}
+
+		brasero_burn_session_set_image_output_full (dialog->priv->session,
+							    format,
+							    image,
+							    toc);
+	}
+	else
+		brasero_burn_session_set_tmpdir (dialog->priv->session, path);
+
+	g_free (path);
+
+	g_timer_start (BRASERO_BURN_DIALOG (dialog)->priv->total_time);
+	return BRASERO_BURN_OK;
+}
+
 static BraseroBurnResult
 brasero_burn_dialog_loss_warnings_cb (GtkDialog *dialog, 
-				      const gchar *title,
 				      const gchar *main_message,
 				      const gchar *secondary_message,
 				      const gchar *button_text,
@@ -658,16 +787,10 @@ brasero_burn_dialog_loss_warnings_cb (GtkDialog *dialog,
 					  GTK_BUTTONS_NONE,
 					  "%s", main_message);
 
-	gtk_window_set_title (GTK_WINDOW (message), title);
-
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
 						 "%s", secondary_message);
 
-	gtk_dialog_add_buttons (GTK_DIALOG (message),
-				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				NULL);
-
-	button = brasero_utils_make_button (_("Replace disc"),
+	button = brasero_utils_make_button (_("_Replace Disc"),
 					    GTK_STOCK_REFRESH,
 					    NULL,
 					    GTK_ICON_SIZE_BUTTON);
@@ -675,6 +798,10 @@ brasero_burn_dialog_loss_warnings_cb (GtkDialog *dialog,
 	gtk_dialog_add_action_widget (GTK_DIALOG (message),
 				      button, GTK_RESPONSE_ACCEPT);
 
+	gtk_dialog_add_buttons (GTK_DIALOG (message),
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				NULL);
+
 	button = brasero_utils_make_button (button_text,
 					    NULL,
 					    button_icon,
@@ -705,10 +832,9 @@ brasero_burn_dialog_data_loss_cb (BraseroBurn *burn,
 				  GtkDialog *dialog)
 {
 	return brasero_burn_dialog_loss_warnings_cb (dialog,
-						     _("Possible Data Loss"),
 						     _("Do you really want to erase the current disc?"),
 						     _("The disc in the drive holds data."),
-						     _("Erase disc"),
+						     _("_Erase Disc"),
 						     "media-optical-blank");
 }
 
@@ -716,36 +842,60 @@ static BraseroBurnResult
 brasero_burn_dialog_previous_session_loss_cb (BraseroBurn *burn,
 					      GtkDialog *dialog)
 {
-	return brasero_burn_dialog_loss_warnings_cb (dialog,
-						     _("Multisession Disc"),
-						     _("Appending new files to a multisession disc is not advised:"),
-						     _("already burnt files will be invisible (though still readable).\nDo you want to continue anyway?"),
-						     _("Continue"),
-						     "media-optical-burn");
+	gchar *secondary;
+	BraseroBurnResult result;
+
+	secondary = g_strdup_printf ("%s\n%s",
+				     _("Already burnt files will be invisible (though still readable)."),
+				     _("Do you want to continue anyway?"));
+				     
+	result = brasero_burn_dialog_loss_warnings_cb (dialog,
+						       _("Appending new files to a multisession disc is not advised."),
+						       secondary,
+						       _("_Continue"),
+						       "media-optical-burn");
+	g_free (secondary);
+	return result;
 }
 
 static BraseroBurnResult
 brasero_burn_dialog_audio_to_appendable_cb (BraseroBurn *burn,
 					    GtkDialog *dialog)
 {
-	return brasero_burn_dialog_loss_warnings_cb (dialog,
-						     _("Multisession Disc"),
-						     _("Appending audio tracks to a CD is not advised:"),
-						     _("you might not be able to listen to them with stereos and CD-TEXT won't be written.\nDo you want to continue anyway?"),
-						     _("Continue"),
-						     "media-optical-burn");
+	gchar *secondary;
+	BraseroBurnResult result;
+
+	secondary = g_strdup_printf ("%s\n%s",
+				     _("You might not be able to listen to them with stereos and CD-TEXT won't be written."),
+				     _("Do you want to continue anyway?"));
+
+	result = brasero_burn_dialog_loss_warnings_cb (dialog,
+						       _("Appending audio tracks to a CD is not advised."),
+						       secondary,
+						       _("_Continue"),
+						       "media-optical-burn");
+	g_free (secondary);
+	return result;
 }
 
 static BraseroBurnResult
 brasero_burn_dialog_rewritable_cb (BraseroBurn *burn,
 				   GtkDialog *dialog)
 {
-	return brasero_burn_dialog_loss_warnings_cb (dialog,
-						     _("Rewritable Disc"),
-						     _("Recording audio tracks on a rewritable disc is not advised:"),
-						     _("you might not be able to listen to it with stereos.\nDo you want to continue anyway?"),
-						     _("Continue"),
-						     "media-optical-burn");
+	gchar *secondary;
+	BraseroBurnResult result;
+
+	secondary = g_strdup_printf ("%s\n%s",
+				     _("You might not be able to listen to it with stereos."),
+				     _("Do you want to continue anyway?"));
+
+	result = brasero_burn_dialog_loss_warnings_cb (dialog,
+						       _("Recording audio tracks on a rewritable disc is not advised."),
+						       secondary,
+						       _("_Continue"),
+						       "media-optical-burn");
+	g_free (secondary);
+	return result;
 }
 
 static BraseroBurnResult
@@ -771,17 +921,16 @@ brasero_burn_dialog_disable_joliet_cb (BraseroBurn *burn,
 					  GTK_DIALOG_MODAL,
 					  GTK_MESSAGE_WARNING,
 					  GTK_BUTTONS_NONE,
-					  _("Do you want to continue with Windows compatibility disabled?"));
+					  _("Do you want to continue with full Windows compatibility disabled?"));
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						  _("Some files don't have a suitable name for a Windows-compatible CD:"));
+						  _("Some files don't have a suitable name for a fully Windows-compatible CD."));
 
-	gtk_window_set_title (GTK_WINDOW (message), _("Windows Compatibility"));
 	gtk_dialog_add_buttons (GTK_DIALOG (message),
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				NULL);
 
-	button = brasero_utils_make_button (_("Continue"),
+	button = brasero_utils_make_button (_("_Continue"),
 					    GTK_STOCK_OK,
 					    NULL,
 					    GTK_ICON_SIZE_BUTTON);
@@ -1032,21 +1181,19 @@ brasero_burn_dialog_dummy_success_cb (BraseroBurn *burn,
 					  GTK_BUTTONS_CANCEL,
 					  _("The simulation was successful."));
 
-	gtk_window_set_title (GTK_WINDOW (message), _("Successful Simulation"));
-
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
 						  _("Real disc burning will take place in 10 seconds."));
 
-	button = brasero_utils_make_button (_("Burn Now"),
+	button = brasero_utils_make_button (_("Burn _Now"),
 					    NULL,
 					    "media-optical-burn",
 					    GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (message), button, GTK_RESPONSE_OK);
 
-	id = g_timeout_add (10000,
-			    brasero_burn_dialog_dummy_success_timeout,
-			    message);
+	id = g_timeout_add_seconds (10,
+				    brasero_burn_dialog_dummy_success_timeout,
+				    message);
 
 	gtk_widget_show (GTK_WIDGET (dialog));
 	gtk_window_set_urgency_hint (GTK_WINDOW (dialog), TRUE);
@@ -1115,24 +1262,25 @@ brasero_burn_dialog_init (BraseroBurnDialog * obj)
 
 	box = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (box);
-	gtk_box_pack_start (GTK_BOX (vbox), box, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), box, FALSE, TRUE, 0);
 
 	obj->priv->header = gtk_label_new (NULL);
 	gtk_widget_show (obj->priv->header);
 	gtk_misc_set_alignment (GTK_MISC (obj->priv->header), 0.0, 0.5);
 	gtk_misc_set_padding (GTK_MISC (obj->priv->header), 0, 18);
-	gtk_box_pack_start (GTK_BOX (box), obj->priv->header, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (box), obj->priv->header, FALSE, TRUE, 0);
 
 	obj->priv->image = gtk_image_new ();
+	gtk_misc_set_alignment (GTK_MISC (obj->priv->image), 1.0, 0.5);
 	gtk_widget_show (obj->priv->image);
-	gtk_box_pack_start (GTK_BOX (box), obj->priv->image, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (box), obj->priv->image, TRUE, TRUE, 0);
 
 	obj->priv->progress = brasero_burn_progress_new ();
 	gtk_widget_show (obj->priv->progress);
 	gtk_box_pack_start (GTK_BOX (vbox),
 			    obj->priv->progress,
 			    FALSE,
-			    FALSE,
+			    TRUE,
 			    0);
 
 	/* buttons */
@@ -1224,6 +1372,10 @@ brasero_burn_dialog_activity_start (BraseroBurnDialog *dialog)
 			  G_CALLBACK (brasero_burn_dialog_cancel_clicked_cb),
 			  dialog);
 
+	/* Reset or set the speed info */
+	g_object_set (dialog->priv->progress,
+		      "show-info", TRUE,
+		      NULL);
 	brasero_burn_progress_set_status (BRASERO_BURN_PROGRESS (dialog->priv->progress),
 					  FALSE,
 					  0.0,
@@ -1278,6 +1430,10 @@ brasero_burn_dialog_setup_session (BraseroBurnDialog *dialog,
 			  "insert-media",
 			  G_CALLBACK (brasero_burn_dialog_insert_disc_cb),
 			  dialog);
+	g_signal_connect (dialog->priv->burn,
+			  "location-request",
+			  G_CALLBACK (brasero_burn_dialog_image_error),
+			  dialog);
 	g_signal_connect (dialog->priv->burn,
 			  "warn-data-loss",
 			  G_CALLBACK (brasero_burn_dialog_data_loss_cb),
@@ -1434,8 +1590,8 @@ brasero_burn_dialog_show_log (BraseroBurnDialog *dialog)
 	logfile = brasero_burn_session_get_log_path (dialog->priv->session);
 	if (g_stat (logfile, &stats) == -1) {
 		brasero_utils_message_dialog (GTK_WIDGET (dialog),
-					      _("The session log cannot be displayed:"),
-					      _("the log file could not be found."),
+					      _("The session log cannot be displayed."),
+					      _("The log file could not be found"),
 					      GTK_MESSAGE_ERROR);
 		gtk_widget_destroy (message);
 		return;
@@ -1458,7 +1614,7 @@ brasero_burn_dialog_show_log (BraseroBurnDialog *dialog)
 			int errsv = errno;
 
 			brasero_utils_message_dialog (GTK_WIDGET (dialog),
-						      _("The session log cannot be displayed:"),
+						      _("The session log cannot be displayed."),
 						      g_strerror (errsv),
 						      GTK_MESSAGE_ERROR);
 			gtk_widget_destroy (message);
@@ -1469,7 +1625,7 @@ brasero_burn_dialog_show_log (BraseroBurnDialog *dialog)
 			int errsv = errno;
 
 			brasero_utils_message_dialog (GTK_WIDGET (dialog),
-						      _("The session log cannot be displayed:"),
+						      _("The session log cannot be displayed."),
 						      g_strerror (errsv),
 						      GTK_MESSAGE_ERROR);
 			gtk_widget_destroy (message);
@@ -1538,7 +1694,7 @@ brasero_burn_dialog_notify_error (BraseroBurnDialog *dialog,
 		g_error_free (error);
 	}
 	else
-		secondary = g_strdup (_("An unknown error occured. Check your disc"));
+		secondary = g_strdup (_("An unknown error occured."));
 
 	if (!GTK_WIDGET_VISIBLE (dialog))
 		gtk_widget_show (GTK_WIDGET (dialog));
@@ -1548,22 +1704,21 @@ brasero_burn_dialog_notify_error (BraseroBurnDialog *dialog,
 					  GTK_DIALOG_MODAL,
 					  GTK_MESSAGE_ERROR,
 					  GTK_BUTTONS_NONE,
-					  _("Error while burning:"));
+					  _("Error while burning."));
 
-	gtk_window_set_title (GTK_WINDOW (message), _("Burning Error"));
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						  "%s.",
+						  "%s",
 						  secondary);
 	g_free (secondary);
 
-	button = brasero_utils_make_button (_("Save log"),
+	button = brasero_utils_make_button (_("_Save Log"),
 					    GTK_STOCK_SAVE_AS,
 					    NULL,
 					    GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show_all (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (message), button, GTK_RESPONSE_APPLY);
 
-	button = brasero_utils_make_button (_("View log"),
+	button = brasero_utils_make_button (_("_View Log"),
 					    GTK_STOCK_EDIT,
 					    NULL,
 					    GTK_ICON_SIZE_BUTTON);
@@ -1589,7 +1744,7 @@ brasero_burn_dialog_notify_error (BraseroBurnDialog *dialog,
 	gtk_widget_destroy (message);
 }
 
-static void
+static gboolean
 brasero_burn_dialog_success_run (BraseroBurnDialog *dialog)
 {
 	gint answer;
@@ -1597,27 +1752,43 @@ brasero_burn_dialog_success_run (BraseroBurnDialog *dialog)
 	answer = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (answer == GTK_RESPONSE_CLOSE) {
 		GtkWidget *contents;
-		GtkWidget *window;
+		GValue *cover_value;
 		const gchar *title;
+		GtkWidget *window;
 		GSList *tracks;
 
 		contents = brasero_jacket_edit_dialog_new (GTK_WIDGET (dialog), &window);
 
 		title = brasero_burn_session_get_label (dialog->priv->session);
 		tracks = brasero_burn_session_get_tracks (dialog->priv->session);
-		brasero_jacket_edit_set_audio_tracks (BRASERO_JACKET_EDIT (contents), title, tracks);
+
+		cover_value = NULL;
+		brasero_burn_session_tag_lookup (dialog->priv->session,
+						 BRASERO_COVER_URI,
+						 &cover_value);
+
+		brasero_jacket_edit_set_audio_tracks (BRASERO_JACKET_EDIT (contents),
+						      title,
+						      cover_value? g_value_get_string (cover_value):NULL,
+						      tracks);
 
 		gtk_dialog_run (GTK_DIALOG (window));
+
+		return FALSE;
 	}
+
+	return (answer == GTK_RESPONSE_OK);
 }
 
-static void
+static gboolean
 brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog)
 {
 	gint64 rate;
+	gboolean res;
 	BraseroMedia media;
 	BraseroDrive *drive;
 	gchar *primary = NULL;
+	GtkWidget *make_another = NULL;
 
 	drive = brasero_burn_session_get_burner (dialog->priv->session);
 	if (dialog->priv->input.type != BRASERO_TRACK_TYPE_DISC)
@@ -1664,6 +1835,14 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog)
 	}
 
 	brasero_burn_dialog_activity_stop (dialog, primary);
+	g_free (primary);
+
+	if (!brasero_burn_session_is_dest_file (dialog->priv->session)) {
+		/* Useful button but it shouldn't be used for images */
+		make_another = gtk_dialog_add_button (GTK_DIALOG (dialog),
+						      _("Make _Another Copy"),
+						      GTK_RESPONSE_OK);
+	}
 
 	/* show total required time and average speed */
 	rate = 0;
@@ -1684,22 +1863,19 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog)
 						    media,
 						    dialog->priv->total_size);
 
-
-	if (brasero_burn_session_get_input_type (dialog->priv->session, NULL) == BRASERO_TRACK_TYPE_AUDIO) {
-		GtkWidget *button;
-
+	if (dialog->priv->input.type == BRASERO_TRACK_TYPE_AUDIO
+	|| (dialog->priv->input.type == BRASERO_TRACK_TYPE_DISC
+	&& (dialog->priv->input.subtype.media & BRASERO_MEDIUM_HAS_AUDIO))) {
 		/* since we succeed offer the possibility to create cover if that's an audio disc */
-		button = brasero_utils_make_button (_("Create cover"),
-						    NULL,
-						    NULL,
-						    GTK_ICON_SIZE_BUTTON);
-		gtk_widget_show (button);
-		gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_CLOSE);
+		gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Create Cover"), GTK_RESPONSE_CLOSE);
 	}
 
-	brasero_burn_dialog_success_run (dialog);
+	res = brasero_burn_dialog_success_run (dialog);
 
-	g_free (primary);
+	if (make_another)
+		gtk_widget_destroy (make_another);
+
+	return res;
 }
 
 static void
@@ -1769,6 +1945,8 @@ brasero_burn_dialog_end_session (BraseroBurnDialog *dialog,
 				 BraseroBurnResult result,
 				 GError *error)
 {
+	gboolean retry = FALSE;
+
 	if (dialog->priv->total_time)
 		g_timer_stop (dialog->priv->total_time);
 
@@ -1794,7 +1972,7 @@ brasero_burn_dialog_end_session (BraseroBurnDialog *dialog,
 			}
 		}
 
-		brasero_burn_dialog_notify_success (dialog);
+		retry = brasero_burn_dialog_notify_success (dialog);
 	}
 
 	if (dialog->priv->burn) {
@@ -1812,17 +1990,47 @@ brasero_burn_dialog_end_session (BraseroBurnDialog *dialog,
 		dialog->priv->total_time = NULL;
 	}
 
-	return FALSE;
+	return retry;
+}
+
+static BraseroBurnResult
+brasero_burn_dialog_record_session (BraseroBurnDialog *dialog,
+				    BraseroMedia media)
+{
+	gboolean retry;
+	GError *error = NULL;
+	BraseroBurnResult result;
+
+	brasero_burn_dialog_update_info (dialog, &dialog->priv->input, media);
+
+	/* start the recording session */
+	brasero_burn_dialog_activity_start (dialog);
+	result = brasero_burn_dialog_setup_session (dialog, &error);
+	if (result != BRASERO_BURN_OK)
+		return result;
+
+	result = brasero_burn_record (dialog->priv->burn,
+				      dialog->priv->session,
+				      &error);
+
+	retry = brasero_burn_dialog_end_session (dialog,
+						 result,
+						 error);
+
+	if (result == BRASERO_BURN_RETRY)
+		return result;
+
+	if (retry)
+		return BRASERO_BURN_RETRY;
+
+	return BRASERO_BURN_OK;
 }
 
 gboolean
 brasero_burn_dialog_run (BraseroBurnDialog *dialog,
-			 BraseroBurnSession *session,
-			 gboolean *destroy)
+			 BraseroBurnSession *session)
 {
 	BraseroMedia media;
-	GError *error = NULL;
-	gboolean close_dialog;
 	BraseroBurnResult result;
 
 	dialog->priv->session = session;
@@ -1846,27 +2054,13 @@ brasero_burn_dialog_run (BraseroBurnDialog *dialog,
 		media = brasero_medium_get_status (medium);
 	}
 
-	brasero_burn_dialog_update_info (dialog, &dialog->priv->input, media);
+	do {
+		result = brasero_burn_dialog_record_session (dialog, media);
+	} while (result == BRASERO_BURN_RETRY);
 
-	/* start the recording session */
-	brasero_burn_dialog_activity_start (dialog);
-
-	result = brasero_burn_dialog_setup_session (dialog, &error);
-	if (result == BRASERO_BURN_OK)
-		result = brasero_burn_record (dialog->priv->burn,
-					      session,
-					      &error);
-
-	close_dialog = brasero_burn_dialog_end_session (dialog,
-							result,
-							error);
-	
 	g_object_unref (dialog->priv->session);
 	dialog->priv->session = NULL;
 
-	if (destroy)
-		*destroy = close_dialog;
-
 	return (result == BRASERO_BURN_OK);
 }
 
@@ -1884,13 +2078,11 @@ brasero_burn_dialog_cancel_dialog (GtkWidget *toplevel)
 					  GTK_BUTTONS_NONE,
 					  _("Do you really want to quit?"));
 
-	gtk_window_set_title (GTK_WINDOW (message), _("Confirm"));
-
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG
 						  (message),
 						  _("Interrupting the process may make disc unusable."));
 
-	button = brasero_utils_make_button (("C_ontinue burning"),
+	button = brasero_utils_make_button (("C_ontinue Burning"),
 					    GTK_STOCK_OK,
 					    NULL,
 					    GTK_ICON_SIZE_BUTTON);
@@ -1898,7 +2090,7 @@ brasero_burn_dialog_cancel_dialog (GtkWidget *toplevel)
 	gtk_dialog_add_action_widget (GTK_DIALOG (message),
 				      button, GTK_RESPONSE_OK);
 
-	button = brasero_utils_make_button (_("_Cancel burning"),
+	button = brasero_utils_make_button (_("_Cancel Burning"),
 					    GTK_STOCK_CANCEL,
 					    NULL,
 					    GTK_ICON_SIZE_BUTTON);
