@@ -230,11 +230,9 @@ brasero_data_session_load_dir_result (GObject *owner,
 			       0,
 			       priv->loaded,
 			       FALSE);
-		/* FIXME */
-/*		error = g_error_new (BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
-				     _("unknown volume type"));
-*/		return;
+
+		/* FIXME: tell the user the error message */
+		return;
 	}
 
 	reference = GPOINTER_TO_INT (data);
