@@ -3,12 +3,10 @@
  * brasero
  * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -70,6 +68,7 @@ struct _BraseroDataVFSPrivate
 enum {
 	UNREADABLE_SIGNAL,
 	RECURSIVE_SIGNAL,
+	IMAGE_SIGNAL,
 	FILTERED_SIGNAL,
 	ACTIVITY_SIGNAL,
 	UNKNOWN_SIGNAL,
@@ -186,7 +185,12 @@ brasero_data_vfs_check_uri_result (BraseroDataVFS *self,
 
 	/* Only signal errors if the node was specifically added by the user
 	 * that is if it is loading. So check the loading GHashTable to know 
-	 * that. Otherwise this URI comes from directory exploration. */
+	 * that. Otherwise this URI comes from directory exploration.
+	 * The problem is the URI returned by brasero-io could be different
+	 * from the one set in the loading hash if there are parent symlinks.
+	 * That's one of the readon why we passed the orignal URI as a
+	 * registered string in the callback. Of course that's not true when
+	 * we're loading directory contents. */
 
 	if (error) {
 		if (error->domain == G_IO_ERROR && error->code == G_IO_ERROR_NOT_FOUND) {
@@ -326,6 +330,10 @@ brasero_data_vfs_directory_check_symlink_loop (BraseroDataVFS *self,
 
 	priv = BRASERO_DATA_VFS_PRIVATE (self);
 
+	/* Of course for a loop to exist, it must be a directory */
+	if (g_file_info_get_file_type (info) != G_FILE_TYPE_DIRECTORY)
+		return FALSE;
+
 	target_uri = g_file_info_get_symlink_target (info);
 	if (!target_uri)
 		return FALSE;
@@ -377,7 +385,8 @@ brasero_data_vfs_directory_check_symlink_loop (BraseroDataVFS *self,
 
 		/* The next graft point must be the natural parent of this one */
 		next = parent->parent;
-		if (!next || next->is_root) {
+		if (!next || next->is_root || next->is_fake) {
+			/* It's not we're done */
 			g_free (parent_uri);
 			break;
 		}
@@ -471,9 +480,7 @@ brasero_data_vfs_directory_load_result (GObject *owner,
 
 		/* See if we are supposed to keep them */
 		if (status != BRASERO_DATA_VFS_RESTORED && priv->filter_hidden) {
-			brasero_data_project_exclude_uri (BRASERO_DATA_PROJECT (self),
-							  uri);
-
+			brasero_data_project_exclude_uri (BRASERO_DATA_PROJECT (self), uri);
 			if (status == BRASERO_DATA_VFS_NONE) {
 				/* Advertise only once this filtered URI */
 				g_signal_emit (self,
@@ -598,6 +605,41 @@ brasero_data_vfs_loading_node_end (GObject *object,
 			       g_hash_table_size (priv->directories));
 }
 
+static BraseroBurnResult
+brasero_data_vfs_emit_image_signal (BraseroDataVFS *self,
+				    const gchar *uri)
+{
+	GValue instance_and_params [2];
+	GValue return_value;
+	GValue *params;
+
+	/* object which signalled */
+	instance_and_params->g_type = 0;
+	g_value_init (instance_and_params, G_TYPE_FROM_INSTANCE (self));
+	g_value_set_instance (instance_and_params, self);
+
+	/* arguments of signal (name) */
+	params = instance_and_params + 1;
+	params->g_type = 0;
+	g_value_init (params, G_TYPE_STRING);
+	g_value_set_string (params, uri);
+
+	/* default to FALSE */
+	return_value.g_type = 0;
+	g_value_init (&return_value, G_TYPE_INT);
+	g_value_set_int (&return_value, BRASERO_BURN_CANCEL);
+
+	g_signal_emitv (instance_and_params,
+			brasero_data_vfs_signals [IMAGE_SIGNAL],
+			0,
+			&return_value);
+
+	g_value_unset (instance_and_params);
+	g_value_unset (params);
+
+	return g_value_get_int (&return_value);
+}
+
 static void
 brasero_data_vfs_loading_node_result (GObject *owner,
 				      GError *error,
@@ -607,13 +649,16 @@ brasero_data_vfs_loading_node_result (GObject *owner,
 {
 	GSList *iter;
 	GSList *nodes;
+	BraseroFileNode *root;
+	BraseroFileTreeStats *stats;
 	gchar *registered = callback_data;
 	BraseroDataVFS *self = BRASERO_DATA_VFS (owner);
 	BraseroDataVFSPrivate *priv = BRASERO_DATA_VFS_PRIVATE (self);
 
 	nodes = g_hash_table_lookup (priv->loading, registered);
+
 	/* check the status of the operation */
-	if (!brasero_data_vfs_check_uri_result (self, uri, error, info)) {
+	if (!brasero_data_vfs_check_uri_result (self, registered, error, info)) {
 		/* we need to remove the loading node that is waiting */
 		for (iter = nodes; iter; iter = iter->next) {
 			BraseroFileNode *node;
@@ -621,12 +666,47 @@ brasero_data_vfs_loading_node_result (GObject *owner,
 
 			reference = GPOINTER_TO_INT (iter->data);
 			node = brasero_data_project_reference_get (BRASERO_DATA_PROJECT (self), reference);
-			brasero_data_project_remove_node (BRASERO_DATA_PROJECT (self), node);
+
+			/* the node could have been removed in the mean time */
+			if (node)
+				brasero_data_project_remove_node (BRASERO_DATA_PROJECT (self), node);
 		}
 
 		return;
 	}
 
+	/* It can happen that the user made a mistake out of ignorance or for
+	 * whatever other reason and dropped an image he wanted to burn.
+	 * So if our file is the only one in the project and if that's an image
+	 * check it is an image. If so, ask him if that's he really want to do. */
+	root = brasero_data_project_get_root (BRASERO_DATA_PROJECT (self));
+	stats = BRASERO_FILE_NODE_STATS (root);
+
+	if (stats && !stats->children
+	&& (!strcmp (g_file_info_get_content_type (info), "application/x-toc")
+	||  !strcmp (g_file_info_get_content_type (info), "application/x-cdrdao-toc")
+	||  !strcmp (g_file_info_get_content_type (info), "application/x-cue")
+	||  !strcmp (g_file_info_get_content_type (info), "application/x-cd-image"))) {
+		BraseroBurnResult result;
+
+		result = brasero_data_vfs_emit_image_signal (self, uri);
+		if (result == BRASERO_BURN_CANCEL) {
+			for (iter = nodes; iter; iter = iter->next) {
+				BraseroFileNode *node;
+				guint reference;
+
+				reference = GPOINTER_TO_INT (iter->data);
+				node = brasero_data_project_reference_get (BRASERO_DATA_PROJECT (self), reference);
+
+				/* the node could have been removed in the mean time */
+				if (node)
+					brasero_data_project_remove_node (BRASERO_DATA_PROJECT (self), node);
+			}
+
+			return;
+		}
+	}
+
 	/* NOTE: we don't check for a broken symlink here since the  user chose
 	 * to add it. So even if it were we would have to add it. The same for
 	 * hidden files. */
@@ -655,12 +735,18 @@ brasero_data_vfs_loading_node_result (GObject *owner,
 		 * reloading. */
 
 		if (!node->is_loading) {
-			brasero_data_project_node_reloaded (BRASERO_DATA_PROJECT (self), node, uri, info);
+			brasero_data_project_node_reloaded (BRASERO_DATA_PROJECT (self),
+							    node,
+							    uri,
+							    info);
 			continue;
 		}
 
 		/* update node */
-		brasero_data_project_node_loaded (BRASERO_DATA_PROJECT (self), node, uri, info);
+		brasero_data_project_node_loaded (BRASERO_DATA_PROJECT (self),
+						  node,
+						  uri,
+						  info);
 
 		/* See what type of file it is. If that's a directory then 
 		 * explore it right away */
@@ -722,43 +808,23 @@ brasero_data_vfs_loading_node (BraseroDataVFS *self,
 	GSList *nodes;
 
 	/* NOTE: this function receives URIs only from utf8 origins (not from
-	 * gnome-vfs for example) so we can assume that this is safe */
+	 * GIO for example) so we can assume that this is safe */
 
 	priv = BRASERO_DATA_VFS_PRIVATE (self);
 
-	if (!node->is_reloading) {
-		gchar *name;
-		GFile *vfs_uri;
-		gchar *unescaped_name;
+	if (!node->is_reloading
+	&&   BRASERO_FILE_NODE_NAME (node)
+	&&  !strcmp (BRASERO_FILE_NODE_NAME (node), G_DIR_SEPARATOR_S)) {
+		/* This is a root directory: we don't add it since a
+		 * child of the root directory can't be a root itself.
+		 * So we add all its contents under its parent. Remove
+		 * the loading node as well. 
+		 * Be careful in the next functions not to use node. */
+		brasero_data_vfs_load_directory (self, node->parent, uri);
 
-		/* g_path_get_basename is not comfortable with uri related
-		 * to the root directory so check that before */
-		vfs_uri = g_file_new_for_uri (uri);
-		name = g_file_get_basename (vfs_uri);
-		g_object_unref (vfs_uri);
-
-		unescaped_name = g_uri_unescape_string (name, NULL);
-		g_free (name);
-		name = unescaped_name;
-
-		if (!name)
-			return TRUE;
-
-		if (!strcmp (name, G_DIR_SEPARATOR_S)) {
-			g_free (name);
-
-			/* This is a root directory: we don't add it since a
-			 * child of the root directory can't be a root itself.
-			 * So we add all its contents under its parent. Remove
-			 * the loading node as well. 
-			 * Be careful in the next functions not to use node. */
-			brasero_data_vfs_load_directory (self, node->parent, uri);
-
-			/* node was invalidated: return FALSE */
-			brasero_data_project_remove_node (BRASERO_DATA_PROJECT (self), node);
-			return FALSE;
-		}
-		g_free (name);
+		/* node was invalidated: return FALSE */
+		brasero_data_project_remove_node (BRASERO_DATA_PROJECT (self), node);
+		return FALSE;
 	}
 
 	/* FIXME: we could know right from the start if that node is is_loading */
@@ -862,18 +928,19 @@ brasero_data_vfs_load_mime (BraseroDataVFS *self,
 	nodes = g_hash_table_lookup (priv->loading, uri);
 	if (nodes) {
 		gchar *registered;
+		GSList *iter;
 
 		registered = brasero_utils_register_string (uri);
 		g_free (uri);
 
-		for (; nodes; nodes = nodes->next) {
+		for (iter = nodes; iter; iter = iter->next) {
 			guint reference;
 			BraseroFileNode *ref_node;
 
-			reference = GPOINTER_TO_INT (nodes->data);
+			reference = GPOINTER_TO_INT (iter->data);
 			ref_node = brasero_data_project_reference_get (BRASERO_DATA_PROJECT (self), reference);
 			if (ref_node == node) {
-				/* 1sk for a higher priority */
+				/* Ask for a higher priority */
 				brasero_io_find_urgent (priv->io,
 							priv->load_uri,
 							brasero_data_vfs_increase_priority_cb,
@@ -886,7 +953,7 @@ brasero_data_vfs_load_mime (BraseroDataVFS *self,
 		/* It's loading, wait for the results */
 		reference = brasero_data_project_reference_new (BRASERO_DATA_PROJECT (self), node);
 		nodes = g_slist_prepend (nodes, GINT_TO_POINTER (reference));
-		g_hash_table_insert (priv->loading, (gchar *) uri, nodes);
+		g_hash_table_insert (priv->loading, registered, nodes);
 
 		/* Yet, ask for a higher priority */
 		brasero_io_find_urgent (priv->io,
@@ -933,7 +1000,8 @@ brasero_data_vfs_node_added (BraseroDataProject *project,
 		if (brasero_data_vfs_loading_node (self, node, uri))
 			goto chain;
 
-		goto chain;
+		/* The node was invalidated. So there's no need to pass it on */
+		return FALSE;
 	}
 
 	/* NOTE: a symlink pointing to a directory will return TRUE. */
@@ -1065,6 +1133,42 @@ brasero_data_vfs_reset (BraseroDataProject *project,
 		BRASERO_DATA_PROJECT_CLASS (brasero_data_vfs_parent_class)->reset (project, num_nodes);
 }
 
+static void
+brasero_data_vfs_filter_hidden_changed (GConfClient *client,
+					guint cxn,
+					GConfEntry *entry,
+					gpointer data)
+{
+	BraseroDataVFSPrivate *priv;
+	GConfValue *value;
+
+	priv = BRASERO_DATA_VFS_PRIVATE (data);
+
+	value = gconf_entry_get_value (entry);
+	if (value->type != GCONF_VALUE_BOOL)
+		return;
+
+	priv->filter_hidden = gconf_value_get_bool (value);
+}
+
+static void
+brasero_data_vfs_filter_broken_sym_changed (GConfClient *client,
+					    guint cxn,
+					    GConfEntry *entry,
+					    gpointer data)
+{
+	BraseroDataVFSPrivate *priv;
+	GConfValue *value;
+
+	priv = BRASERO_DATA_VFS_PRIVATE (data);
+
+	value = gconf_entry_get_value (entry);
+	if (value->type != GCONF_VALUE_BOOL)
+		return;
+
+	priv->filter_broken_sym = gconf_value_get_bool (value);
+}
+
 static void
 brasero_data_vfs_init (BraseroDataVFS *object)
 {
@@ -1081,6 +1185,18 @@ brasero_data_vfs_init (BraseroDataVFS *object)
 	priv->filter_broken_sym = gconf_client_get_bool (client,
 							 BRASERO_FILTER_BROKEN_SYM_KEY,
 							 NULL);
+	gconf_client_notify_add (client,
+				 BRASERO_FILTER_HIDDEN_KEY,
+				 brasero_data_vfs_filter_hidden_changed,
+				 object,
+				 NULL,
+				 NULL);
+	gconf_client_notify_add (client,
+				 BRASERO_FILTER_BROKEN_SYM_KEY,
+				 brasero_data_vfs_filter_broken_sym_changed,
+				 object,
+				 NULL,
+				 NULL);
 	g_object_unref (client);
 
 	/* create the hash tables */
@@ -1100,6 +1216,7 @@ brasero_data_vfs_finalize (GObject *object)
 	brasero_data_vfs_clear (BRASERO_DATA_VFS (object));
 
 	priv = BRASERO_DATA_VFS_PRIVATE (object);
+
 	if (priv->loading) {
 		g_hash_table_destroy (priv->loading);
 		priv->loading = NULL;
@@ -1153,6 +1270,17 @@ brasero_data_vfs_class_init (BraseroDataVFSClass *klass)
 			  1,
 			  G_TYPE_BOOLEAN);
 
+	brasero_data_vfs_signals [IMAGE_SIGNAL] = 
+	    g_signal_new ("image_uri",
+			  G_TYPE_FROM_CLASS (klass),
+			  G_SIGNAL_RUN_LAST|G_SIGNAL_NO_RECURSE,
+			  0,
+			  NULL, NULL,
+			  brasero_marshal_INT__STRING,
+			  G_TYPE_INT,
+			  1,
+			  G_TYPE_STRING);
+
 	brasero_data_vfs_signals [FILTERED_SIGNAL] = 
 	    g_signal_new ("filtered_uri",
 			  G_TYPE_FROM_CLASS (klass),
