@@ -32,16 +32,7 @@
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
 
-#include <gtk/gtktoolbar.h>
-#include <gtk/gtkuimanager.h>
-#include <gtk/gtkcontainer.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtktreednd.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtkeventbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkmisc.h>
+#include <gtk/gtk.h>
 
 #include "brasero-marshal.h"
 #include "brasero-disc.h"
@@ -142,23 +133,6 @@ brasero_disc_base_init (gpointer g_class)
 	initialized = TRUE;
 }
 
-BraseroDiscResult
-brasero_disc_can_add_uri (BraseroDisc *disc,
-			  const gchar *uri)
-{
-	BraseroDiscIface *iface;
-
-	g_return_val_if_fail (BRASERO_IS_DISC (disc), BRASERO_DISC_ERROR_UNKNOWN);
-	g_return_val_if_fail (uri != NULL, BRASERO_DISC_ERROR_UNKNOWN);
-	
-	iface = BRASERO_DISC_GET_IFACE (disc);
-	if (iface->can_add_uri)
-		return (* iface->can_add_uri) (disc, uri);
-
-	/* default to OK */
-	return BRASERO_DISC_OK;
-}
-
 BraseroDiscResult
 brasero_disc_add_uri (BraseroDisc *disc,
 		      const gchar *uri)
@@ -212,7 +186,9 @@ brasero_disc_reset (BraseroDisc *disc)
 }
 
 BraseroDiscResult
-brasero_disc_get_status (BraseroDisc *disc)
+brasero_disc_get_status (BraseroDisc *disc,
+			 gint *remaining,
+			 gchar **current_task)
 {
 	BraseroDiscIface *iface;
 
@@ -220,7 +196,9 @@ brasero_disc_get_status (BraseroDisc *disc)
 	
 	iface = BRASERO_DISC_GET_IFACE (disc);
 	if (iface->get_status)
-		return (* iface->get_status) (disc);
+		return (* iface->get_status) (disc,
+					      remaining,
+					      current_task);
 
 	return BRASERO_DISC_ERROR_UNKNOWN;
 }
@@ -672,8 +650,8 @@ brasero_disc_get_use_info_notebook (void)
 	 * You simply have to translate messages in the best form
          * for a list of actions. */
 	message_add_header = g_strconcat ("<big>", _("To add files to this project you can:"), "\n</big>", NULL);
-	message_add = g_strconcat ("\t* ", _("click the \"Add\" button to show the selection pane"), "\n",
-				   "\t* ", _("select files in selection pane and click the \"Add\" button"), "\n",
+	message_add = g_strconcat ("\t* ", _("click the \"Add\" button to show a selection dialog"), "\n",
+				   "\t* ", _("select files in the selection pane and click the \"Add\" button"), "\n",
 				   "\t* ", _("drag files in this area from the selection pane or from the file manager"), "\n",
 				   "\t* ", _("double click on files in the selection pane"), "\n",
 
