@@ -3,12 +3,10 @@
  * brasero
  * Copyright (C) Rouquier Philippe 2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,6 +34,7 @@
 #include "brasero-jacket-view.h"
 #include "brasero-jacket-font.h"
 #include "brasero-utils.h"
+#include "brasero-tool-color-picker.h"
 
 #include "burn-track.h"
 
@@ -56,6 +55,8 @@ struct _BraseroJacketEditPrivate
 	GtkWidget *underline;
 	GtkWidget *italic;
 	GtkWidget *bold;
+
+	GtkWidget *background;
 };
 
 #define BRASERO_JACKET_EDIT_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_JACKET_EDIT, BraseroJacketEditPrivate))
@@ -68,7 +69,6 @@ enum {
 	SNAP_NUM_COL
 };
 
-
 G_DEFINE_TYPE (BraseroJacketEdit, brasero_jacket_edit, GTK_TYPE_VBOX);
 
 static void
@@ -333,7 +333,7 @@ brasero_jacket_edit_colours_changed_cb (GtkColorButton *button,
 	if (!priv->current_view)
 		return;
 
-	gtk_color_button_get_color (button, &color);
+	brasero_tool_color_picker_get_color (BRASERO_TOOL_COLOR_PICKER (button), &color);
 
 	buffer = brasero_jacket_view_get_active_buffer (BRASERO_JACKET_VIEW (priv->current_view));
 	tag = gtk_text_buffer_create_tag (buffer, NULL,
@@ -365,13 +365,15 @@ brasero_jacket_edit_font_changed_cb (BraseroJacketFont *button,
 	if (!priv->current_view)
 		return;
 
+	buffer = brasero_jacket_view_get_active_buffer (BRASERO_JACKET_VIEW (priv->current_view));
+	if (!buffer)
+		return;
+
 	font_name = brasero_jacket_font_get_name (button);
 	if (!font_name)
 		return;
 
 	desc = pango_font_description_from_string (font_name);
-
-	buffer = brasero_jacket_view_get_active_buffer (BRASERO_JACKET_VIEW (priv->current_view));
 	tag = gtk_text_buffer_create_tag (buffer, NULL,
 					  "font-desc", desc,
 					  NULL);
@@ -384,6 +386,17 @@ brasero_jacket_edit_font_changed_cb (BraseroJacketFont *button,
 		brasero_jacket_buffer_add_default_tag (BRASERO_JACKET_BUFFER (buffer), tag);
 }
 
+static void
+brasero_jacket_edit_configure_background_pressed_cb (GtkToolButton *button,
+						     BraseroJacketEdit *self)
+{
+	BraseroJacketEditPrivate *priv;
+
+	priv = BRASERO_JACKET_EDIT_PRIVATE (self);
+	if (priv->current_view)
+		brasero_jacket_view_configure_background (BRASERO_JACKET_VIEW (priv->current_view));
+}
+
 static void
 brasero_jacket_edit_update_button_state (BraseroJacketEdit *self)
 {
@@ -396,10 +409,14 @@ brasero_jacket_edit_update_button_state (BraseroJacketEdit *self)
 
 	priv = BRASERO_JACKET_EDIT_PRIVATE (self);
 
-	if (priv->current_view)
+	if (priv->current_view) {
 		buffer = brasero_jacket_view_get_active_buffer (BRASERO_JACKET_VIEW (priv->current_view));
-	else
+		gtk_widget_set_sensitive (priv->background, TRUE);
+	}
+	else {
 		buffer = NULL;
+		gtk_widget_set_sensitive (priv->background, FALSE);
+	}
 
 	if (!buffer)
 		return;
@@ -416,7 +433,7 @@ brasero_jacket_edit_update_button_state (BraseroJacketEdit *self)
 	attributes = brasero_jacket_view_get_default_attributes (BRASERO_JACKET_VIEW (priv->current_view));
 	gtk_text_iter_get_attributes (&iter, attributes);
 
-	gtk_color_button_set_color (GTK_COLOR_BUTTON (priv->colours), &attributes->appearance.fg_color);
+	brasero_tool_color_picker_set_color (BRASERO_TOOL_COLOR_PICKER (priv->colours), &attributes->appearance.fg_color);
 	
 	font_name = pango_font_description_to_string (attributes->font);
 	brasero_jacket_font_set_name (BRASERO_JACKET_FONT (priv->fonts), font_name);
@@ -522,6 +539,7 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 	gtk_widget_show (toolbar);
 	gtk_box_pack_start (GTK_BOX (object), toolbar, FALSE, TRUE, 0);
 
+	/* Items */
 	item = GTK_WIDGET (gtk_tool_button_new_from_stock (GTK_STOCK_PRINT));
 	gtk_widget_show (item);
 	g_signal_connect (item,
@@ -530,6 +548,26 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 			  object);
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
 
+	item = GTK_WIDGET (gtk_separator_tool_item_new ());
+	gtk_widget_show (item);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
+
+	item = GTK_WIDGET (gtk_tool_button_new (NULL, _("Bac_kground Properties")));
+	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (item), "background");
+	gtk_tool_button_set_use_underline (GTK_TOOL_BUTTON (item), TRUE);
+	gtk_widget_show (item);
+	gtk_widget_set_sensitive (item, FALSE);
+	g_signal_connect (item,
+			  "clicked",
+			  G_CALLBACK (brasero_jacket_edit_configure_background_pressed_cb),
+			  object);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
+	priv->background = item;
+
+	item = GTK_WIDGET (gtk_separator_tool_item_new ());
+	gtk_widget_show (item);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
+
 	item = GTK_WIDGET (gtk_radio_tool_button_new_from_stock (NULL, GTK_STOCK_JUSTIFY_RIGHT));
 	gtk_widget_show (item);
 	g_signal_connect (item,
@@ -557,6 +595,10 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
 	priv->left = item;
 
+	item = GTK_WIDGET (gtk_separator_tool_item_new ());
+	gtk_widget_show (item);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
+
 	item = GTK_WIDGET (gtk_toggle_tool_button_new_from_stock (GTK_STOCK_UNDERLINE));
 	gtk_widget_show (item);
 	g_signal_connect (item,
@@ -584,6 +626,10 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
 	priv->bold = item;
 
+	item = GTK_WIDGET (gtk_separator_tool_item_new ());
+	gtk_widget_show (item);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
+
 	priv->fonts = brasero_jacket_font_new ();
 	gtk_widget_show (priv->fonts);
 	g_signal_connect (priv->fonts,
@@ -601,18 +647,16 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 	gtk_tool_item_set_expand (GTK_TOOL_ITEM (item), FALSE);
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
 
-	priv->colours = gtk_color_button_new ();
-	gtk_button_set_focus_on_click (GTK_BUTTON (priv->colours), FALSE);
+	priv->colours = brasero_tool_color_picker_new ();
+	brasero_tool_color_picker_set_text (BRASERO_TOOL_COLOR_PICKER (priv->colours),
+					    _("_Text Colour"));
 	gtk_widget_show (priv->colours);
 	g_signal_connect (priv->colours,
 			  "color-set",
 			  G_CALLBACK (brasero_jacket_edit_colours_changed_cb),
 			  object);
 
-	item = GTK_WIDGET (gtk_tool_item_new ());
-	gtk_widget_show (item);
-	gtk_container_add (GTK_CONTAINER (item), priv->colours);
-	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 1);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (priv->colours), 1);
 
 	/* contents */
 	vbox = gtk_vbox_new (FALSE, 4);
@@ -624,7 +668,6 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll), GTK_SHADOW_IN);
 	gtk_widget_show (scroll);
 	gtk_box_pack_start (GTK_BOX (vbox), scroll, TRUE, TRUE, 0);
 
@@ -673,6 +716,7 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 void
 brasero_jacket_edit_set_audio_tracks (BraseroJacketEdit *self,
 				      const gchar *label,
+				      const gchar *cover,
 				      GSList *tracks)
 {
 	BraseroJacketEditPrivate *priv;
@@ -682,22 +726,37 @@ brasero_jacket_edit_set_audio_tracks (BraseroJacketEdit *self,
 
 	priv = BRASERO_JACKET_EDIT_PRIVATE (self);
 
-	buffer = brasero_jacket_view_get_body_buffer (BRASERO_JACKET_VIEW (priv->front));
+	/* Set background for front cover */
+	if (cover) {
+		gchar *path;
 
-	/* create the tags */
-	gtk_text_buffer_create_tag (buffer,
-				    "Title",
-				    "justification", GTK_JUSTIFY_CENTER,
-				    "weight", PANGO_WEIGHT_BOLD,
-				    "scale", PANGO_SCALE_X_LARGE,
-				    "stretch", PANGO_STRETCH_ULTRA_EXPANDED,
-				    NULL);
+		path = g_filename_from_uri (cover, NULL, NULL);
+		if (!path)
+			path = g_strdup (cover);
 
-	gtk_text_buffer_get_start_iter (buffer, &start);
+		brasero_jacket_view_set_image_style (BRASERO_JACKET_VIEW (priv->front), BRASERO_JACKET_IMAGE_STRETCH);
+		brasero_jacket_view_set_image (BRASERO_JACKET_VIEW (priv->front), path);
+		g_free (path);
+	}
+	else {
+		/* Otherwise we create a very rough one */
+		buffer = brasero_jacket_view_get_body_buffer (BRASERO_JACKET_VIEW (priv->front));
 
-	if (label) {
-		BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, "\n\n\n\n", "Title", &start);
-		BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, label, "Title", &start);
+		/* create the tags */
+		gtk_text_buffer_create_tag (buffer,
+					    "Title",
+					    "justification", GTK_JUSTIFY_CENTER,
+					    "weight", PANGO_WEIGHT_BOLD,
+					    "scale", PANGO_SCALE_X_LARGE,
+					    "stretch", PANGO_STRETCH_ULTRA_EXPANDED,
+					    NULL);
+
+		gtk_text_buffer_get_start_iter (buffer, &start);
+
+		if (label) {
+			BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, "\n\n\n\n", "Title", &start);
+			BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, label, "Title", &start);
+		}
 	}
 
 	buffer = brasero_jacket_view_get_body_buffer (BRASERO_JACKET_VIEW (priv->back));
@@ -747,34 +806,42 @@ brasero_jacket_edit_set_audio_tracks (BraseroJacketEdit *self,
 
 		info = brasero_track_get_audio_info (track);
 
-		if (info->title) {
-			BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, info->title, "Subtitle", &start);
+		if (info) {
+			if (info->title) {
+				BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, info->title, "Subtitle", &start);
+			}
+			else {
+				BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, _("Unknown song"), "Subtitle", &start);
+			}
+
+			BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, "\t\t", "Subtitle", &start);
+
+			time = brasero_utils_get_time_string (brasero_track_get_audio_end (track) -
+							      brasero_track_get_audio_start (track),
+							      TRUE,
+							      FALSE);
+			BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, time, "Subtitle", &start);
+			g_free (time);
+
+			BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, "\n", "Subtitle", &start);
+
+			if (info->artist) {
+				/* Translators: "by" is followed by the name of an artist.
+				 * This text is the one written on the cover of a disc.
+				 * Before it there is the name of the song.
+				 * I had to break it because it is in a GtkTextBuffer
+				 * and every word has a different tag. */
+				BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, _("by"), "Artist", &start);
+				BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, " ", "Artist", &start);
+				BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, info->artist, "Artist", &start);
+				BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, " ", "Artist", &start);
+			}
+
+			if (info->composer)
+				BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, info->composer, "Subtitle", &start);
+
+			BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, "\n\n", "Subtitle", &start);
 		}
-		else {
-			BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, _("Unknown song"), "Subtitle", &start);
-		}
-
-		BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, "\t\t", "Subtitle", &start);
-
-		time = brasero_utils_get_time_string (brasero_track_get_audio_end (track) -
-						      brasero_track_get_audio_start (track),
-						      TRUE,
-						      FALSE);
-		BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, time, "Subtitle", &start);
-		g_free (time);
-
-		BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, "\n", "Subtitle", &start);
-
-		if (info->artist) {
-			BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, _("by "), "Artist", &start);
-			BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, info->artist, "Artist", &start);
-			BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, " ", "Artist", &start);
-		}
-
-		if (info->composer)
-			BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, info->composer, "Subtitle", &start);
-
-		BRASERO_JACKET_EDIT_INSERT_TAGGED_TEXT (buffer, "\n\n", "Subtitle", &start);
 	}
 
 	/* side */
