@@ -3,12 +3,10 @@
  * brasero
  * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,8 +26,7 @@
 
 #include <glib/gi18n.h>
 
-#include <gtk/gtknotebook.h>
-#include <gtk/gtkdialog.h>
+#include <gtk/gtk.h>
 
 #include "brasero-pref.h"
 #include "brasero-plugin-manager-ui.h"
@@ -65,7 +62,7 @@ brasero_pref_init (BraseroPref *object)
 	gtk_dialog_set_has_separator (GTK_DIALOG (object), FALSE);
 
 	gtk_window_set_default_size (GTK_WINDOW (object), 550, 600);
-	gtk_window_set_title (GTK_WINDOW (object), "Brasero Plugins");
+	gtk_window_set_title (GTK_WINDOW (object), _("Brasero Plugins"));
 
 	notebook = gtk_notebook_new ();
 
