@@ -34,16 +34,7 @@
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
 
-#include <gtk/gtkstock.h>
-#include <gtk/gtkaction.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtkfilefilter.h>
-#include <gtk/gtkfilechooser.h>
-#include <gtk/gtkfilechooserwidget.h>
-#include <gtk/gtkfilechooserdialog.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkuimanager.h>
-#include <gtk/gtkscrolledwindow.h>
+#include <gtk/gtk.h>
 
 #include "brasero-app.h"
 #include "brasero-utils.h"
@@ -118,15 +109,15 @@ static GtkActionEntry entries [] = {
 	{"NewChoose", GTK_STOCK_NEW, N_("_Empty Project"), NULL,
 	 N_("Let you choose your new project"), G_CALLBACK (brasero_project_manager_new_empty_prj_cb)},
 	{"NewAudio", "media-optical-audio-new", N_("New _Audio Project"), NULL,
-	 N_("Create a new audio project"), G_CALLBACK (brasero_project_manager_new_audio_prj_cb)},
+	 N_("Create a traditional audio CD that will be playable on computers and stereos"), G_CALLBACK (brasero_project_manager_new_audio_prj_cb)},
 	{"NewData", "media-optical-data-new", N_("New _Data Project"), NULL,
-	 N_("Create a new data project"), G_CALLBACK (brasero_project_manager_new_data_prj_cb)},
+	 N_("Create a CD/DVD containing any type of data that can only be read on a computer"), G_CALLBACK (brasero_project_manager_new_data_prj_cb)},
 	{"NewVideo", "media-optical-video-new", N_("New _Video Project"), NULL,
-	 N_("Create a new video project"), G_CALLBACK (brasero_project_manager_new_video_prj_cb)},
+	 N_("Create a video DVD or a SVCD that are readable on TV readers"), G_CALLBACK (brasero_project_manager_new_video_prj_cb)},
 	{"NewCopy", "media-optical-copy", N_("Copy _Disc..."), NULL,
-	 N_("Copy a disc"), G_CALLBACK (brasero_project_manager_new_copy_prj_cb)},
+	 N_("Create a 1:1 copy of an audio CD or a data CD/DVD on your hardisk or on another CD/DVD"), G_CALLBACK (brasero_project_manager_new_copy_prj_cb)},
 	{"NewIso", "iso-image-burn", N_("_Burn Image..."), NULL,
-	 N_("Burn an image"), G_CALLBACK (brasero_project_manager_new_iso_prj_cb)},
+	 N_("Burn an existing CD/DVD image to disc"), G_CALLBACK (brasero_project_manager_new_iso_prj_cb)},
 
 	{"Open", GTK_STOCK_OPEN, N_("_Open..."), NULL,
 	 N_("Open a project"), G_CALLBACK (brasero_project_manager_open_cb)},
@@ -370,9 +361,12 @@ brasero_project_manager_selected_uris_preview (gpointer data)
 	for (iter = manager->priv->selected; iter && *iter; iter ++)
 		list = g_slist_prepend (list, *iter);
 
-	flags = BRASERO_IO_INFO_RECURSIVE|BRASERO_IO_INFO_IDLE;
-	if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO)
+	flags = BRASERO_IO_INFO_IDLE;
+	if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO
+	||  manager->priv->type == BRASERO_PROJECT_TYPE_VIDEO)
 		flags |= BRASERO_IO_INFO_METADATA;
+	else if (manager->priv->type == BRASERO_PROJECT_TYPE_DATA)
+		flags |= BRASERO_IO_INFO_RECURSIVE;
 
 	brasero_io_get_file_count (manager->priv->io,
 				   list,
@@ -525,7 +519,6 @@ brasero_project_manager_burn (BraseroProjectManager *manager,
 {
 	GtkWidget *toplevel;
 	GtkWidget *dialog;
-	gboolean destroy;
 
 	/* now setup the burn dialog */
 	dialog = brasero_burn_dialog_new ();
@@ -538,21 +531,16 @@ brasero_project_manager_burn (BraseroProjectManager *manager,
 	gtk_widget_show (dialog);
 
 	brasero_burn_dialog_run (BRASERO_BURN_DIALOG (dialog),
-				 session,
-				 &destroy);
+				 session);
 
 	gtk_widget_destroy (dialog);
 
-	if (!destroy) {
-		brasero_project_manager_switch (manager,
-						BRASERO_PROJECT_TYPE_INVALID,
-						NULL,
-						NULL,
-						TRUE);
-		gtk_widget_show (toplevel);
-	}
-	else
-		gtk_widget_destroy (toplevel);
+	brasero_project_manager_switch (manager,
+					BRASERO_PROJECT_TYPE_INVALID,
+					NULL,
+					NULL,
+					TRUE);
+	gtk_widget_show (toplevel);
 }
 
 static void
@@ -607,7 +595,8 @@ brasero_project_manager_burn_iso_dialog (BraseroProjectManager *manager,
 
 static void
 brasero_project_manager_copy_disc (BraseroProjectManager *manager,
-				   const gchar *device)
+				   const gchar *device,
+				   const gchar *cover)
 {
 	BraseroBurnSession *session;
 	GtkResponseType result;
@@ -653,6 +642,18 @@ brasero_project_manager_copy_disc (BraseroProjectManager *manager,
 	session = brasero_burn_options_get_session (BRASERO_BURN_OPTIONS (dialog));
 	gtk_widget_destroy (dialog);
 
+	/* Set a cover if any. */
+	if (cover) {
+		GValue *value;
+
+		value = g_new0 (GValue, 1);
+		g_value_init (value, G_TYPE_STRING);
+		g_value_set_string (value, cover);
+		brasero_burn_session_tag_add (session,
+					      BRASERO_COVER_URI,
+					      value);
+	}
+
 	brasero_project_manager_burn (manager, session);
 	g_object_unref (session);
 
@@ -751,7 +752,7 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		}
 
 		if (toplevel)
-			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New Video disc project"));
+			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New Video Disc Project"));
 	}
 	else if (type == BRASERO_PROJECT_TYPE_ISO) {
 		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_NONE);
@@ -774,7 +775,7 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		if (toplevel)
 			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - Disc Copy"));
 
-		brasero_project_manager_copy_disc (manager, uri);
+		brasero_project_manager_copy_disc (manager, uri, NULL);
 	}
 }
 
@@ -883,13 +884,17 @@ brasero_project_manager_video (BraseroProjectManager *manager,
 
 void
 brasero_project_manager_copy (BraseroProjectManager *manager,
-			      const gchar *device)
+			      const gchar *device,
+			      const gchar *cover)
 {
-	brasero_project_manager_switch (manager,
-					BRASERO_PROJECT_TYPE_COPY,
-					NULL,
-					device,
-					TRUE);
+	if (manager->priv->oneshot)
+		brasero_project_manager_copy_disc (manager, device, cover);
+	else
+		brasero_project_manager_switch (manager,
+						BRASERO_PROJECT_TYPE_COPY,
+						NULL,
+						device,
+						TRUE);
 }
 
 void
@@ -931,6 +936,21 @@ brasero_project_manager_open_project (BraseroProjectManager *manager,
 	return type;
 }
 
+void
+brasero_project_manager_burn_project (BraseroProjectManager *manager,
+				      const gchar *uri)
+{
+	BraseroProjectType type;
+
+	brasero_project_manager_set_oneshot (manager, TRUE);
+	type = brasero_project_open_project (BRASERO_PROJECT (manager->priv->project), uri);
+	if (type == BRASERO_PROJECT_TYPE_INVALID)
+		return;
+
+	manager->priv->type = type;
+	brasero_project_burn (BRASERO_PROJECT (manager->priv->project));
+}
+
 #ifdef BUILD_PLAYLIST
 
 BraseroProjectType
@@ -1063,11 +1083,11 @@ brasero_project_manager_open_uri (BraseroProjectManager *manager,
 	else {
 		gchar *string;
 
-		string = g_strdup_printf (_("The project '%s' does not exist."), uri);
-		brasero_utils_message_dialog (gtk_widget_get_toplevel (GTK_WIDGET (manager)),
-					      _("Error while loading the project:"),
-					      string,
-					      GTK_MESSAGE_ERROR);
+		string = g_strdup_printf (_("The project \"%s\" does not exist"), uri);
+		brasero_app_alert (BRASERO_APP (gtk_widget_get_toplevel (GTK_WIDGET (manager))),
+				   _("Error while loading the project."),
+				   string,
+				   GTK_MESSAGE_ERROR);
 		g_free (string);
 
 		type = BRASERO_PROJECT_TYPE_INVALID;
@@ -1155,6 +1175,14 @@ brasero_project_manager_load_session (BraseroProjectManager *manager,
     	return TRUE;
 }
 
+static void
+brasero_project_manager_last_saved_clicked_cb (BraseroProjectTypeChooser *chooser,
+					       const gchar *path,
+					       BraseroProjectManager *manager)
+{
+	brasero_project_manager_load_session (manager, path);
+}
+
 gboolean
 brasero_project_manager_save_session (BraseroProjectManager *manager,
 				      const gchar *path,
@@ -1199,11 +1227,11 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 
 	action = gtk_action_group_get_action (obj->priv->action_group, "NewChoose");
 	g_object_set (action,
-		      "short-label", _("New"), /* for toolbar buttons */
+		      "short-label", _("_New"), /* for toolbar buttons */
 		      NULL);
 	action = gtk_action_group_get_action (obj->priv->action_group, "Open");
 	g_object_set (action,
-		      "short-label", _("Open"), /* for toolbar buttons */
+		      "short-label", _("_Open"), /* for toolbar buttons */
 		      NULL);
 
 	/* add the project type chooser to the notebook */
@@ -1217,6 +1245,10 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 			  "recent-clicked",
 			  G_CALLBACK (brasero_project_manager_recent_clicked_cb),
 			  obj);
+	g_signal_connect (type,
+			  "last-saved",
+			  G_CALLBACK (brasero_project_manager_last_saved_clicked_cb),
+			  obj);
 	gtk_notebook_prepend_page (GTK_NOTEBOOK (obj), type, NULL);
 
 	/* add the layout */
@@ -1251,7 +1283,6 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 				   chooser,
 				   "Chooser",
 				   _("Browse the file system"),
-				   _("Display File Browser"),
 				   GTK_STOCK_DIRECTORY,
 				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA|BRASERO_LAYOUT_VIDEO);
 
@@ -1275,7 +1306,6 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 				   search,
 				   "Search",
 				   _("Search files using keywords"),
-				   _("Display Search"),
 				   GTK_STOCK_FIND,
 				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA|BRASERO_LAYOUT_VIDEO);
 
@@ -1296,7 +1326,6 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 				   playlist,
 				   "Playlist",
 				   _("Display playlists and their contents"),
-				   _("Display Playlists"),
 				   "audio-x-generic", 
 				   BRASERO_LAYOUT_AUDIO);
 
