@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2008 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2005-2008 <bonfire-app@wanadoo.fr>
  * 
  * brasero is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -211,6 +211,7 @@ redo:
 			new_name = brasero_rename_number_string (self, name);
 			break;
 		default:
+			new_name = NULL;
 			break;
 		}
 
@@ -230,6 +231,7 @@ redo:
 		}
 		g_free (name);
 	}
+
 	g_list_foreach (selected, (GFunc) gtk_tree_path_free, NULL);
 	g_list_free (selected);
 	return TRUE;
@@ -292,6 +294,7 @@ brasero_rename_init (BraseroRename *object)
 	gtk_widget_show (hbox);
 	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), hbox, NULL);
 
+	/* Translators: This is a verb. This is completed later */
 	label = gtk_label_new (_("Insert"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
@@ -303,8 +306,15 @@ brasero_rename_init (BraseroRename *object)
 
 	combo = gtk_combo_box_new_text ();
 	gtk_widget_show (combo);
+
+	/* Translators: This finishes previous action "Insert". It goes like
+	 * this: "Insert" [Entry] "at the beginning". */
 	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("at the begining"));
+
+	/* Translators: This finishes previous action "Insert". It goes like
+	 * this: "Insert" [Entry] "at the end". */
 	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("at the end"));
+
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
 	gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0);
 	priv->insert_combo = combo;
@@ -328,6 +338,7 @@ brasero_rename_init (BraseroRename *object)
 	gtk_widget_show (hbox);
 	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), hbox, NULL);
 
+	/* Translators: this is a verb */
 	label = gtk_label_new (_("Substitute"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
@@ -337,6 +348,8 @@ brasero_rename_init (BraseroRename *object)
 	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
 	priv->substitute_entry = entry;
 
+	/* Translators: this goes with above verb to say "Substitute" [Entry]
+	 * "by" [Entry]. */
 	label = gtk_label_new (_("by"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
