@@ -7,19 +7,21 @@
 ****************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 
@@ -38,29 +40,11 @@
 
 #include <gio/gio.h>
 
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkfilefilter.h>
-#include <gtk/gtkselection.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcellrendererprogress.h>
-#include <gtk/gtktreeviewcolumn.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtktreemodelfilter.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkcomboboxentry.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtk.h>
 
 #include <beagle/beagle.h>
 
+#include "brasero-app.h"
 #include "brasero-utils.h"
 #include "brasero-search-entry.h"
 #include "brasero-mime-filter.h"
@@ -299,6 +283,7 @@ brasero_search_column_clicked (GtkTreeViewColumn *column,
 static void
 brasero_search_init (BraseroSearch *obj)
 {
+	gchar *string;
 	GtkWidget *box;
 	GtkWidget *box1;
 	GtkWidget *label;
@@ -336,7 +321,7 @@ brasero_search_init (BraseroSearch *obj)
 	box = gtk_hbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (obj), box, FALSE, FALSE, 0);
 
-	button = brasero_utils_make_button (_("Previous results"),
+	button = brasero_utils_make_button (_("Previous Results"),
 					    GTK_STOCK_GO_BACK,
 					    NULL,
 					    GTK_ICON_SIZE_BUTTON);
@@ -349,12 +334,15 @@ brasero_search_init (BraseroSearch *obj)
 	gtk_box_pack_start (GTK_BOX (box), button, FALSE, TRUE, 0);
 	obj->priv->left = button;
 
-	label = gtk_label_new (_("<b>No results</b>"));
+	string = g_strdup_printf ("<b>%s</b>", _("No results"));
+	label = gtk_label_new (string);
+	g_free (string);
+
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 	gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
 	obj->priv->results_label = label;
 
-	button = brasero_utils_make_button (_("Next results"),
+	button = brasero_utils_make_button (_("Next Results"),
 					    GTK_STOCK_GO_FORWARD,
 					    NULL,
 					    GTK_ICON_SIZE_BUTTON);
@@ -589,9 +577,9 @@ brasero_search_new ()
 		gtk_widget_set_sensitive (GTK_WIDGET (obj), FALSE);
 
 		/* we will retry in 10 seconds */
-		obj->priv->id = g_timeout_add (10000,
-					       (GSourceFunc) brasero_search_try_again,
-					       obj);
+		obj->priv->id = g_timeout_add_seconds (10,
+						       (GSourceFunc) brasero_search_try_again,
+						       obj);
 	}
 
 	return GTK_WIDGET (obj);
@@ -795,16 +783,19 @@ brasero_search_update_header (BraseroSearch *search)
 
 	if (search->priv->hits_num) {
 		gint last;
+		gchar *tmp;
 
 		last = search->priv->first_hit + search->priv->max_results;
 		last = last <= search->priv->hits_num ? last : search->priv->hits_num;
-		string = g_strdup_printf (_("<b>Results %i - %i (out of %i)</b>"),
-					  search->priv->first_hit + 1,
-					  last,
-					  search->priv->hits_num);
+		tmp = g_strdup_printf (_("Results %i - %i (out of %i)"),
+				       search->priv->first_hit + 1,
+				       last,
+				       search->priv->hits_num);
+		string = g_strdup_printf ("<b>%s</b>", tmp);
+		g_free (tmp);
 	}
 	else
-		string = g_strdup (_("<b>No results</b>"));
+		string = g_strdup_printf ("<b>%s</b>", _("No results"));
 
 	gtk_label_set_markup (GTK_LABEL (search->priv->results_label), string);
 	g_free (string);
@@ -1035,7 +1026,6 @@ brasero_search_beagle_finished_cb (BeagleQuery *query,
 static void
 brasero_search_beagle_error_dialog (BraseroSearch *search, GError *error)
 {
-	GtkWidget *dialog;
 	GtkWidget *toplevel;
 
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (search));
@@ -1044,20 +1034,10 @@ brasero_search_beagle_error_dialog (BraseroSearch *search, GError *error)
 		return;
 	}
 
-	dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-					 GTK_DIALOG_DESTROY_WITH_PARENT|
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE,
-					 _("Error querying beagle:"));
-
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Search error"));
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  error->message);
-
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
+	brasero_app_alert (BRASERO_APP (toplevel),
+			   _("Error querying beagle."),
+			   error->message,
+			   GTK_MESSAGE_ERROR);
 }
 
 static void
