@@ -7,19 +7,21 @@
 ****************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,25 +36,11 @@
 #include <glib/gi18n-lib.h>
 #include <glib-object.h>
 
-#include <gtk/gtktable.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkcomboboxentry.h>
-#include <gtk/gtkentry.h>
+#include <gtk/gtk.h>
+
+#include <beagle/beagle.h>
+
 #include <gconf/gconf-client.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcelllayout.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkdialog.h>
 
 #include "brasero-search-entry.h"
 #include "brasero-layout.h"
@@ -70,6 +58,8 @@ struct BraseroSearchEntryPrivate {
 	guint cxn;
 	gint search_id;
 
+	BraseroLayoutType ctx;
+
 	gchar *keywords;
 
 	GtkWidget *documents;
@@ -79,134 +69,6 @@ struct BraseroSearchEntryPrivate {
 
 };
 
-/* cut and pasted from nautilus */
-struct _MimeTypeGroup{
-	gchar *name;
-	gchar *mimetypes[30];
-};
-typedef struct _MimeTypeGroup MimeTypeGroup;
-const static MimeTypeGroup mime_type_groups [] = {
-	{ N_("Find all available music"),
-	  { "application/ogg",
-	    "audio/ac3",
-	    "audio/basic",
-	    "audio/midi",
-	    "audio/x-flac",
-	    "audio/mp4",
-	    "audio/mpeg",
-	    "audio/x-vorbis+ogg", /* beagle stores ogg-vorbis files under this mime type */
-	    "audio/x-mpeg",
-	    "audio/x-ms-asx",
-	    "audio/x-pn-realaudio",
-	    NULL
-	  }
-	},
-	{ N_("Find all available videos"),
-	  { "video/mp4",
-	    "video/3gpp",
-	    "video/mpeg",
-	    "video/quicktime",
-	    "video/vivo",
-	    "video/x-avi",
-	    "video/x-mng",
-	    "video/x-ms-asf",
-	    "video/x-ms-wmv",
-	    "video/x-msvideo",
-	    "video/x-nsv",
-	    "video/x-real-video",
-	    NULL
-	  }
-	},
-	{ N_("Find all available pictures"),
-	  { "application/vnd.oasis.opendocument.image",
-	    "application/x-krita",
-	    "image/bmp",
-	    "image/cgm",
-	    "image/gif",
-	    "image/jpeg",
-	    "image/jpeg2000",
-	    "image/png",
-	    "image/svg+xml",
-	    "image/tiff",
-	    "image/x-compressed-xcf",
-	    "image/x-pcx",
-	    "image/x-photo-cd",
-	    "image/x-psd",
-	    "image/x-tga",
-	    "image/x-xcf",
-	    "application/illustrator",
-	    "application/vnd.corel-draw",
-	    "application/vnd.stardivision.draw",
-	    "application/vnd.oasis.opendocument.graphics",
-	    "application/x-dia-diagram",
-	    "application/x-karbon",
-	    "application/x-killustrator",
-	    "application/x-kivio",
-	    "application/x-kontour",
-	    "application/x-wpg",
-	    NULL
-	  }
-	},
-	{ N_("Find all available documents"),
-	  { "application/rtf",
-	    "application/msword",
-	    "application/vnd.sun.xml.writer",
-	    "application/vnd.sun.xml.writer.global",
-	    "application/vnd.sun.xml.writer.template",
-	    "application/vnd.oasis.opendocument.text",
-	    "application/vnd.oasis.opendocument.text-template",
-	    "application/x-abiword",
-	    "application/x-applix-word",
-	    "application/x-mswrite",
-	    "application/docbook+xml",
-	    "application/x-kword",
-	    "application/x-kword-crypt",
-	    "application/x-lyx",
-	    NULL
-	  }
-	},
-	{ N_("Find all available spreadsheets"),
-	  { "application/vnd.lotus-1-2-3",
-	    "application/vnd.ms-excel",
-	    "application/vnd.stardivision.calc",
-	    "application/vnd.sun.xml.calc",
-	    "application/vnd.oasis.opendocument.spreadsheet",
-	    "application/x-applix-spreadsheet",
-	    "application/x-gnumeric",
-	    "application/x-kspread",
-	    "application/x-kspread-crypt",
-	    "application/x-quattropro",
-	    "application/x-sc",
-	    "application/x-siag",
-	    NULL
-	  }
-	},
-	{ N_("Find all available presentations"),
-	  { "application/vnd.ms-powerpoint",
-	    "application/vnd.sun.xml.impress",
-	    "application/vnd.oasis.opendocument.presentation",
-	    "application/x-magicpoint",
-	    "application/x-kpresenter",
-	    NULL
-	  }
-	},
-	{ N_("Find all available Pdf / Postscripts"),
-	  { "application/pdf",
-	    "application/postscript",
-	    "application/x-dvi",
-	    "image/x-eps",
-	    NULL
-	  }
-	},
-	{NULL},
-	{ N_("Text File"),
-	  { "text/plain",
-	    NULL
-	  }
-	},
-	{ NULL }
-};
-
 enum {
 	BRASERO_SEARCH_ENTRY_DISPLAY_COL,
 	BRASERO_SEARCH_ENTRY_BACKGRD_COL,
@@ -258,7 +120,7 @@ brasero_search_entry_get_type ()
 			(GInstanceInitFunc) brasero_search_entry_init,
 		};
 
-		type = g_type_register_static (GTK_TYPE_VBOX,
+		type = g_type_register_static (GTK_TYPE_TABLE,
 					       "BraseroSearchEntry",
 					       &our_info,
 					       0);
@@ -310,7 +172,7 @@ brasero_search_entry_separator_func (GtkTreeModel *model,
 static void
 brasero_search_entry_init (BraseroSearchEntry *obj)
 {
-	GtkWidget *box;
+	gchar *string;
 	GtkWidget *table;
 	GtkWidget *label;
 	GtkWidget *entry;
@@ -319,19 +181,33 @@ brasero_search_entry_init (BraseroSearchEntry *obj)
 	GtkCellRenderer *renderer;
 	GtkEntryCompletion *completion;
 
-	gtk_box_set_spacing (GTK_BOX (obj), 6);
+	gtk_table_set_row_spacings (GTK_TABLE (obj), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (obj), 6);
+	gtk_table_resize (GTK_TABLE (obj), 2, 3);
 	obj->priv = g_new0 (BraseroSearchEntryPrivate, 1);
 
-	box = gtk_hbox_new (FALSE, 6);
-	gtk_box_pack_start (GTK_BOX (obj), box, FALSE, FALSE, 0);
+	string = g_strdup_printf ("<b>%s</b>", _("Search:"));
+	label = gtk_label_new (string);
+	g_free (string);
 
-	label = gtk_label_new (_("<b>Search:\t</b>"));
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
+	gtk_table_attach (GTK_TABLE (obj),
+			  label,
+			  0, 1,
+			  0, 1,
+			  GTK_FILL,
+			  GTK_FILL,
+			  2, 2);
 
 	obj->priv->button = gtk_button_new_from_stock (GTK_STOCK_FIND);
-	/* gtk_widget_set_sensitive (obj->priv->button, FALSE); */
-	gtk_box_pack_end (GTK_BOX (box), obj->priv->button, FALSE, FALSE, 0);
+	gtk_table_attach (GTK_TABLE (obj),
+			  obj->priv->button,
+			  2, 3,
+			  0, 1,
+			  GTK_FILL,
+			  GTK_FILL,
+			  0, 0);
+
 	g_signal_connect (G_OBJECT (obj->priv->button),
 			  "clicked",
 			  G_CALLBACK (brasero_search_entry_button_clicked_cb),
@@ -373,32 +249,28 @@ brasero_search_entry_init (BraseroSearchEntry *obj)
 	gtk_entry_completion_set_text_column (completion, BRASERO_SEARCH_ENTRY_DISPLAY_COL);
 	gtk_entry_set_completion (GTK_ENTRY (entry), completion);
 
-	gtk_box_pack_start (GTK_BOX (box),
-			    obj->priv->combo,
-			    TRUE,
-			    TRUE,
-			    0);
+	gtk_table_attach (GTK_TABLE (obj),
+			  obj->priv->combo,
+			  1, 2,
+			  0, 1,
+			  GTK_FILL|GTK_EXPAND,
+			  GTK_FILL,
+			  0, 0);
 
-	/* table */
-	table = gtk_table_new (2, 3, FALSE);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+	/* Table with filtering options */
+	table = gtk_table_new (2, 2, FALSE);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 10);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 
-	gtk_box_pack_end (GTK_BOX (obj), table, FALSE, FALSE, 0);
+	gtk_table_attach (GTK_TABLE (obj),
+			  table,
+			  1, 2,
+			  1, 2,
+			  GTK_FILL,
+			  GTK_FILL,
+			  0, 0);
 
-	label = gtk_label_new (_("<b>only in\t</b>"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
-	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_table_attach (GTK_TABLE (table),
-			  label,
-			  0, 1,
-			  0, 1,
-			  0,
-			  0,
-			  0,
-			  0);
-
-	obj->priv->documents = gtk_check_button_new_with_mnemonic (_("_text documents"));
+	obj->priv->documents = gtk_check_button_new_with_mnemonic (_("In _text documents"));
 	gtk_button_set_alignment (GTK_BUTTON (obj->priv->documents), 0.0, 0.0);
 	gtk_table_attach (GTK_TABLE (table),
 			  obj->priv->documents,
@@ -413,7 +285,7 @@ brasero_search_entry_init (BraseroSearchEntry *obj)
 			  G_CALLBACK (brasero_search_entry_category_clicked_cb),
 			  obj);
 
-	obj->priv->pictures = gtk_check_button_new_with_mnemonic (_("_pictures"));
+	obj->priv->pictures = gtk_check_button_new_with_mnemonic (_("In _pictures"));
 	gtk_button_set_alignment (GTK_BUTTON (obj->priv->pictures), 0.0, 0.0);
 	gtk_table_attach (GTK_TABLE (table),
 			  obj->priv->pictures,
@@ -428,7 +300,7 @@ brasero_search_entry_init (BraseroSearchEntry *obj)
 			  G_CALLBACK (brasero_search_entry_category_clicked_cb),
 			  obj);
 
-	obj->priv->music = gtk_check_button_new_with_mnemonic (_("_music"));
+	obj->priv->music = gtk_check_button_new_with_mnemonic (_("In _music"));
 	gtk_button_set_alignment (GTK_BUTTON (obj->priv->music), 0.0, 0.0);
 	gtk_table_attach (GTK_TABLE (table),
 			  obj->priv->music,
@@ -443,7 +315,7 @@ brasero_search_entry_init (BraseroSearchEntry *obj)
 			  G_CALLBACK (brasero_search_entry_category_clicked_cb),
 			  obj);
 
-	obj->priv->video = gtk_check_button_new_with_mnemonic (_("_video"));
+	obj->priv->video = gtk_check_button_new_with_mnemonic (_("In _videos"));
 	gtk_button_set_alignment (GTK_BUTTON (obj->priv->video), 0.0, 0.0);
 	gtk_table_attach (GTK_TABLE (table),
 			  obj->priv->video,
@@ -681,9 +553,9 @@ brasero_search_entry_activated (BraseroSearchEntry *entry,
 
 	if (!search_now) {
 		/* gtk_widget_set_sensitive (entry->priv->button, TRUE); */
-		entry->priv->search_id = g_timeout_add (2000,
-							(GSourceFunc) brasero_search_entry_activated_real,
-							entry);
+		entry->priv->search_id = g_timeout_add_seconds (2,
+								(GSourceFunc) brasero_search_entry_activated_real,
+								entry);
 	}
 	else
 		brasero_search_entry_check_keywords (entry);
@@ -806,42 +678,87 @@ brasero_search_entry_add_current_keyword_to_history (BraseroSearchEntry *entry)
 	brasero_search_entry_set_history (entry);
 }
 
-static void
-_add_mime_types_to_query (BeagleQuery *query, const MimeTypeGroup *group)
-{
-	gchar **mime;
-
-	mime = (gchar **) group->mimetypes;
-	while (*mime) {
-		beagle_query_add_mime_type (query, *mime);
-		mime ++;
-	}
-}
-
 BeagleQuery *
 brasero_search_entry_get_query (BraseroSearchEntry *entry)
 {
 	BeagleQuery *query;
+	BeagleQueryPartHuman *text;
+	BeagleQueryPartOr *or_part = NULL;
 
+	/* Not sure about all this */
 	query = beagle_query_new ();
-	beagle_query_add_source (query, "Files");
+
+	if (strcmp (entry->priv->keywords, _("All files"))) {
+		BeagleQueryPartHuman *text;
+
+		text = beagle_query_part_human_new ();
+		beagle_query_part_human_set_string (text, entry->priv->keywords);
+		beagle_query_part_set_logic (BEAGLE_QUERY_PART (text),
+					     BEAGLE_QUERY_PART_LOGIC_REQUIRED);
+
+		beagle_query_add_part (query, BEAGLE_QUERY_PART (text));
+	}
+
+	text = beagle_query_part_human_new ();
+	beagle_query_part_human_set_string (text, "type:File");
+	beagle_query_add_part (query, BEAGLE_QUERY_PART (text));
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (entry->priv->documents))) {
-		_add_mime_types_to_query (query, mime_type_groups + 3);
+		BeagleQueryPartProperty *filetype;
+
+		if (!or_part)
+			or_part = beagle_query_part_or_new ();
+
+		filetype = beagle_query_part_property_new ();
+		beagle_query_part_property_set_property_type (filetype, BEAGLE_PROPERTY_TYPE_KEYWORD);
+		beagle_query_part_property_set_key (filetype, "beagle:FileType");
+		beagle_query_part_property_set_value (filetype, "document");
+		beagle_query_part_or_add_subpart (or_part, BEAGLE_QUERY_PART (filetype));
 	}
+
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (entry->priv->pictures))) {
-		_add_mime_types_to_query (query, mime_type_groups + 2);
+		BeagleQueryPartProperty *filetype;
+
+		if (!or_part)
+			or_part = beagle_query_part_or_new ();
+
+		filetype = beagle_query_part_property_new ();
+		beagle_query_part_property_set_property_type (filetype, BEAGLE_PROPERTY_TYPE_KEYWORD);
+		beagle_query_part_property_set_key (filetype, "beagle:FileType");
+		beagle_query_part_property_set_value (filetype, "image");
+		beagle_query_part_or_add_subpart (or_part, BEAGLE_QUERY_PART (filetype));
 	}
+
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (entry->priv->music))) {
-		_add_mime_types_to_query (query, mime_type_groups);
+		BeagleQueryPartProperty *filetype;
+
+		if (!or_part)
+			or_part = beagle_query_part_or_new ();
+
+		filetype = beagle_query_part_property_new ();
+		beagle_query_part_property_set_property_type (filetype, BEAGLE_PROPERTY_TYPE_KEYWORD);
+		beagle_query_part_property_set_key (filetype, "beagle:FileType");
+		beagle_query_part_property_set_value (filetype, "audio");
+		beagle_query_part_or_add_subpart (or_part, BEAGLE_QUERY_PART (filetype));
 	}
+
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (entry->priv->video))) {
-		_add_mime_types_to_query (query, mime_type_groups + 1);
+		BeagleQueryPartProperty *filetype;
+
+		if (!or_part)
+			or_part = beagle_query_part_or_new ();
+
+		filetype = beagle_query_part_property_new ();
+		beagle_query_part_property_set_property_type (filetype, BEAGLE_PROPERTY_TYPE_KEYWORD);
+		beagle_query_part_property_set_key (filetype, "beagle:FileType");
+		beagle_query_part_property_set_value (filetype, "video");
+		beagle_query_part_or_add_subpart (or_part, BEAGLE_QUERY_PART (filetype));
 	}
 
-	if (strcmp (entry->priv->keywords, _("All files")))
-		beagle_query_add_text (query, entry->priv->keywords);
+	if (!or_part)
+		return query;
 
+	beagle_query_add_part (query, BEAGLE_QUERY_PART (or_part));
 	return query;
 }
 
@@ -849,12 +766,22 @@ void
 brasero_search_entry_set_context (BraseroSearchEntry *self,
 				  BraseroLayoutType type)
 {
+	if (self->priv->ctx == type)
+		return;
+
+	self->priv->ctx = type;
 	if (type == BRASERO_LAYOUT_AUDIO) {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->video), FALSE);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->music), TRUE);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->pictures), FALSE);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->documents), FALSE);
 	}
+	else if (type == BRASERO_LAYOUT_VIDEO) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->video), TRUE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->music), FALSE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->pictures), FALSE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->documents), FALSE);
+	}
 }
 
 #endif /*BUILD_SEARCH*/
