@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,14 +32,7 @@
 #include <glib-object.h>
 #include <glib/gi18n.h>
 
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkspinbutton.h>
+#include <gtk/gtk.h>
 
 #include <gst/gst.h>
 
@@ -57,7 +52,13 @@ struct BraseroSongPropsPrivate {
 	GtkWidget *length;
 	GtkWidget *start;
 	GtkWidget *end;
+
 	GtkWidget *gap;
+	GtkWidget *gap_label;
+
+	guint title_set:1;
+	guint artist_set:1;
+	guint composer_set:1;
 };
 
 static GObjectClass *parent_class = NULL;
@@ -115,9 +116,31 @@ brasero_song_props_gap_changed_cb (GtkSpinButton *button,
 	g_free (length_str);
 }
 
+static void
+brasero_song_props_title_set (GtkEntry *entry,
+			      BraseroSongProps *self)
+{
+	self->priv->title_set = TRUE;
+}
+
+static void
+brasero_song_props_artist_set (GtkEntry *entry,
+			       BraseroSongProps *self)
+{
+	self->priv->artist_set = TRUE;
+}
+
+static void
+brasero_song_props_composer_set (GtkEntry *entry,
+				 BraseroSongProps *self)
+{
+	self->priv->composer_set = TRUE;
+}
+
 static void
 brasero_song_props_init (BraseroSongProps *obj)
 {
+	gchar *title_str;
 	GtkWidget *label;
 	GtkWidget *table;
 	GtkWidget *frame;
@@ -148,6 +171,10 @@ brasero_song_props_init (BraseroSongProps *obj)
 	label = gtk_label_new (_("Title:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	obj->priv->title = gtk_entry_new ();
+	g_signal_connect (obj->priv->title,
+			  "changed",
+			  G_CALLBACK (brasero_song_props_title_set),
+			  obj);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_table_attach_defaults (GTK_TABLE (table), obj->priv->title, 1, 2, 0, 1);
 	gtk_widget_set_tooltip_text (obj->priv->title,
@@ -156,14 +183,22 @@ brasero_song_props_init (BraseroSongProps *obj)
 	label = gtk_label_new (_("Artist:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	obj->priv->artist = gtk_entry_new ();
+	g_signal_connect (obj->priv->artist,
+			  "changed",
+			  G_CALLBACK (brasero_song_props_artist_set),
+			  obj);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_table_attach_defaults (GTK_TABLE (table), obj->priv->artist, 1, 2, 1, 2);
 	gtk_widget_set_tooltip_text (obj->priv->artist,
 				     _("This information will be written to the disc using CD-TEXT technology. It can be read and displayed by some audio CD players."));
 
-	label = gtk_label_new (_("Composer:\t"));
+	label = gtk_label_new (_("Composer:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	obj->priv->composer = gtk_entry_new ();
+	g_signal_connect (obj->priv->composer,
+			  "changed",
+			  G_CALLBACK (brasero_song_props_composer_set),
+			  obj);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_table_attach_defaults (GTK_TABLE (table), obj->priv->composer, 1, 2, 2, 3);
 	gtk_widget_set_tooltip_text (obj->priv->composer,
@@ -182,25 +217,29 @@ brasero_song_props_init (BraseroSongProps *obj)
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
 
-	frame = brasero_utils_pack_properties (_("<b>Options</b>"),
+	title_str = g_strdup_printf ("<b>%s</b>", "Options");
+	frame = brasero_utils_pack_properties (title_str,
 					       table,
 					       NULL);
+	g_free (title_str);
+
 	gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (obj)->vbox), frame, FALSE, FALSE, 0);
 
-	label = gtk_label_new (_("Song start:\t"));
+	label = gtk_label_new (_("Song start:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	obj->priv->start = brasero_time_button_new ();
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_table_attach (GTK_TABLE (table), obj->priv->start, 1, 2, 0, 1, 0, 0, 0, 0);
 
-	label = gtk_label_new (_("Song end:\t"));
+	label = gtk_label_new (_("Song end:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	obj->priv->end = brasero_time_button_new ();
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_table_attach (GTK_TABLE (table), obj->priv->end, 1, 2, 1, 2, 0, 0, 0, 0);
 
-	label = gtk_label_new (_("Pause length:\t"));
+	label = gtk_label_new (_("Pause length:"));
+	obj->priv->gap_label = label;
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	obj->priv->gap = gtk_spin_button_new_with_range (0.0, 100.0, 1.0);
 	alignment = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
@@ -210,7 +249,7 @@ brasero_song_props_init (BraseroSongProps *obj)
 	gtk_widget_set_tooltip_text (obj->priv->gap,
 				     _("Gives the length of the pause that should follow the track"));
 
-	label = gtk_label_new (_("Track length:\t"));
+	label = gtk_label_new (_("Track length:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
 	obj->priv->length = gtk_label_new (NULL);
@@ -239,7 +278,22 @@ brasero_song_props_init (BraseroSongProps *obj)
 				GTK_STOCK_APPLY, GTK_RESPONSE_ACCEPT,
 				NULL);
 
-	gtk_window_set_title (GTK_WINDOW (obj), _("Song information"));
+	gtk_window_set_title (GTK_WINDOW (obj), _("Song Information"));
+}
+
+static gchar *
+brasero_song_props_get_entry_value (GtkEntry *entry)
+{
+	const gchar *text;
+
+	text = gtk_entry_get_text (entry);
+	if (!text)
+		return NULL;
+
+	if (text [0] == '\0')
+		return NULL;
+
+	return g_strdup (text);
 }
 
 void
@@ -252,17 +306,35 @@ brasero_song_props_get_properties (BraseroSongProps *self,
 				   gint64 *end,
 				   gint64 *gap)
 {
-	if (artist)
-		*artist = g_strdup (gtk_entry_get_text (GTK_ENTRY (self->priv->artist)));
-	if (title)
-		*title = g_strdup (gtk_entry_get_text (GTK_ENTRY (self->priv->title)));
-	if (composer)
-		*composer = g_strdup (gtk_entry_get_text (GTK_ENTRY (self->priv->composer)));
+	if (artist) {
+		if (self->priv->artist_set)
+			*artist = brasero_song_props_get_entry_value (GTK_ENTRY (self->priv->artist));
+		else
+			*artist = NULL;
+	}
+
+	if (title) {
+		if (self->priv->title_set)
+			*title = brasero_song_props_get_entry_value (GTK_ENTRY (self->priv->title));
+		else
+			*title = NULL;
+	}
+
+	if (composer) {
+		if (self->priv->composer_set)
+			*composer = brasero_song_props_get_entry_value (GTK_ENTRY (self->priv->composer));
+		else
+			*composer = NULL;
+	}
+
 	if (isrc) {
 		const gchar *string;
 
-		string = gtk_entry_get_text (GTK_ENTRY (self->priv->isrc));
-		*isrc = (gint) g_strtod (string, NULL);
+		string = brasero_song_props_get_entry_value (GTK_ENTRY (self->priv->isrc));
+		if (string)
+			*isrc = (gint) g_strtod (string, NULL);
+		else
+			*isrc = 0;
 	}
 
 	if (start)
@@ -288,9 +360,24 @@ brasero_song_props_set_properties (BraseroSongProps *self,
 	gchar *string;
 	gdouble secs;
 
-	string = g_strdup_printf (_("<b>Song information for track %02i</b>"), track_num);
-	gtk_label_set_markup (GTK_LABEL (self->priv->label), string);
-	g_free (string);
+	if (track_num >= 0) {
+		gchar *tmp;
+
+		tmp = g_strdup_printf (_("Song information for track %02i"), track_num);
+		string = g_strdup_printf ("<b>%s</b>", tmp);
+		g_free (tmp);
+
+		gtk_label_set_markup (GTK_LABEL (self->priv->label), string);
+		g_free (string);
+	}
+	else {
+		brasero_time_button_set_show_frames (BRASERO_TIME_BUTTON (self->priv->start), FALSE);
+		brasero_time_button_set_show_frames (BRASERO_TIME_BUTTON (self->priv->end), FALSE);
+
+		gtk_widget_hide (self->priv->gap_label);
+		gtk_widget_hide (self->priv->label);
+		gtk_widget_hide (self->priv->gap);
+	}
 
 	if (artist)
 		gtk_entry_set_text (GTK_ENTRY (self->priv->artist), artist);
@@ -304,8 +391,14 @@ brasero_song_props_set_properties (BraseroSongProps *self,
 		g_free (string);
 	}
 
-	secs = gap / GST_SECOND;
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (self->priv->gap), secs);
+	if (gap > 0) {
+		secs = gap / GST_SECOND;
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (self->priv->gap), secs);
+	}
+	else {
+		gtk_widget_hide (self->priv->gap);
+		gtk_widget_hide (self->priv->gap_label);
+	}
 
 	brasero_time_button_set_max (BRASERO_TIME_BUTTON (self->priv->start), end - 1);
 	brasero_time_button_set_value (BRASERO_TIME_BUTTON (self->priv->start), start);
@@ -320,6 +413,10 @@ brasero_song_props_set_properties (BraseroSongProps *self,
 					   self);
 
 	brasero_song_props_update_length (self);
+
+	self->priv->title_set = FALSE;
+	self->priv->artist_set = FALSE;
+	self->priv->composer_set = FALSE;
 }
 
 static void
