@@ -3,12 +3,10 @@
  * brasero
  * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,23 +27,7 @@
 #include <glib.h>
 #include <glib/gi18n-lib.h>
 
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtktreeviewcolumn.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkradiobutton.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtknotebook.h>
+#include <gtk/gtk.h>
 
 #include "brasero-split-dialog.h"
 #include "brasero-player.h"
@@ -174,12 +156,11 @@ brasero_split_dialog_size_error (BraseroSplitDialog *self)
 					  GTK_BUTTONS_NONE,
 					  _("Do you really want to split the track?"));
 
-	gtk_window_set_title (GTK_WINDOW (message), _("Size Error"));
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
 						  _("If you split the track, the size of the new track will be shorter than 6 seconds and will be padded."));
 
 	gtk_dialog_add_button (GTK_DIALOG (message),
-			       _("_Don't split"),
+			       GTK_STOCK_CANCEL,
 			       GTK_RESPONSE_CANCEL);
 	gtk_dialog_add_button (GTK_DIALOG (message),
 			       _("_Split"),
@@ -200,7 +181,9 @@ brasero_split_dialog_cut (BraseroSplitDialog *self,
 			  gboolean warn)
 {
 	BraseroSplitDialogPrivate *priv;
+	BraseroAudioSlice slice = {0,0};
 	GtkTreeModel *model;
+	GtkTreeIter child;
 	GtkTreeIter iter;
 	gchar *length_str;
 	gchar *start_str;
@@ -274,8 +257,8 @@ brasero_split_dialog_cut (BraseroSplitDialog *self,
 		return TRUE;
 	}
 
+	/* Try to find an already created slice encompassing the position */
 	do {
-		GtkTreeIter child;
 		gint64 start;
 		gint64 end;
 
@@ -299,8 +282,51 @@ brasero_split_dialog_cut (BraseroSplitDialog *self,
 		&& !brasero_split_dialog_size_error (self))
 			return FALSE;
 
+		/* Found one */
+		slice.start = start;
+		slice.end = end;
+		break;
+
+	} while (gtk_tree_model_iter_next (model, &iter));
+
+	/* see if we found a slice, if not create a new one starting at pos
+	 * until the end of the song */
+
+	if (slice.start == 0 && slice.end == 0) {
+		slice.start = pos;
+
+		/* check if we need to stop this slice at the end of the song
+		 * or at the start of the next slice. */
+		if (gtk_tree_model_get_iter_first (model, &iter)) {
+			do {
+				gint64 start;
+				gint64 end;
+
+				gtk_tree_model_get (model, &iter,
+						    START_COL, &start,
+						    END_COL, &end,
+						    -1);
+
+				if (pos >= start)
+					continue;
+
+				/* Found one */
+				slice.end = start - 1;
+			} while (gtk_tree_model_iter_next (model, &iter));
+		}
+
+		if (!slice.end)
+			slice.end = priv->end;
+
+		/* check the size of the new slice */
+		if (warn
+		&& (slice.end - slice.start) < BRASERO_MIN_AUDIO_TRACK_LENGTH
+		&& !brasero_split_dialog_size_error (self))
+			return FALSE;
+	}
+	else {
 		/* we are in the middle of an existing slice */
-		length_str = brasero_utils_get_time_string (pos - start, TRUE, FALSE);
+		length_str = brasero_utils_get_time_string (pos - slice.start, TRUE, FALSE);
 		end_str = brasero_utils_get_time_string (pos, TRUE, FALSE);
 
 		gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model),
@@ -308,36 +334,37 @@ brasero_split_dialog_cut (BraseroSplitDialog *self,
 								&iter);
 
 		gtk_list_store_set (priv->model, &child,
-				    END_COL, (gint64)  pos,
-				    LENGTH_COL, (gint64) (pos - start),
+				    END_COL, (gint64) pos,
+				    LENGTH_COL, (gint64) (pos - slice.start),
 				    END_STR_COL, end_str,
 				    LENGTH_STR_COL, length_str,
 				    -1);
 		g_free (length_str);
 		g_free (end_str);
 
-		gtk_list_store_append (priv->model, &child);
-
+		/* move the position by one */
 		pos ++;
-		length_str = brasero_utils_get_time_string (end - pos, TRUE, FALSE);
-		start_str = brasero_utils_get_time_string (pos, TRUE, FALSE);
-		end_str = brasero_utils_get_time_string (end, TRUE, FALSE);
+	}
 
-		gtk_list_store_set (priv->model, &child,
-				    START_COL, pos,
-				    END_COL, end,
-				    LENGTH_COL, (gint64) (end - pos),
-				    START_STR_COL, start_str,
-				    END_STR_COL, end_str,
-				    LENGTH_STR_COL, length_str,
-				    -1);
+	/* create a new one */
+	gtk_list_store_append (priv->model, &child);
 
-		g_free (length_str);
-		g_free (start_str);
-		g_free (end_str);
-		break;
+	length_str = brasero_utils_get_time_string (slice.end - pos, TRUE, FALSE);
+	start_str = brasero_utils_get_time_string (pos, TRUE, FALSE);
+	end_str = brasero_utils_get_time_string (slice.end, TRUE, FALSE);
 
-	} while (gtk_tree_model_iter_next (model, &iter));
+	gtk_list_store_set (priv->model, &child,
+			    START_COL, pos,
+			    END_COL, slice.end,
+			    LENGTH_COL, (gint64) (slice.end - pos),
+			    START_STR_COL, start_str,
+			    END_STR_COL, end_str,
+			    LENGTH_STR_COL, length_str,
+			    -1);
+
+	g_free (length_str);
+	g_free (start_str);
+	g_free (end_str);
 
 	return TRUE;
 }
@@ -563,22 +590,10 @@ brasero_split_dialog_remove_range (BraseroSplitDialog *self,
 static void
 brasero_split_dialog_no_silence_message (BraseroSplitDialog *self)
 {
-	GtkWidget *message;
-
-	/* no silences found */
-	message = gtk_message_dialog_new (GTK_WINDOW (self),
-					  GTK_DIALOG_DESTROY_WITH_PARENT|
-					  GTK_DIALOG_MODAL,
-					  GTK_MESSAGE_WARNING,
-					  GTK_BUTTONS_CLOSE,
-					  _("The track wasn't split:"));
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						  _("no silence could be retrieved."));
-
-	gtk_window_set_title (GTK_WINDOW (message), _("No Silence"));
-	gtk_dialog_run (GTK_DIALOG (message));
-	gtk_widget_destroy (message);
+	brasero_utils_message_dialog (GTK_WIDGET (self),
+				      _("The track wasn't split."),
+				      _("No silence could be detected"),
+				      GTK_MESSAGE_WARNING);
 }
 
 static void
@@ -599,25 +614,14 @@ brasero_split_dialog_metadata_finished_cb (BraseroMetadata *metadata,
 	priv->metadata = NULL;
 
 	if (error) {
-		GtkWidget *message;
-
-		/* error while retrieve silences */
-		message = gtk_message_dialog_new (GTK_WINDOW (self),
-						  GTK_DIALOG_DESTROY_WITH_PARENT|
-						  GTK_DIALOG_MODAL,
-						  GTK_MESSAGE_ERROR,
-						  GTK_BUTTONS_CLOSE,
-						  _("An error occured while retrieving silences:"));
-		gtk_window_set_title (GTK_WINDOW (message), _("Error"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-							  error->message);
-
-		gtk_dialog_run (GTK_DIALOG (message));
-		gtk_widget_destroy (message);
+		brasero_utils_message_dialog (GTK_WIDGET (self),
+					      _("An error occured while detecting silences."),
+					      error->message,
+					      GTK_MESSAGE_ERROR);
 		return;
 	}
 
-	brasero_metadata_set_info (metadata, &info);
+	brasero_metadata_get_result (metadata, &info, NULL);
 	if (!info.silences) {
 		brasero_split_dialog_no_silence_message (self);
 		return;
@@ -662,7 +666,10 @@ brasero_split_dialog_metadata_finished_cb (BraseroMetadata *metadata,
 }
 
 static gboolean
-brasero_split_dialog_clear_confirm_dialog (BraseroSplitDialog *self)
+brasero_split_dialog_clear_confirm_dialog (BraseroSplitDialog *self,
+					   const gchar *primary,
+					   const gchar *cancel_button,
+					   const gchar *ok_button)
 {
 	BraseroSplitDialogPrivate *priv;
 	GtkResponseType answer;
@@ -680,17 +687,16 @@ brasero_split_dialog_clear_confirm_dialog (BraseroSplitDialog *self)
 					  GTK_DIALOG_MODAL,
 					  GTK_MESSAGE_QUESTION,
 					  GTK_BUTTONS_NONE,
-					  _("Do you really want to carry on with automatic splitting?"));
+					  primary);
 
-	gtk_window_set_title (GTK_WINDOW (message), _("Automatic Split"));
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
 						  _("This will remove all previous results."));
 
 	gtk_dialog_add_button (GTK_DIALOG (message),
-			       _("_Don't split"),
+			       cancel_button,
 			       GTK_RESPONSE_CANCEL);
 	gtk_dialog_add_button (GTK_DIALOG (message),
-			       _("_Split"),
+			       ok_button,
 			       GTK_RESPONSE_YES);
 
 	answer = gtk_dialog_run (GTK_DIALOG (message));
@@ -723,7 +729,10 @@ brasero_split_dialog_cut_clicked_cb (GtkButton *button,
 	}
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
-	if (!brasero_split_dialog_clear_confirm_dialog (self))
+	if (!brasero_split_dialog_clear_confirm_dialog (self,
+							_("Do you really want to carry on with automatic splitting?"),
+							_("_Don't split"),
+							_("_Split")))
 		return;
 
 	if (page == 1) {
@@ -937,7 +946,10 @@ brasero_split_dialog_reset_clicked_cb (GtkButton *button,
 	GtkTreeModel *model;
 
 	priv = BRASERO_SPLIT_DIALOG_PRIVATE (self);
-	if (!brasero_split_dialog_clear_confirm_dialog (self))
+	if (!brasero_split_dialog_clear_confirm_dialog (self,
+							_("Do you really want to empty the slices preview?"),
+							GTK_STOCK_CANCEL,
+							_("Re_move All")))
 		return;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
@@ -1007,6 +1019,7 @@ brasero_split_dialog_row_deleted_cb (GtkTreeModel *model,
 static void
 brasero_split_dialog_init (BraseroSplitDialog *object)
 {
+	gchar *title;
 	GtkWidget *vbox;
 	GtkWidget *hbox;
 	GtkWidget *vbox2;
@@ -1122,14 +1135,16 @@ brasero_split_dialog_init (BraseroSplitDialog *object)
 	gtk_widget_show (priv->silence_label);
 	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), priv->silence_label, NULL);
 
+	title = g_strdup_printf ("<b>%s</b>", _("Slicing Method"));
 	gtk_box_pack_start (GTK_BOX (vbox),
-			    brasero_utils_pack_properties (_("<b>Slicing Method</b>"),
+			    brasero_utils_pack_properties (title,
 							   priv->notebook,
 							   hbox,
 							   NULL),
 			    FALSE,
 			    FALSE,
 			    0);
+	g_free (title);
 
 	/* slices preview */
 	hbox = gtk_hbox_new (FALSE, 6);
@@ -1265,13 +1280,15 @@ brasero_split_dialog_init (BraseroSplitDialog *object)
 	gtk_box_pack_start (GTK_BOX (vbox2), label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox2), hbox, TRUE, TRUE, 0);
 
+	title = g_strdup_printf ("<b>%s</b>", _("Slices Preview"));
 	gtk_box_pack_start (GTK_BOX (vbox),
-			    brasero_utils_pack_properties (_("<b>Slices Preview</b>"),
+			    brasero_utils_pack_properties (title,
 							   vbox2,
 							   NULL),
 			    TRUE,
 			    TRUE,
 			    0);
+	g_free (title);
 
 	gtk_combo_box_set_active (GTK_COMBO_BOX (priv->combo), 0);
 }
