@@ -223,6 +223,13 @@ brasero_src_image_set_track (BraseroSrcImage *dialog,
 	g_free (path);
 
 	if (string) {
+		/* This is hackish and meant to avoid ellipsization to make the
+		 * label to small. */
+		if (strlen (string) > strlen (_("Click here to select an _image")) + 5)
+			gtk_label_set_ellipsize (GTK_LABEL (priv->label), PANGO_ELLIPSIZE_START);
+		else
+			gtk_label_set_ellipsize (GTK_LABEL (priv->label), PANGO_ELLIPSIZE_NONE);
+
 		gtk_label_set_text (GTK_LABEL (priv->label), string);
 		g_free (string);
 	}
@@ -686,7 +693,7 @@ brasero_src_image_init (BraseroSrcImage *object)
 	label = gtk_label_new_with_mnemonic (string);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_START);
+	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_NONE);
 	gtk_widget_show (label);
 	g_free (string);
 
