@@ -35,22 +35,11 @@
 #include <stdio.h>
 
 #include <glib.h>
-#include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
+#include <glib/gi18n-lib.h>
 
 #include <gtk/gtk.h>
 
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkmessagedialog.h>
-
 #include "brasero-sum-dialog.h"
 #include "brasero-tool-dialog.h"
 #include "burn-drive.h"
@@ -92,7 +81,6 @@ brasero_sum_dialog_stop (BraseroSumDialog *self)
 
 static gboolean
 brasero_sum_dialog_message (BraseroSumDialog *self,
-			    const gchar *title,
 			    const gchar *primary_message,
 			    const gchar *secondary_message,
 			    GtkMessageType type)
@@ -115,10 +103,9 @@ brasero_sum_dialog_message (BraseroSumDialog *self,
 					  GTK_BUTTONS_NONE,
 					  "%s", primary_message);
 
-	gtk_window_set_title (GTK_WINDOW (message), title);
-
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						  "%s", secondary_message);
+						  "%s.",
+						  secondary_message);
 
 	button = brasero_utils_make_button (_("Check _Again"),
 					    GTK_STOCK_FIND,
@@ -151,9 +138,8 @@ brasero_sum_dialog_message_error (BraseroSumDialog *self,
 					NULL);
 
 	return brasero_sum_dialog_message (self,
-					   _("File Integrity Check Error"),
-					   _("The file integrity check cannot be performed:"),
-					   error ? error->message:_("unknown error"),
+					   _("The file integrity check could not be performed."),
+					   error ? error->message:_("An unknown error occured"),
 					   GTK_MESSAGE_ERROR);
 }
 
@@ -165,9 +151,8 @@ brasero_sum_dialog_success (BraseroSumDialog *self)
 					NULL);
 
 	return brasero_sum_dialog_message (self,
-					   _("File Integrity Check Success"),
-					   _("The file integrity was performed successfully:"),
-					   _("there seems to be no corrupted file on the disc."),
+					   _("The file integrity was performed successfully."),
+					   _("There seems to be no corrupted file on the disc"),
 					   GTK_MESSAGE_INFO);
 }
 
@@ -199,7 +184,6 @@ brasero_sum_dialog_corruption_warning (BraseroSumDialog *self,
 						      "%s", string);
 	g_free (string);
 
-	gtk_window_set_title (GTK_WINDOW (message),  _("File Integrity Check Error"));
 	gtk_window_set_resizable (GTK_WINDOW (message), TRUE);
 	gtk_widget_set_size_request (GTK_WIDGET (message), 440, 300);
 
@@ -238,7 +222,7 @@ brasero_sum_dialog_corruption_warning (BraseroSumDialog *self,
 	gtk_tree_view_column_add_attribute (column, renderer,
 					    "text", BRASERO_SUM_DIALOG_PATH);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
-	gtk_tree_view_column_set_title (column, _("Corrupted files"));
+	gtk_tree_view_column_set_title (column, _("Corrupted Files"));
 
 	scroll = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll),
@@ -308,11 +292,16 @@ brasero_sum_dialog_download (BraseroSumDialog *self,
 				   g_get_tmp_dir ());
 	fd = g_mkstemp (tmppath);
 	if (fd < 0) {
+		int errnum = errno;
+
 		g_free (tmppath);
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("a temporary file couldn't be created"));
+			     "%s (%s)",
+			     _("A file could not be created at the location specified for temporary files"),
+			     g_strerror (errnum));
+
 		return BRASERO_BURN_ERR;
 	}
 	close (fd);
@@ -367,7 +356,8 @@ brasero_sum_dialog_get_file_checksum (BraseroSumDialog *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("URI is not valid"));
+			     _("\"%s\" is not a valid URI"),
+			     file_path);
 		return BRASERO_BURN_ERR;
 	}
 
@@ -478,9 +468,8 @@ brasero_sum_dialog_check_md5_file (BraseroSumDialog *self,
     	uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (self->priv->md5_chooser));
 	if (!uri) {
 		retval = brasero_sum_dialog_message (self,
-						     _("File Integrity Check Error"),
-						     _("The file integrity check cannot be performed:"),
-						     error ? error->message:_("no md5 file was given."),
+						     _("The file integrity check could not be performed."),
+						     error ? error->message:_("No md5 file was given."),
 						     GTK_MESSAGE_ERROR);
 		return retval;
 	}
@@ -588,7 +577,7 @@ brasero_sum_dialog_set_track_checksum_type (BraseroSumDialog *self,
 	g_set_error (error,
 		     BRASERO_ERROR,
 		     BRASERO_ERROR_GENERAL,
-		     _("no checksum file could be found on the disc."));
+		     _("No checksum file could be found on the disc"));
 
 	return BRASERO_CHECKSUM_NONE;
 }
@@ -615,15 +604,17 @@ brasero_sum_dialog_check_disc_sum (BraseroSumDialog *self,
 								    &error);
 
 	if (checksum_type == BRASERO_CHECKSUM_NONE) {
-		brasero_sum_dialog_message_error (self, error);
+		retval = brasero_sum_dialog_message_error (self, error);
 		brasero_track_unref (track);
-		g_error_free (error);
-		return FALSE;
+
+		if (error)
+			g_error_free (error);
+
+		return retval;
 	}
 
 	/* no eject at the end (it's default) */
 	brasero_burn_session_remove_flag (self->priv->session, BRASERO_BURN_FLAG_EJECT);
-
 	brasero_burn_session_add_track (self->priv->session, track);
 
 	/* It's good practice to unref the track afterwards as we don't need it
@@ -784,7 +775,7 @@ brasero_sum_dialog_new ()
 	BraseroSumDialog *obj;
 	
 	obj = BRASERO_SUM_DIALOG (g_object_new (BRASERO_TYPE_SUM_DIALOG, NULL));
-	gtk_window_set_title (GTK_WINDOW (obj), "Disc Checking");
+	gtk_window_set_title (GTK_WINDOW (obj), _("Disc Checking"));
 	
 	return GTK_WIDGET (obj);
 }
