@@ -31,16 +31,7 @@
 #include <glib.h>
 #include <glib/gi18n-lib.h>
 
-#include <gtk/gtklabel.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkprogressbar.h>
-#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtk.h>
 
 #include "brasero-utils.h"
 #include "brasero-progress.h"
@@ -74,26 +65,32 @@ static void
 brasero_tool_dialog_media_error (BraseroToolDialog *self)
 {
 	brasero_utils_message_dialog (GTK_WIDGET (self),
-				     _("The operation cannot be performed:"),
-				     _("the inserted media is busy."),
+				     _("The operation cannot be performed."),
+				     _("The disc is not supported"),
 				     GTK_MESSAGE_ERROR);
 }
 
 static void
 brasero_tool_dialog_media_busy (BraseroToolDialog *self)
 {
+	gchar *string;
+
+	string = g_strdup_printf ("%s. %s",
+				  _("The drive is busy"),
+				  _("Make sure another application is not using it"));
 	brasero_utils_message_dialog (GTK_WIDGET (self),
-				     _("The operation cannot be performed:"),
-				     _("the inserted media is not supported."),
+				     _("The operation cannot be performed."),
+				     string,
 				     GTK_MESSAGE_ERROR);
+	g_free (string);
 }
 
 static void
 brasero_tool_dialog_no_media (BraseroToolDialog *self)
 {
 	brasero_utils_message_dialog (GTK_WIDGET (self),
-				     _("The operation cannot be performed:"),
-				     _("the drive is empty."),
+				     _("The operation cannot be performed."),
+				     _("The drive is empty"),
 				     GTK_MESSAGE_ERROR);
 }
 
@@ -370,15 +367,13 @@ brasero_tool_dialog_cancel_dialog (GtkWidget *toplevel)
 					  GTK_BUTTONS_NONE,
 					  _("Do you really want to quit?"));
 
-	gtk_window_set_title (GTK_WINDOW (message), _("Confirm"));
-
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
 						  _("Interrupting the process may make disc unusable."));
 	gtk_dialog_add_buttons (GTK_DIALOG (message),
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				NULL);
 
-	button = brasero_utils_make_button (_("Continue"),
+	button = brasero_utils_make_button (_("_Continue"),
 					    GTK_STOCK_OK,
 					    NULL,
 					    GTK_ICON_SIZE_BUTTON);
@@ -482,8 +477,6 @@ brasero_tool_dialog_init (BraseroToolDialog *obj)
 
 	obj->priv->selector = brasero_medium_selection_new ();
 	gtk_widget_show (GTK_WIDGET (obj->priv->selector));
-	gtk_widget_set_tooltip_text (obj->priv->selector,
-				     _("Choose a media"));
 
 	title_str = g_strdup_printf ("<b>%s</b>", _("Select a disc"));
 	gtk_box_pack_start (GTK_BOX (obj->priv->upper_box),
@@ -506,7 +499,7 @@ brasero_tool_dialog_init (BraseroToolDialog *obj)
 
 	/* lower part */
 	obj->priv->lower_box = gtk_vbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (obj->priv->lower_box), 6);
+	gtk_container_set_border_width (GTK_CONTAINER (obj->priv->lower_box), 12);
 	gtk_widget_set_sensitive (obj->priv->lower_box, FALSE);
 	gtk_widget_show (obj->priv->lower_box);
 
@@ -540,7 +533,7 @@ brasero_tool_dialog_init (BraseroToolDialog *obj)
 			    obj->priv->lower_box,
 			    FALSE,
 			    FALSE,
-			    6);
+			    0);
 
 	/* buttons */
 	obj->priv->cancel = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
