@@ -1,13 +1,11 @@
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2007 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,6 +36,7 @@
 #include "burn-plugin.h"
 #include "burn-plugin-private.h"
 #include "burn-caps.h"
+#include "burn-media.h"
 
 #define BRASERO_PLUGIN_PRIORITY_KEY			"/apps/brasero/config/priority"
 
@@ -389,11 +388,11 @@ brasero_plugin_get_gconf_priority_key (BraseroPlugin *self)
 	caps = brasero_burn_caps_get_default ();
 
 	/* make sure the name used has characters usable by GConf */
-	if (brasero_burn_caps_plugin_can_burn (caps, self) == BRASERO_BURN_OK)
+	if (brasero_plugin_can_burn (self) == BRASERO_BURN_OK)
 		type = 0;
-	else if (brasero_burn_caps_plugin_can_image (caps, self) == BRASERO_BURN_OK)
+	else if (brasero_plugin_can_image (self) == BRASERO_BURN_OK)
 		type = 1;
-	else if (brasero_burn_caps_plugin_can_convert (caps, self) == BRASERO_BURN_OK)
+	else if (brasero_plugin_can_convert (self) == BRASERO_BURN_OK)
 		type = 2;
 
 	gconf_name = g_strdup (priv->name);
@@ -429,19 +428,13 @@ brasero_plugin_define (BraseroPlugin *self,
 }
 
 void
-brasero_plugin_register_group (BraseroPlugin *self,
-			       const gchar *name)
+brasero_plugin_set_group (BraseroPlugin *self,
+			  gint group_id)
 {
-	guint id;
-	BraseroBurnCaps *caps;
 	BraseroPluginPrivate *priv;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
-
-	caps = brasero_burn_caps_get_default ();
-	id = brasero_burn_caps_register_plugin_group (caps, name);
-	priv->group = id;
-	g_object_unref (caps);
+	priv->group = group_id;
 }
 
 guint
@@ -512,12 +505,22 @@ brasero_plugin_set_flags (BraseroPlugin *self,
 			  BraseroBurnFlag compulsory)
 {
 	BraseroPluginPrivate *priv;
+	GSList *list;
+	GSList *iter;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
-	priv->flags = brasero_plugin_set_flags_real (priv->flags,
-						     media,
-						     supported,
-						     compulsory);
+
+	list = brasero_media_get_all_list (media);
+	for (iter = list; iter; iter = iter->next) {
+		BraseroMedia medium;
+
+		medium = GPOINTER_TO_INT (iter->data);
+		priv->flags = brasero_plugin_set_flags_real (priv->flags,
+							     medium,
+							     supported,
+							     compulsory);
+	}
+	g_slist_free (list);
 }
 
 static gboolean
@@ -537,6 +540,7 @@ brasero_plugin_get_all_flags (GSList *flags_list,
 
 	flags = brasero_plugin_get_flags (flags_list, media);
 	if (!flags) {
+
 		if (supported_retval)
 			*supported_retval = BRASERO_BURN_FLAG_NONE;
 		if (compulsory_retval)
@@ -605,6 +609,7 @@ brasero_plugin_check_image_flags (BraseroPlugin *self,
 	BraseroPluginPrivate *priv;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
+
 	current &= BRASERO_PLUGIN_IMAGE_FLAG_MASK;
 
 	/* If there is no flag that's no use checking anything. If there is no
@@ -622,6 +627,21 @@ brasero_plugin_check_image_flags (BraseroPlugin *self,
 					     NULL);
 }
 
+gboolean
+brasero_plugin_check_media_restrictions (BraseroPlugin *self,
+					 BraseroMedia media)
+{
+	BraseroPluginPrivate *priv;
+
+	priv = BRASERO_PLUGIN_PRIVATE (self);
+
+	/* no restrictions */
+	if (!priv->flags)
+		return TRUE;
+
+	return (brasero_plugin_get_flags (priv->flags, media) != NULL);
+}
+
 gboolean
 brasero_plugin_get_record_flags (BraseroPlugin *self,
 				 BraseroMedia media,
@@ -1058,7 +1078,12 @@ brasero_plugin_finalize (GObject *object)
 		priv->notify_priority = 0;
 		g_object_unref (client);
 	}
-		
+
+	if (priv->error) {
+		g_free (priv->error);
+		priv->error = NULL;
+	}
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
