@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -51,8 +53,6 @@ G_DEFINE_TYPE (BraseroBurnSession, brasero_burn_session, G_TYPE_OBJECT);
 struct _BraseroSessionSetting {
 	BraseroDrive *burner;
 
-	guint num_copies;
-
 	/**
 	 * Used when outputting an image instead of burning
 	 */
@@ -76,8 +76,6 @@ struct _BraseroBurnSessionPrivate {
 	FILE *session;
 	gchar *session_path;
 
-	GSList *wrong_checksums;
-
 	GSList *tmpfiles;
 
 	BraseroSessionSetting settings [1];
@@ -245,6 +243,21 @@ brasero_burn_session_free_tracks (BraseroBurnSession *self)
 		       0);
 }
 
+void
+brasero_burn_session_clear_current_track (BraseroBurnSession *self)
+{
+	BraseroBurnSessionPrivate *priv;
+
+	g_return_if_fail (BRASERO_IS_BURN_SESSION (self));
+
+	priv = BRASERO_BURN_SESSION_PRIVATE (self);
+
+	brasero_burn_session_stop_src_drive_monitoring (self);
+	g_slist_foreach (priv->tracks, (GFunc) brasero_track_unref, NULL);
+	g_slist_free (priv->tracks);
+	priv->tracks = NULL;
+}
+
 BraseroBurnResult
 brasero_burn_session_add_track (BraseroBurnSession *self,
 				BraseroTrack *new_track)
@@ -277,7 +290,8 @@ brasero_burn_session_add_track (BraseroBurnSession *self,
 	brasero_burn_session_stop_src_drive_monitoring (self);
 
 	/* if there is already a track, then we replace it on condition that it
-	 * has the same type */
+	 * has the same type and it's not AUDIO (only one allowed to have many)
+	 */
 	if (brasero_track_get_type (new_track, NULL) != BRASERO_TRACK_TYPE_AUDIO
 	||  brasero_burn_session_get_input_type (self, NULL) != BRASERO_TRACK_TYPE_AUDIO) {
 		g_slist_foreach (priv->tracks, (GFunc) brasero_track_unref, NULL);
@@ -370,7 +384,6 @@ brasero_burn_session_dest_media_removed (BraseroDrive *drive,
 					 BraseroMedium *medium,
 					 BraseroBurnSession *self)
 {
-	
 	g_signal_emit (self,
 		       brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL],
 		       0);
@@ -473,167 +486,6 @@ brasero_burn_session_get_rate (BraseroBurnSession *self)
 		return MIN (max_rate, priv->settings->rate);
 }
 
-void
-brasero_burn_session_set_num_copies (BraseroBurnSession *self,
-				     guint copies)
-{
-	BraseroBurnSessionPrivate *priv;
-
-	g_return_if_fail (BRASERO_IS_BURN_SESSION (self));
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
-	if (!BRASERO_BURN_SESSION_WRITE_TO_DISC (priv))
-		return;
-
-	priv->settings->num_copies = copies;
-}
-
-guint
-brasero_burn_session_get_num_copies (BraseroBurnSession *self)
-{
-	BraseroBurnSessionPrivate *priv;
-
-	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), 0);
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
-	if (!BRASERO_BURN_SESSION_WRITE_TO_DISC (priv))
-		return 1;
-
-	return priv->settings->num_copies;
-}
-
-static gchar *
-brasero_burn_session_get_file_complement (BraseroBurnSession *self,
-					  BraseroImageFormat format,
-					  const gchar *path)
-{
-	gchar *retval = NULL;
-	BraseroBurnSessionPrivate *priv;
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
-	if (format == BRASERO_IMAGE_FORMAT_CLONE) {
-		if (g_str_has_suffix (path, ".toc"))
-			retval = g_strdup_printf ("%.*sraw",
-						  strlen (path) - 3,
-						  path);
-		else
-			retval = g_strdup_printf ("%s.raw", path);
-	}
-	else if (format == BRASERO_IMAGE_FORMAT_CUE) {
-		if (g_str_has_suffix (path, ".cue"))
-			retval = g_strdup_printf ("%.*sbin",
-						  strlen (path) - 3,
-						  path);
-		else
-			retval = g_strdup_printf ("%s.bin", path);
-	}
-	else if (format == BRASERO_IMAGE_FORMAT_CDRDAO) {
-		if (g_str_has_suffix (path, ".toc"))
-			retval = g_strdup_printf ("%.*sbin",
-						  strlen (path) - 3,
-						  path);
-		else
-			retval = g_strdup_printf ("%s.bin", path);
-	}
-	else
-		retval = NULL;
-
-	return retval;
-}
-
-static BraseroBurnResult
-brasero_burn_session_file_test (BraseroBurnSession *self,
-				const gchar *path,
-				GError **error)
-{
-	BraseroBurnSessionPrivate *priv;
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
-	if (!path) {
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("no path"));
-		return BRASERO_BURN_ERR;
-	}
-
-	if (!g_file_test (path, G_FILE_TEST_EXISTS))
-		return BRASERO_BURN_OK;
-	
-	if (priv->settings->flags & BRASERO_BURN_FLAG_DONT_OVERWRITE) {
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("%s already exists"),
-			     path);
-		return BRASERO_BURN_ERR;
-	}
-
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_burn_session_set_image_output_retval (BraseroBurnSession *self,
-					      BraseroImageFormat format,
-					      gchar **image,
-					      gchar **toc,
-					      gchar *output,
-					      gchar *complement)
-{
-	BraseroBurnSessionPrivate *priv;
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
-	switch (format) {
-	case BRASERO_IMAGE_FORMAT_BIN:
-	case BRASERO_IMAGE_FORMAT_NONE:
-		if (image)
-			*image = output;
-		else
-			g_free (output);
-
-		if (toc)
-			*toc = NULL;
-
-		g_free (complement);
-		break;
-
-	case BRASERO_IMAGE_FORMAT_CLONE:
-		if (image)
-			*image = output;
-		else
-			g_free (output);
-
-		if (toc)
-			*toc = complement;
-		else
-			g_free (complement);
-		break;
-
-	case BRASERO_IMAGE_FORMAT_CUE:
-	case BRASERO_IMAGE_FORMAT_CDRDAO:
-		if (image)
-			*image = complement;
-		else
-			g_free (complement);
-
-		if (toc)
-			*toc = output;
-		else
-			g_free (output);
-		break;
-
-	default:
-		return BRASERO_BURN_ERR;
-	}
-
-	return BRASERO_BURN_OK;
-}
-
 /**
  * This function returns a path only if we should output to a file image
  * and not burn.
@@ -641,11 +493,12 @@ brasero_burn_session_set_image_output_retval (BraseroBurnSession *self,
 
 BraseroBurnResult
 brasero_burn_session_get_output (BraseroBurnSession *self,
-				 gchar **image,
-				 gchar **toc,
+				 gchar **image_ret,
+				 gchar **toc_ret,
 				 GError **error)
 {
-	BraseroBurnResult result;
+	gchar *toc = NULL;
+	gchar *image = NULL;
 	BraseroBurnSessionPrivate *priv;
 
 	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
@@ -657,40 +510,54 @@ brasero_burn_session_get_output (BraseroBurnSession *self,
 		return BRASERO_BURN_ERR;
 	}
 
-	/* output paths were set so test them and returns them if OK */
-	if (priv->settings->image) {
-		result = brasero_burn_session_file_test (self,
-							 priv->settings->image,
-							 error);
-		if (result != BRASERO_BURN_OK) {
-			BRASERO_BURN_LOG ("Problem with image existence");
-			return result;
-		}
-	}
-	else {
-		BRASERO_BURN_LOG ("no output specified");
+	image = g_strdup (priv->settings->image);
+	toc = g_strdup (priv->settings->toc);
 
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("no output specified"));
+	if (!image && !toc)
 		return BRASERO_BURN_ERR;
-	}
 
-	if (priv->settings->toc) {
-		result = brasero_burn_session_file_test (self,
-							 priv->settings->toc,
-							 error);
-		if (result != BRASERO_BURN_OK) {
-			BRASERO_BURN_LOG ("Problem with toc existence");
-			return result;
+	if (image_ret) {
+		/* output paths were set so test them and returns them if OK */
+		if (!image && toc) {
+			gchar *complement;
+			BraseroImageFormat format;
+
+			/* get the cuesheet complement */
+			format = brasero_burn_session_get_output_format (self);
+			complement = brasero_image_format_get_complement (format, toc);
+			if (!complement) {
+				BRASERO_BURN_LOG ("no output specified");
+
+				g_set_error (error,
+					     BRASERO_BURN_ERROR,
+					     BRASERO_BURN_ERROR_OUTPUT_NONE,
+					     _("No path was specified for the image output"));
+
+				g_free (toc);
+				return BRASERO_BURN_ERR;
+			}
+
+			*image_ret = complement;
+		}
+		else if (image)
+			*image_ret = image;
+		else {
+			BRASERO_BURN_LOG ("no output specified");
+
+			g_set_error (error,
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_OUTPUT_NONE,
+				     _("No path was specified for the image output"));
+			return BRASERO_BURN_ERR;
 		}
 	}
+	else
+		g_free (image);
 
-	if (image)
-		*image = g_strdup (priv->settings->image);
-	if (toc)
-		*toc = g_strdup (priv->settings->toc);
+	if (toc_ret)
+		*toc_ret = toc;
+	else
+		g_free (toc);
 
 	return BRASERO_BURN_OK;
 }
@@ -727,23 +594,24 @@ brasero_burn_session_set_image_output_full (BraseroBurnSession *self,
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 
-	if (!BRASERO_BURN_SESSION_WRITE_TO_FILE (priv)) {
-		BraseroMediumMonitor *monitor;
-		BraseroDrive *drive;
-		GSList *list;
-
-		monitor = brasero_medium_monitor_get_default ();
-		list = brasero_medium_monitor_get_media (monitor, BRASERO_MEDIA_TYPE_FILE);
-		drive = brasero_medium_get_drive (list->data);
-		brasero_burn_session_set_burner (self, drive);
-		g_object_unref (monitor);
-		g_slist_free (list);
-	}
-
 	if (priv->settings->format == format
 	&&  BRASERO_STR_EQUAL (image, priv->settings->image)
-	&&  BRASERO_STR_EQUAL (toc, priv->settings->toc))
+	&&  BRASERO_STR_EQUAL (toc, priv->settings->toc)) {
+		if (!BRASERO_BURN_SESSION_WRITE_TO_FILE (priv)) {
+			BraseroMediumMonitor *monitor;
+			BraseroDrive *drive;
+			GSList *list;
+
+			monitor = brasero_medium_monitor_get_default ();
+			list = brasero_medium_monitor_get_media (monitor, BRASERO_MEDIA_TYPE_FILE);
+			drive = brasero_medium_get_drive (list->data);
+			brasero_burn_session_set_burner (self, drive);
+			g_object_unref (monitor);
+			g_slist_free (list);
+		}
+
 		return BRASERO_BURN_OK;
+	}
 
 	if (priv->settings->image)
 		g_free (priv->settings->image);
@@ -763,49 +631,26 @@ brasero_burn_session_set_image_output_full (BraseroBurnSession *self,
 
 	priv->settings->format = format;
 
-	g_signal_emit (self,
-		       brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL],
-		       0);
+	if (!BRASERO_BURN_SESSION_WRITE_TO_FILE (priv)) {
+		BraseroMediumMonitor *monitor;
+		BraseroDrive *drive;
+		GSList *list;
+
+		monitor = brasero_medium_monitor_get_default ();
+		list = brasero_medium_monitor_get_media (monitor,BRASERO_MEDIA_TYPE_FILE);
+		drive = brasero_medium_get_drive (list->data);
+		brasero_burn_session_set_burner (self, drive);
+		g_object_unref (monitor);
+		g_slist_free (list);
+	}
+	else
+		g_signal_emit (self,
+			       brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL],
+			       0);
+
 	return BRASERO_BURN_OK;
 }
 
-BraseroBurnResult
-brasero_burn_session_set_image_output (BraseroBurnSession *self,
-				       BraseroImageFormat format,
-				       const gchar *path)
-{
-	BraseroBurnSessionPrivate *priv;
-	BraseroBurnResult result;
-	gchar *complement;
-	gchar *image = NULL;
-	gchar *toc = NULL;
-
-	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
-	/* find the file complement */
-	complement = brasero_burn_session_get_file_complement (self,
-							       format,
-							       path);
-
-	brasero_burn_session_set_image_output_retval (self,
-						      format,
-						      &image,
-						      &toc,
-						      g_strdup (path),
-						      complement);
-
-	result = brasero_burn_session_set_image_output_full (self,
-							     format,
-							     image,
-							     toc);
-	g_free (image);
-	g_free (toc);
-
-	return result;
-}
-
 /**
  *
  */
@@ -867,12 +712,21 @@ brasero_burn_session_get_tmp_dir (BraseroBurnSession *self,
 
 	*path = mkdtemp (tmp);
 	if (*path == NULL) {
+                int errsv = errno;
+
+		BRASERO_BURN_LOG ("Impossible to create tmp directory");
 		g_free (tmp);
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("a temporary directory could not be created (%s)"),
-			     strerror (errno));
+		if (errsv != EACCES)
+			g_set_error (error, 
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_GENERAL,
+				     "%s",
+				     g_strerror (errsv));
+		else
+			g_set_error (error,
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_PERMISSION,
+				     _("You do not have the required permission to write at this location"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -915,12 +769,22 @@ brasero_burn_session_get_tmp_file (BraseroBurnSession *self,
 
 	fd = g_mkstemp (tmp);
 	if (fd == -1) {
+                int errsv = errno;
+
 		g_free (tmp);
-		g_set_error (error, 
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("a temporary file can't be created: %s"),
-			     strerror (errno));
+		BRASERO_BURN_LOG ("Impossible to create tmp file");
+		if (errsv != EACCES)
+			g_set_error (error, 
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_GENERAL,
+				     "%s",
+				     g_strerror (errsv));
+		else
+			g_set_error (error, 
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_PERMISSION,
+				     _("You do not have the required permission to write at this location"));
+
 		return BRASERO_BURN_ERR;
 	}
 
@@ -933,6 +797,40 @@ brasero_burn_session_get_tmp_file (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
+static gchar *
+brasero_burn_session_get_image_complement (BraseroBurnSession *self,
+					   BraseroImageFormat format,
+					   const gchar *path)
+{
+	gchar *retval = NULL;
+	BraseroBurnSessionPrivate *priv;
+
+	priv = BRASERO_BURN_SESSION_PRIVATE (self);
+
+	if (format == BRASERO_IMAGE_FORMAT_CLONE)
+		retval = g_strdup_printf ("%s.toc", path);
+	else if (format == BRASERO_IMAGE_FORMAT_CUE) {
+		if (g_str_has_suffix (path, ".bin"))
+			retval = g_strdup_printf ("%.*scue",
+						  strlen (path) - 3,
+						  path);
+		else
+			retval = g_strdup_printf ("%s.cue", path);
+	}
+	else if (format == BRASERO_IMAGE_FORMAT_CDRDAO) {
+		if (g_str_has_suffix (path, ".bin"))
+			retval = g_strdup_printf ("%.*stoc",
+						  strlen (path) - 3,
+						  path);
+		else
+			retval = g_strdup_printf ("%s.toc", path);
+	}
+	else
+		retval = NULL;
+
+	return retval;
+}
+
 BraseroBurnResult
 brasero_burn_session_get_tmp_image (BraseroBurnSession *self,
 				    BraseroImageFormat format,
@@ -950,20 +848,21 @@ brasero_burn_session_get_tmp_image (BraseroBurnSession *self,
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 
 	/* Image tmp file */
-	result = brasero_burn_session_get_tmp_file (self, NULL, &path, error);
+	result = brasero_burn_session_get_tmp_file (self,
+						    (format == BRASERO_IMAGE_FORMAT_CLONE)? NULL:".bin",
+						    &path,
+						    error);
 	if (result != BRASERO_BURN_OK)
 		return result;
 
 	if (format != BRASERO_IMAGE_FORMAT_BIN) {
 		/* toc tmp file */
-		complement = brasero_burn_session_get_file_complement (self, format, path);
+		complement = brasero_burn_session_get_image_complement (self, format, path);
 		if (complement) {
-			result = brasero_burn_session_file_test (self,
-								 complement,
-								 error);
-			if (result != BRASERO_BURN_OK) {
+			/* That shouldn't happen ... */
+			if (g_file_test (complement, G_FILE_TEST_EXISTS)) {
 				g_free (complement);
-				return result;
+				return BRASERO_BURN_ERR;
 			}
 		}
 	}
@@ -972,12 +871,15 @@ brasero_burn_session_get_tmp_image (BraseroBurnSession *self,
 		priv->tmpfiles = g_slist_prepend (priv->tmpfiles,
 						  g_strdup (complement));
 
-	brasero_burn_session_set_image_output_retval (self,
-						      format,
-						      image,
-						      toc,
-						      path,
-						      complement);
+	if (image)
+		*image = path;
+	else
+		g_free (path);
+
+	if (toc)
+		*toc = complement;
+	else
+		g_free (complement);
 
 	return BRASERO_BURN_OK;
 }
@@ -1094,6 +996,22 @@ brasero_burn_session_tag_value_free (gpointer user_data)
 	g_free (value);
 }
 
+BraseroBurnResult
+brasero_burn_session_tag_remove (BraseroBurnSession *self,
+				 const gchar *tag)
+{
+	BraseroBurnSessionPrivate *priv;
+
+	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
+
+	priv = BRASERO_BURN_SESSION_PRIVATE (self);
+	if (!priv->tags)
+		return BRASERO_BURN_ERR;
+
+	g_hash_table_remove (priv->tags, tag);
+	return BRASERO_BURN_OK;
+}
+
 BraseroBurnResult
 brasero_burn_session_tag_add (BraseroBurnSession *self,
 			      const gchar *tag,
@@ -1387,37 +1305,6 @@ brasero_burn_session_same_src_dest_drive (BraseroBurnSession *self)
 }
 
 
-/**
- *
- */
-
-void
-brasero_burn_session_add_wrong_checksum (BraseroBurnSession *self,
-					 const gchar *path)
-{
-	BraseroBurnSessionPrivate *priv;
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-	priv->wrong_checksums = g_slist_prepend (priv->wrong_checksums, g_strdup (path));
-}
-
-GSList *
-brasero_burn_session_get_wrong_checksums (BraseroBurnSession *self)
-{
-	BraseroBurnSessionPrivate *priv;
-	GSList *retval;
-
-	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), NULL);
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
-	/* reset our list so it will return only the new ones next time */
-	retval = priv->wrong_checksums;
-	priv->wrong_checksums = NULL;
-
-	return retval;
-}
-
 /****************************** this part is for log ***************************/
 void
 brasero_burn_session_logv (BraseroBurnSession *self,
@@ -1438,10 +1325,7 @@ brasero_burn_session_logv (BraseroBurnSession *self,
 	if (!priv->session)
 		return;
 
-	if (arg_list)
-		message = g_strdup_vprintf (format, arg_list);
-	else
-		message = g_strdup (format);
+	message = g_strdup_vprintf (format, arg_list);
 
 	/* we also need to validate the messages to be in UTF-8 */
 	if (!g_utf8_validate (message, -1, (const gchar**) &offending))
@@ -1544,7 +1428,6 @@ brasero_burn_session_start (BraseroBurnSession *self)
 		medium = brasero_drive_get_medium (priv->settings->burner);
 		BRASERO_BURN_LOG_DISC_TYPE (brasero_medium_get_status (medium), "media type\t=");
 		BRASERO_BURN_LOG ("speed\t= %i", priv->settings->rate);
-		BRASERO_BURN_LOG ("number of copies\t= %i", priv->settings->num_copies);
 	}
 	else {
 		type.type = BRASERO_TRACK_TYPE_IMAGE;
@@ -1569,88 +1452,6 @@ brasero_burn_session_stop (BraseroBurnSession *self)
 	}
 }
 
-/**
- * Misc
- */
-
-gchar *
-brasero_burn_session_get_config_key (BraseroBurnSession *self,
-				     const gchar *property)
-{
-	BraseroMedium *medium;
-	BraseroDrive *drive;
-	gchar *display_name;
-	gchar *key = NULL;
-	gchar *disc_type;
-
-	drive = brasero_burn_session_get_burner (self);
-	if (!drive)
-		return NULL;
-
-	medium = brasero_drive_get_medium (drive);
-	if (brasero_medium_get_status (medium) == BRASERO_MEDIUM_NONE)
-		return NULL;
-	
-	/* make sure display_name doesn't contain any forbidden characters */
-	display_name = brasero_drive_get_display_name (drive);
-	g_strdelimit (display_name, " +()", '_');
-
-	disc_type = g_strdup (brasero_medium_get_type_string (medium));
-	if (!disc_type) {
-		g_free (display_name);
-		return NULL;
-	}
-
-	g_strdelimit (disc_type, " +()", '_');
-
-	switch (brasero_burn_session_get_input_type (self, NULL)) {
-	case BRASERO_TRACK_TYPE_NONE:
-		key = g_strdup_printf ("%s/%s/none_%s/%s",
-				       BRASERO_DRIVE_PROPERTIES_KEY,
-				       display_name,
-				       disc_type,
-				       property);
-		break;
-	case BRASERO_TRACK_TYPE_DISC:
-		key = g_strdup_printf ("%s/%s/disc_%s/%s",
-				       BRASERO_DRIVE_PROPERTIES_KEY,
-				       display_name,
-				       disc_type,
-				       property);
-		break;
-
-	case BRASERO_TRACK_TYPE_DATA:
-		key = g_strdup_printf ("%s/%s/data_%s/%s",
-				       BRASERO_DRIVE_PROPERTIES_KEY,
-				       display_name,
-				       disc_type,
-				       property);
-		break;
-
-	case BRASERO_TRACK_TYPE_IMAGE:
-		key = g_strdup_printf ("%s/%s/image_%s/%s",
-				       BRASERO_DRIVE_PROPERTIES_KEY,
-				       display_name,
-				       disc_type,
-				       property);
-		break;
-
-	case BRASERO_TRACK_TYPE_AUDIO:
-		key = g_strdup_printf ("%s/%s/audio_%s/%s",
-				       BRASERO_DRIVE_PROPERTIES_KEY,
-				       display_name,
-				       disc_type,
-				       property);
-		break;
-	default:
-		break;
-	}
-
-	g_free (display_name);
-	g_free (disc_type);
-	return key;
-}
-
 /**
  *
  */
@@ -1662,6 +1463,68 @@ brasero_burn_session_track_list_free (GSList *list)
 	g_slist_free (list);
 }
 
+/**
+ * Utility to clean tmp files
+ */
+
+static gboolean
+brasero_burn_session_clean (const gchar *path);
+
+static gboolean
+brasero_burn_session_clean_directory (const gchar *path)
+{
+	GDir *dir;
+	const gchar *name;
+
+	dir = g_dir_open (path, 0, NULL);
+	if (!dir)
+		return FALSE;
+
+	while ((name = g_dir_read_name (dir))) {
+		gchar *tmp;
+
+		tmp = g_build_filename (G_DIR_SEPARATOR_S,
+					path,
+					name,
+					NULL);
+
+		if (!brasero_burn_session_clean (tmp)) {
+			g_dir_close (dir);
+			g_free (tmp);
+			return FALSE;
+		}
+
+		g_free (tmp);
+	}
+
+	g_dir_close (dir);
+	return TRUE;
+}
+
+static gboolean
+brasero_burn_session_clean (const gchar *path)
+{
+	gboolean result = TRUE;
+
+	if (!path)
+		return TRUE;
+
+	BRASERO_BURN_LOG ("Cleaning %s", path);
+
+	/* NOTE: g_file_test follows symbolic links */
+	if (g_file_test (path, G_FILE_TEST_IS_DIR)
+	&& !g_file_test (path, G_FILE_TEST_IS_SYMLINK))
+		brasero_burn_session_clean_directory (path);
+
+	/* NOTE : we don't follow paths as certain files are simply linked */
+	if (g_remove (path)) {
+		BRASERO_BURN_LOG ("Cannot remove file %s (%s)", path, g_strerror (errno));
+		result = FALSE;
+	}
+
+	return result;
+}
+
 static void
 brasero_burn_session_finalize (GObject *object)
 {
@@ -1720,7 +1583,7 @@ brasero_burn_session_finalize (GObject *object)
 
 		tmpfile = iter->data;
 
-		g_remove (tmpfile);
+		brasero_burn_session_clean (tmpfile);
 		g_free (tmpfile);
 	}
 	g_slist_free (priv->tmpfiles);
@@ -1736,12 +1599,6 @@ brasero_burn_session_finalize (GObject *object)
 		priv->session_path = NULL;
 	}
 
-	if (priv->wrong_checksums) {
-		g_slist_foreach (priv->wrong_checksums, (GFunc) g_free, NULL);
-		g_slist_free (priv->wrong_checksums);
-		priv->wrong_checksums = NULL;
-	}
-
 	brasero_session_settings_clean (priv->settings);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -1765,7 +1622,7 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 	brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL] =
 	    g_signal_new ("output_changed",
 			  BRASERO_TYPE_BURN_SESSION,
-			  G_SIGNAL_RUN_LAST|G_SIGNAL_ACTION|G_SIGNAL_NO_RECURSE,
+			  G_SIGNAL_RUN_FIRST,
 			  G_STRUCT_OFFSET (BraseroBurnSessionClass, output_changed),
 			  NULL,
 			  NULL,
@@ -1776,7 +1633,7 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 	brasero_burn_session_signals [INPUT_CHANGED_SIGNAL] =
 	    g_signal_new ("input_changed",
 			  BRASERO_TYPE_BURN_SESSION,
-			  G_SIGNAL_RUN_LAST|G_SIGNAL_ACTION|G_SIGNAL_NO_RECURSE,
+			  G_SIGNAL_RUN_FIRST,
 			  G_STRUCT_OFFSET (BraseroBurnSessionClass, input_changed),
 			  NULL,
 			  NULL,
