@@ -38,6 +38,7 @@ struct _BraseroVolumePrivate
 {
 	GCancellable *cancel;
 
+	guint timeout_id;
 	GMainLoop *loop;
 	gboolean result;
 	GError *error;
@@ -45,17 +46,6 @@ struct _BraseroVolumePrivate
 
 #define BRASERO_VOLUME_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_VOLUME, BraseroVolumePrivate))
 
-enum
-{
-	MOUNTED,
-	UNMOUNTED,
-
-	LAST_SIGNAL
-};
-
-
-static guint volume_signals[LAST_SIGNAL] = { 0 };
-
 G_DEFINE_TYPE (BraseroVolume, brasero_volume, BRASERO_TYPE_MEDIUM);
 
 static GVolume *
@@ -169,8 +159,12 @@ brasero_volume_get_gdrive (BraseroVolume *self)
 gboolean
 brasero_volume_is_mounted (BraseroVolume *self)
 {
-	GMount *mount;
-	GVolume *volume;
+	GList *iter;
+	GList *mounts;
+	gboolean result;
+	BraseroDrive *drive;
+	GVolumeMonitor *monitor;
+	const gchar *volume_path;
 	BraseroVolumePrivate *priv;
 
 	if (!self)
@@ -178,23 +172,42 @@ brasero_volume_is_mounted (BraseroVolume *self)
 
 	priv = BRASERO_VOLUME_PRIVATE (self);
 
-	volume = brasero_volume_get_gvolume (self);
-	if (!volume)
+	drive = brasero_medium_get_drive (BRASERO_MEDIUM (self));
+
+#if defined(HAVE_STRUCT_USCSI_CMD)
+	volume_path = brasero_drive_get_block_device (drive);
+#else
+	volume_path = brasero_drive_get_device (drive);
+#endif
+
+	if (!volume_path)
 		return FALSE;
 
-	if (!g_volume_can_mount (volume)) {
-		/* if it can't be mounted then it's unmounted ... */
-		g_object_unref (volume);
-		return FALSE;
+	monitor = g_volume_monitor_get ();
+	mounts = g_volume_monitor_get_mounts (monitor);
+	g_object_unref (monitor);
+
+	result = FALSE;
+	for (iter = mounts; iter; iter = iter->next) {
+		GMount *mount;
+		GVolume *volume;
+		gchar *device_path;
+
+		mount = iter->data;
+		volume = g_mount_get_volume (mount);
+		device_path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+		if (!device_path)
+			continue;		
+
+		if (!strcmp (device_path, volume_path)) {
+			result = TRUE;
+			break;
+		}
 	}
+	g_list_foreach (mounts, (GFunc) g_object_unref, NULL);
+	g_list_free (mounts);
 
-	mount = g_volume_get_mount (volume);
-	g_object_unref (volume);
-	if (!mount)
-		return FALSE;
-
-	g_object_unref (mount);
-	return TRUE;
+	return result;
 }
 
 gchar *
@@ -226,7 +239,7 @@ brasero_volume_get_mount_point (BraseroVolume *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the disc mount point could not be retrieved."));
+			     _("The disc mount point could not be retrieved"));
 	}
 	else {
 		local_path = g_file_get_path (root);
@@ -236,34 +249,6 @@ brasero_volume_get_mount_point (BraseroVolume *self,
 	return local_path;
 }
 
-static gboolean
-brasero_volume_wait_for_operation_end (BraseroVolume *self,
-				       GError **error)
-{
-	BraseroVolumePrivate *priv;
-
-	priv = BRASERO_VOLUME_PRIVATE (self);
-
-	/* FIXME! that's where we should put a timeout (30 sec ?) */
-	priv->loop = g_main_loop_new (NULL, FALSE);
-	g_main_loop_run (priv->loop);
-
-	g_main_loop_unref (priv->loop);
-	priv->loop = NULL;
-
-	if (priv->error) {
-		if (error)
-			g_propagate_error (error, priv->error);
-		else
-			g_error_free (priv->error);
-
-		priv->error = NULL;
-	}
-	g_cancellable_reset (priv->cancel);
-
-	return priv->result;
-}
-
 static void
 brasero_volume_operation_end (BraseroVolume *self)
 {
@@ -279,6 +264,58 @@ brasero_volume_operation_end (BraseroVolume *self)
 	g_main_loop_quit (priv->loop);	
 }
 
+static gboolean
+brasero_volume_operation_timeout (gpointer data)
+{
+	BraseroVolume *self = BRASERO_VOLUME (data);
+	BraseroVolumePrivate *priv;
+
+	priv = BRASERO_VOLUME_PRIVATE (self);
+	brasero_volume_operation_end (self);
+
+	BRASERO_BURN_LOG ("Volume/Disc operation timed out");
+	priv->timeout_id = 0;
+	priv->result = FALSE;
+	return FALSE;
+}
+
+static gboolean
+brasero_volume_wait_for_operation_end (BraseroVolume *self,
+				       GError **error)
+{
+	BraseroVolumePrivate *priv;
+
+	priv = BRASERO_VOLUME_PRIVATE (self);
+
+	/* put a timeout (30 sec) */
+	priv->timeout_id = g_timeout_add_seconds (20,
+						  brasero_volume_operation_timeout,
+						  self);
+
+	priv->loop = g_main_loop_new (NULL, FALSE);
+	g_main_loop_run (priv->loop);
+
+	g_main_loop_unref (priv->loop);
+	priv->loop = NULL;
+
+	if (priv->timeout_id) {
+		g_source_remove (priv->timeout_id);
+		priv->timeout_id = 0;
+	}
+
+	if (priv->error) {
+		if (error)
+			g_propagate_error (error, priv->error);
+		else
+			g_error_free (priv->error);
+
+		priv->error = NULL;
+	}
+	g_cancellable_reset (priv->cancel);
+
+	return priv->result;
+}
+
 static void
 brasero_volume_umounted_cb (GVolumeMonitor *monitor,
 			    GMount *mount,
@@ -301,6 +338,8 @@ brasero_volume_umounted_cb (GVolumeMonitor *monitor,
 	}
 
 	g_object_unref (vol_mount);
+
+	/* Check it's the one we were looking for */
 	if (vol_mount != mount)
 		return;
 
@@ -324,6 +363,8 @@ brasero_volume_umount_finish (GObject *source,
 					       result,
 					       &priv->error);
 
+	BRASERO_BURN_LOG ("Umount operation completed (result = %d)", priv->result);
+
 	if (priv->error) {
 		if (priv->error->code == G_IO_ERROR_FAILED_HANDLED) {
 			/* means we shouldn't display any error message since 
@@ -337,12 +378,14 @@ brasero_volume_umount_finish (GObject *source,
 			priv->error = NULL;
 			priv->result = TRUE;
 		}
-	}
 
-	if (priv->result)
-		g_signal_emit (self,
-			       volume_signals[UNMOUNTED],
-			       0);
+		/* Since there was an error. The "unmounted" signal won't be 
+		 * emitted by GVolumeMonitor and therefore we'd get stuck if
+		 * we didn't get out of the loop. */
+		brasero_volume_operation_end (self);
+	}
+	else if (!priv->result)
+		brasero_volume_operation_end (self);
 }
 
 gboolean
@@ -384,10 +427,10 @@ brasero_volume_umount (BraseroVolume *self,
 		GVolumeMonitor *monitor;
 
 		monitor = g_volume_monitor_get ();
-		umount_sig = g_signal_connect (monitor,
-					       "mount-removed",
-					       G_CALLBACK (brasero_volume_umounted_cb),
-					       self);
+		umount_sig = g_signal_connect_after (monitor,
+						     "mount-removed",
+						     G_CALLBACK (brasero_volume_umounted_cb),
+						     self);
 
 		g_mount_unmount (mount,
 				 G_MOUNT_UNMOUNT_NONE,
@@ -440,11 +483,6 @@ brasero_volume_mount_finish (GObject *source,
 	}
 
 	brasero_volume_operation_end (self);
-
-	if (priv->result)
-		g_signal_emit (self,
-			       volume_signals[MOUNTED],
-			       0);
 }
 
 gboolean
@@ -769,6 +807,11 @@ brasero_volume_finalize (GObject *object)
 		priv->cancel = NULL;
 	}
 
+	if (priv->timeout_id) {
+		g_source_remove (priv->timeout_id);
+		priv->timeout_id = 0;
+	}
+
 	if (priv->loop && g_main_loop_is_running (priv->loop))
 		g_main_loop_quit (priv->loop);
 
@@ -783,26 +826,6 @@ brasero_volume_class_init (BraseroVolumeClass *klass)
 	g_type_class_add_private (klass, sizeof (BraseroVolumePrivate));
 
 	object_class->finalize = brasero_volume_finalize;
-
-	volume_signals[MOUNTED] =
-		g_signal_new ("mounted",
-		              G_OBJECT_CLASS_TYPE (klass),
-		              G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_ACTION,
-		              0,
-		              NULL, NULL,
-		              g_cclosure_marshal_VOID__VOID,
-		              G_TYPE_NONE, 0,
-		              G_TYPE_NONE);
-
-	volume_signals[UNMOUNTED] =
-		g_signal_new ("unmounted",
-		              G_OBJECT_CLASS_TYPE (klass),
-		              G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_ACTION,
-		              0,
-		              NULL, NULL,
-		              g_cclosure_marshal_VOID__VOID,
-		              G_TYPE_NONE, 0,
-		              G_TYPE_NONE);
 }
 
 BraseroVolume *
