@@ -122,6 +122,7 @@ typedef enum {
 	WARN_AUDIO_TO_APPENDABLE_SIGNAL,
 	WARN_REWRITABLE_SIGNAL,
 	INSERT_MEDIA_REQUEST_SIGNAL,
+	LOCATION_REQUEST_SIGNAL,
 	PROGRESS_CHANGED_SIGNAL,
 	ACTION_CHANGED_SIGNAL,
 	DUMMY_SUCCESS_SIGNAL,
@@ -289,7 +290,7 @@ brasero_burn_eject (BraseroBurn *self,
 				g_set_error (error,
 					     BRASERO_BURN_ERROR,
 					     BRASERO_BURN_ERROR_GENERAL,
-					     _("the media in %s can't be ejected"),
+					     _("The disc in \"%s\" cannot be ejected"),
 					     name);
 			g_free (name);
 			return BRASERO_BURN_ERR;
@@ -332,7 +333,7 @@ brasero_burn_eject_dest_media (BraseroBurn *self,
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
-				     _("\"%s\" can't be unlocked"),
+				     _("\"%s\" cannot be unlocked"),
 				     name);
 			g_free (name);
 			return BRASERO_BURN_ERR;
@@ -379,7 +380,7 @@ brasero_burn_eject_src_media (BraseroBurn *self,
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
-				     _("\"%s\" can't be unlocked"),
+				     _("\"%s\" cannot be unlocked"),
 				     name);
 			g_free (name);
 			return BRASERO_BURN_ERR;
@@ -434,6 +435,41 @@ brasero_burn_ask_for_media (BraseroBurn *burn,
 	return g_value_get_int (&return_value);
 }
 
+static BraseroBurnResult
+brasero_burn_ask_for_location (BraseroBurn *burn,
+			       GError *received_error,
+			       gboolean is_temporary,
+			       GError **error)
+{
+	GValue instance_and_params [3];
+	GValue return_value;
+
+	instance_and_params [0].g_type = 0;
+	g_value_init (instance_and_params, G_TYPE_FROM_INSTANCE (burn));
+	g_value_set_instance (instance_and_params, burn);
+	
+	instance_and_params [1].g_type = 0;
+	g_value_init (instance_and_params + 1, G_TYPE_POINTER);
+	g_value_set_pointer (instance_and_params + 1, received_error);
+	
+	instance_and_params [2].g_type = 0;
+	g_value_init (instance_and_params + 2, G_TYPE_BOOLEAN);
+	g_value_set_boolean (instance_and_params + 2, is_temporary);
+	
+	return_value.g_type = 0;
+	g_value_init (&return_value, G_TYPE_INT);
+	g_value_set_int (&return_value, BRASERO_BURN_CANCEL);
+
+	g_signal_emitv (instance_and_params,
+			brasero_burn_signals [LOCATION_REQUEST_SIGNAL],
+			0,
+			&return_value);
+
+	g_value_unset (instance_and_params);
+	g_value_unset (instance_and_params + 1);
+
+	return g_value_get_int (&return_value);
+}
 static BraseroBurnResult
 brasero_burn_ask_for_src_media (BraseroBurn *burn,
 				BraseroBurnError error_type,
@@ -474,8 +510,8 @@ brasero_burn_ask_for_dest_media (BraseroBurn *burn,
 		if (!priv->dest) {
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
-				     _("no drive specified"));
+				     BRASERO_BURN_ERROR_OUTPUT_NONE,
+				     _("No burner specified"));
 			return BRASERO_BURN_ERR;
 		}
 	}
@@ -512,7 +548,7 @@ brasero_burn_lock_src_media (BraseroBurn *burn,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("no drive specified as source"));
+			     _("No source drive specified"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -530,19 +566,19 @@ again:
 	 * get any information from the drive */
 	media = brasero_medium_get_status (medium);
 	if (media == BRASERO_MEDIUM_NONE)
-		error_type = BRASERO_BURN_ERROR_MEDIA_NONE;
+		error_type = BRASERO_BURN_ERROR_MEDIUM_NONE;
 	else if (media == BRASERO_MEDIUM_BUSY)
-		error_type = BRASERO_BURN_ERROR_MEDIA_BUSY;
+		error_type = BRASERO_BURN_ERROR_DRIVE_BUSY;
 	else if (media == BRASERO_MEDIUM_UNSUPPORTED)
-		error_type = BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED;
+		error_type = BRASERO_BURN_ERROR_MEDIUM_INVALID;
 	else if (media & BRASERO_MEDIUM_BLANK)
-		error_type = BRASERO_BURN_ERROR_MEDIA_BLANK;
+		error_type = BRASERO_BURN_ERROR_MEDIUM_NO_DATA;
 	else
 		error_type = BRASERO_BURN_ERROR_NONE;
 
 	if (media & BRASERO_MEDIUM_BLANK) {
 		result = brasero_burn_ask_for_src_media (burn,
-							 BRASERO_BURN_ERROR_MEDIA_BLANK,
+							 BRASERO_BURN_ERROR_MEDIUM_NO_DATA,
 							 BRASERO_MEDIUM_HAS_DATA,
 							 error);
 		if (result != BRASERO_BURN_OK)
@@ -552,11 +588,11 @@ again:
 	}
 
 	if (!priv->src_locked
-	&&  !brasero_drive_lock (priv->src, _("ongoing copying process"), &failure)) {
+	&&  !brasero_drive_lock (priv->src, _("Ongoing copying process"), &failure)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the drive can't be locked (%s)"),
+			     _("The drive cannot be locked (%s)"),
 			     failure);
 		return BRASERO_BURN_ERR;
 	}
@@ -599,8 +635,8 @@ brasero_burn_lock_rewritable_media (BraseroBurn *burn,
 	if (!priv->dest) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("no drive specified"));
+			     BRASERO_BURN_ERROR_OUTPUT_NONE,
+			     _("No burner specified"));
 		return BRASERO_BURN_NOT_SUPPORTED;
 	}
 
@@ -610,8 +646,8 @@ brasero_burn_lock_rewritable_media (BraseroBurn *burn,
 	if (!brasero_medium_can_be_rewritten (medium)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the drive has no rewriting capabilities"));
+			     BRASERO_BURN_ERROR_MEDIUM_NOT_REWRITABLE,
+			     _("The drive has no rewriting capabilities"));
 		return BRASERO_BURN_NOT_SUPPORTED;
 	}
 
@@ -623,13 +659,13 @@ brasero_burn_lock_rewritable_media (BraseroBurn *burn,
 
 	media = brasero_medium_get_status (medium);
 	if (media == BRASERO_MEDIUM_NONE)
-		error_type = BRASERO_BURN_ERROR_MEDIA_NONE;
+		error_type = BRASERO_BURN_ERROR_MEDIUM_NONE;
 	else if (media == BRASERO_MEDIUM_BUSY)
-		error_type = BRASERO_BURN_ERROR_MEDIA_BUSY;
+		error_type = BRASERO_BURN_ERROR_DRIVE_BUSY;
 	else if (media == BRASERO_MEDIUM_UNSUPPORTED)
-		error_type = BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED;
+		error_type = BRASERO_BURN_ERROR_MEDIUM_INVALID;
 	else if (!(media & BRASERO_MEDIUM_REWRITABLE))
-		error_type = BRASERO_BURN_ERROR_MEDIA_NOT_REWRITABLE;
+		error_type = BRASERO_BURN_ERROR_MEDIUM_NOT_REWRITABLE;
 	else
 		error_type = BRASERO_BURN_ERROR_NONE;
 
@@ -647,11 +683,11 @@ brasero_burn_lock_rewritable_media (BraseroBurn *burn,
 	}
 
 	if (!priv->dest_locked
-	&&  !brasero_drive_lock (priv->dest, _("ongoing blanking process"), &failure)) {
+	&&  !brasero_drive_lock (priv->dest, _("Ongoing blanking process"), &failure)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the drive can't be locked (%s)"),
+			     _("The drive cannot be locked (%s)"),
 			     failure);
 		return BRASERO_BURN_ERR;
 	}
@@ -706,7 +742,7 @@ brasero_burn_is_loaded_dest_media_supported (BraseroBurn *burn,
 
 	if (!(flags & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE)) {
 		*must_blank = FALSE;
-		return BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED;
+		return BRASERO_BURN_ERROR_MEDIUM_INVALID;
 	}
 
 	/* let's see what our media is missing and what's not supported */
@@ -723,12 +759,10 @@ brasero_burn_is_loaded_dest_media_supported (BraseroBurn *burn,
 			return BRASERO_BURN_ERROR_NONE;
 		}
 
-		return BRASERO_BURN_ERROR_MEDIA_NOT_WRITABLE;
+		return BRASERO_BURN_ERROR_MEDIUM_NOT_WRITABLE;
 	}
-	else if (unsupported & BRASERO_MEDIUM_DVD)
-		return BRASERO_BURN_ERROR_DVD_NOT_SUPPORTED;
 
-	return BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED;
+	return BRASERO_BURN_ERROR_MEDIUM_INVALID;
 }
 
 static BraseroBurnResult
@@ -750,8 +784,8 @@ brasero_burn_lock_dest_media (BraseroBurn *burn,
 	if (!priv->dest) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("no drive specified"));
+			     BRASERO_BURN_ERROR_OUTPUT_NONE,
+			     _("No burner specified"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -763,7 +797,7 @@ brasero_burn_lock_dest_media (BraseroBurn *burn,
 	medium = brasero_drive_get_medium (priv->dest);
 	if (!medium) {
 		result = BRASERO_BURN_NEED_RELOAD;
-		berror = BRASERO_BURN_ERROR_MEDIA_NONE;
+		berror = BRASERO_BURN_ERROR_MEDIUM_NONE;
 		goto end;
 	}
 
@@ -771,7 +805,7 @@ brasero_burn_lock_dest_media (BraseroBurn *burn,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the drive has no burning capabilities"));
+			     _("The drive cannot burn or the disc cannot be burnt"));
 		BRASERO_BURN_NOT_SUPPORTED_LOG (burn);
 	}
 
@@ -800,19 +834,19 @@ brasero_burn_lock_dest_media (BraseroBurn *burn,
 
 	if (media == BRASERO_MEDIUM_NONE) {
 		result = BRASERO_BURN_NEED_RELOAD;
-		berror = BRASERO_BURN_ERROR_MEDIA_NONE;
+		berror = BRASERO_BURN_ERROR_MEDIUM_NONE;
 		goto end;
 	}
 
 	if (media == BRASERO_MEDIUM_UNSUPPORTED) {
 		result = BRASERO_BURN_NEED_RELOAD;
-		berror = BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED;
+		berror = BRASERO_BURN_ERROR_MEDIUM_INVALID;
 		goto end;
 	}
 
 	if (media == BRASERO_MEDIUM_BUSY) {
 		result = BRASERO_BURN_NEED_RELOAD;
-		berror = BRASERO_BURN_ERROR_MEDIA_BUSY;
+		berror = BRASERO_BURN_ERROR_DRIVE_BUSY;
 		goto end;
 	}
 
@@ -832,7 +866,9 @@ brasero_burn_lock_dest_media (BraseroBurn *burn,
 		
 		/* we warn the user is going to lose data even if in the case of
 		 * DVD+/-RW we don't really blank the disc we rather overwrite */
-		result = brasero_burn_emit_signal (burn, WARN_DATA_LOSS_SIGNAL, BRASERO_BURN_CANCEL);
+		result = brasero_burn_emit_signal (burn,
+						   WARN_DATA_LOSS_SIGNAL,
+						   BRASERO_BURN_CANCEL);
 		if (result != BRASERO_BURN_OK)
 			goto end;
 	}
@@ -844,7 +880,9 @@ brasero_burn_lock_dest_media (BraseroBurn *burn,
 			 * append audio to appendable disc. That's because audio
 			 * tracks have little chance to be readable by common CD
 			 * player as last tracks */
-			result = brasero_burn_emit_signal (burn, WARN_AUDIO_TO_APPENDABLE_SIGNAL, BRASERO_BURN_CANCEL);
+			result = brasero_burn_emit_signal (burn,
+							   WARN_AUDIO_TO_APPENDABLE_SIGNAL,
+							   BRASERO_BURN_CANCEL);
 			if (result != BRASERO_BURN_OK)
 				goto end;
 		}
@@ -857,7 +895,9 @@ brasero_burn_lock_dest_media (BraseroBurn *burn,
 			/* warn the users that their previous data
 			 * session (s) will not be mounted by default by
 			 * the OS and that it'll be invisible */
-			result = brasero_burn_emit_signal (burn, WARN_PREVIOUS_SESSION_LOSS_SIGNAL, BRASERO_BURN_CANCEL);
+			result = brasero_burn_emit_signal (burn,
+							   WARN_PREVIOUS_SESSION_LOSS_SIGNAL,
+							   BRASERO_BURN_CANCEL);
 			if (result != BRASERO_BURN_OK)
 				goto end;
 		}
@@ -888,11 +928,11 @@ brasero_burn_lock_dest_media (BraseroBurn *burn,
 	}
 
 	if (!priv->dest_locked
-	&&  !brasero_drive_lock (priv->dest, _("ongoing burning process"), &failure)) {
+	&&  !brasero_drive_lock (priv->dest, _("Ongoing burning process"), &failure)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the drive can't be locked (%s)"),
+			     _("The drive cannot be locked (%s)"),
 			     failure);
 		return BRASERO_BURN_ERR;
 	}
@@ -906,7 +946,7 @@ end:
 		brasero_drive_unlock (priv->dest);
 	}
 
-	if (result == BRASERO_BURN_ERROR_RELOAD_MEDIA && ret_error)
+	if (result == BRASERO_BURN_NEED_RELOAD && ret_error)
 		*ret_error = berror;
 
 	return result;
@@ -967,10 +1007,11 @@ brasero_burn_mount_media (BraseroBurn *self,
 	medium = brasero_drive_get_medium (priv->dest);
 	while (!brasero_volume_is_mounted (BRASERO_VOLUME (medium))) {
 		if (retries++ > MAX_MOUNT_ATTEMPTS) {
+			/* FIXME: there is a spelling mistake */
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
-				     _("the disc could not be mounted (max attemps reached)"));
+				     _("The disc could not be mounted (max attemps reached)"));
 			return BRASERO_BURN_ERR;
 		}
 
@@ -1017,11 +1058,11 @@ again:
 			return result;
 	}
 	else if (media == BRASERO_MEDIUM_BUSY)
-		error_type = BRASERO_BURN_ERROR_MEDIA_BUSY;
+		error_type = BRASERO_BURN_ERROR_DRIVE_BUSY;
 	else if (media == BRASERO_MEDIUM_UNSUPPORTED)
-		error_type = BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED;
+		error_type = BRASERO_BURN_ERROR_MEDIUM_INVALID;
 	else if (media & BRASERO_MEDIUM_BLANK)
-		error_type = BRASERO_BURN_ERROR_MEDIA_BLANK;
+		error_type = BRASERO_BURN_ERROR_MEDIUM_NO_DATA;
 
 	if (error_type != BRASERO_BURN_ERROR_NONE) {
 		result = brasero_burn_ask_for_dest_media (burn,
@@ -1035,11 +1076,11 @@ again:
 	}
 
 	if (!priv->dest_locked
-	&&  !brasero_drive_lock (priv->dest, _("ongoing checksuming operation"), &failure)) {
+	&&  !brasero_drive_lock (priv->dest, _("Ongoing checksuming operation"), &failure)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the drive can't be locked (%s)"),
+			     _("The drive cannot be locked (%s)"),
 			     failure);
 		return BRASERO_BURN_ERR;
 	}
@@ -1141,7 +1182,8 @@ brasero_burn_progress_changed (BraseroTaskCtx *task,
 				   (gdouble) priv->task_nb;
 	}
 	else
-		overall_progress =  (gdouble) priv->tasks_done / (gdouble) priv->task_nb;
+		overall_progress =  (gdouble) priv->tasks_done /
+				    (gdouble) priv->task_nb;
 
 	g_signal_emit (burn,
 		       brasero_burn_signals [PROGRESS_CHANGED_SIGNAL],
@@ -1166,40 +1208,7 @@ brasero_burn_action_changed (BraseroTask *task,
 			     BraseroBurnAction action,
 			     BraseroBurn *burn)
 {
-	BraseroBurnPrivate *priv;
-	BraseroMedia media;
-
 	brasero_burn_action_changed_real (burn, action);
-
-	if (action != BRASERO_BURN_ACTION_START_RECORDING)
-		return;
-
-	priv = BRASERO_BURN_PRIVATE (burn);
-
-	media = brasero_burn_session_get_dest_media (priv->session);
-	if ((BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
-	||   BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED))) {
-		BraseroBurnFlag flags;
-		BraseroMedium *medium;
-		BraseroDrive *drive;
-		gint64 len = 0;
-
-		drive = brasero_burn_session_get_burner (priv->session);
-		medium = brasero_drive_get_medium (drive);
-		flags = brasero_burn_session_get_flags (priv->session);
-
-		/* we need to save some parameters for later checksuming */
-		brasero_task_ctx_get_session_output_size (BRASERO_TASK_CTX (priv->task),
-							  &len,
-							  NULL);
-
-		if (flags & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND))
-			priv->session_start = brasero_medium_get_next_writable_address (medium);
-		else
-			priv->session_start = 0;
-
-		priv->session_end = priv->session_start + len;
-	}
 }
 
 void
@@ -1313,8 +1322,10 @@ brasero_burn_run_eraser (BraseroBurn *burn, GError **error)
 	&& !brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_BUSY_DRIVE,
-			     _("the drive seems to be busy"));
+			     BRASERO_BURN_ERROR_DRIVE_BUSY,
+			     "%s. %s",
+			     _("The drive is busy"),
+			     _("Make sure another application is not using it"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -1339,18 +1350,20 @@ start:
 
 	medium = brasero_drive_get_medium (src);
 
-	/* this is just in case */
+	/* This is just in case */
 	if (medium
 	&&  brasero_volume_is_mounted (BRASERO_VOLUME (medium))
 	&& !brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_BUSY_DRIVE,
-			     _("the drive seems to be busy"));
+			     BRASERO_BURN_ERROR_DRIVE_BUSY,
+			     "%s. %s",
+			     _("The drive is busy"),
+			     _("Make sure another application is not using it"));
 		return BRASERO_BURN_ERR;
 	}
 
-	/* if it succeeds then the new track(s) will be at the top of
+	/* If it succeeds then the new track(s) will be at the top of
 	 * session tracks stack and therefore usable by the recorder.
 	 * NOTE: it's up to the job to push the current tracks. */
 	if (fake)
@@ -1380,9 +1393,27 @@ start:
 	if (!ret_error)
 		return result;
 
+	if (brasero_burn_session_is_dest_file (priv->session)) {
+		gchar *image = NULL;
+		gchar *toc = NULL;
+
+		/* If it was an image that was output, remove it. If that was
+		 * a temporary image, it will be removed by BraseroBurnSession 
+		 * object. But if it was a final image, it would be left and
+		 * would clutter the disk, wasting space. */
+		brasero_burn_session_get_output (priv->session,
+						 &image,
+						 &toc,
+						 NULL);
+		if (image)
+			g_remove (image);
+		if (toc)
+			g_remove (toc);
+	}
+
 	/* See if we can recover from the error */
 	error_code = ret_error->code;
-	if (error_code == BRASERO_BURN_ERROR_JOLIET_TREE) {
+	if (error_code == BRASERO_BURN_ERROR_IMAGE_JOLIET) {
 		/* clean the error anyway since at worst the user will cancel */
 		g_error_free (ret_error);
 		ret_error = NULL;
@@ -1395,7 +1426,7 @@ start:
 
 		goto start;
 	}
-	else if (error_code == BRASERO_BURN_ERROR_MEDIA_BLANK) {
+	else if (error_code == BRASERO_BURN_ERROR_MEDIUM_NO_DATA) {
 		/* clean the error anyway since at worst the user will cancel */
 		g_error_free (ret_error);
 		ret_error = NULL;
@@ -1409,14 +1440,39 @@ start:
 
 		goto start;
 	}
-	/* (error_code == BRASERO_BURN_ERROR_MEDIA_SPACE) */
-	/* That's an imager (outputs an image to the disc) so that means that here
-	 * the problem comes from the hard drive being too small not from the media
-	 * there is nothing we can do here except fail. We could one day send a 
-	 * signal so that a dialog asking for a new hard drive location is shown
-	 */
+	else if (error_code == BRASERO_BURN_ERROR_DISK_SPACE
+	     ||  error_code == BRASERO_BURN_ERROR_PERMISSION) {
+		gboolean is_temp;
 
-	/* not recoverable propagate the error */
+		/* That's an imager (outputs an image to the disc) so that means
+		 * that here the problem comes from the hard drive being too
+		 * small or we don't have the right permission. */
+
+		/* NOTE: Image file creation is always the last to take place 
+		 * when it's not temporary. Another job should not take place
+		 * afterwards */
+		if (!brasero_burn_session_is_dest_file (priv->session))
+			is_temp = TRUE;
+		else
+			is_temp = FALSE;
+
+		result = brasero_burn_ask_for_location (burn,
+							ret_error,
+							is_temp,
+							error);
+
+		/* clean the error anyway since at worst the user will cancel */
+		g_error_free (ret_error);
+		ret_error = NULL;
+
+		if (result != BRASERO_BURN_OK)
+			return result;
+
+		goto start;
+	}
+
+	/* If we reached this point that means the error was not recoverable.
+	 * Propagate the error. */
 	if (error && ret_error)
 		g_propagate_error (error, ret_error);
 
@@ -1482,8 +1538,10 @@ start:
 	&& !brasero_volume_umount (BRASERO_VOLUME (src_medium), TRUE, NULL)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_BUSY_DRIVE,
-			     _("the drive seems to be busy"));
+			     BRASERO_BURN_ERROR_DRIVE_BUSY,
+			     "%s. %s",
+			     _("The drive is busy"),
+			     _("Make sure another application is not using it"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -1491,8 +1549,10 @@ start:
 	&& !brasero_volume_umount (BRASERO_VOLUME (burnt_medium), TRUE, NULL)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_BUSY_DRIVE,
-			     _("the drive seems to be busy"));
+			     BRASERO_BURN_ERROR_DRIVE_BUSY,
+			     "%s. %s",
+			     _("The drive is busy"),
+			     _("Make sure another application is not using it"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -1521,7 +1581,7 @@ start:
 
 	/* see if error is recoverable */
 	error_code = ret_error->code;
-	if (error_code == BRASERO_BURN_ERROR_JOLIET_TREE) {
+	if (error_code == BRASERO_BURN_ERROR_IMAGE_JOLIET) {
 		/* NOTE: this error can only come from the source when 
 		 * burning on the fly => no need to recreate an imager */
 
@@ -1539,7 +1599,7 @@ start:
 		ret_error = NULL;
 		goto start;
 	}
-	else if (error_code == BRASERO_BURN_ERROR_MEDIA_BLANK) {
+	else if (error_code == BRASERO_BURN_ERROR_MEDIUM_NEED_RELOADING) {
 		/* NOTE: this error can only come from the source when 
 		 * burning on the fly => no need to recreate an imager */
 
@@ -1583,7 +1643,7 @@ start:
 		brasero_burn_session_set_rate (priv->session, rate);
 		goto start;
 	}
-	else if (error_code == BRASERO_BURN_ERROR_MEDIA_SPACE) {
+	else if (error_code == BRASERO_BURN_ERROR_MEDIUM_SPACE) {
 		/* NOTE: this error can only come from the dest drive */
 
 		/* clean error and indicates this is a recoverable error */
@@ -1603,8 +1663,10 @@ start:
 		   (BRASERO_BURN_FLAG_APPEND|BRASERO_BURN_FLAG_MERGE)) {
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_MEDIA_SPACE,
-				     _("it's not possible to merge to this media because it hasn't got enough space"));
+				     BRASERO_BURN_ERROR_MEDIUM_SPACE,
+				     "%s. %s",
+				     _("Merging data is impossible with this disc"),
+				     _("Not enough space available on the disc"));
 			return BRASERO_BURN_ERR;
 		}
 
@@ -1617,8 +1679,8 @@ start:
 
 		goto start;
 	}
-	else if (error_code > BRASERO_BURN_ERROR_MEDIA_SPACE
-	     &&  error_code <  BRASERO_BURN_ERROR_CD_NOT_SUPPORTED) {
+	else if (error_code >= BRASERO_BURN_ERROR_MEDIUM_NONE
+	     &&  error_code <=  BRASERO_BURN_ERROR_MEDIUM_NEED_RELOADING) {
 		/* NOTE: these errors can only come from the dest drive */
 
 		/* clean error and indicates this is a recoverable error */
@@ -1722,6 +1784,31 @@ brasero_burn_run_tasks (BraseroBurn *burn,
 		if (result != BRASERO_BURN_OK)
 			break;
 
+		/* try to get the output size */
+		if (BRASERO_MEDIUM_RANDOM_WRITABLE (brasero_burn_session_get_dest_media (priv->session))) {
+			gint64 len = 0;
+			BraseroDrive *drive;
+			BraseroMedium *medium;
+
+			brasero_task_ctx_get_session_output_size (BRASERO_TASK_CTX (priv->task),
+								  &len,
+								  NULL);
+
+			drive = brasero_burn_session_get_burner (priv->session);
+			medium = brasero_drive_get_medium (drive);
+
+			if (brasero_burn_session_get_flags (priv->session) & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND))
+				priv->session_start = brasero_medium_get_next_writable_address (medium);
+			else
+				priv->session_start = 0;
+
+			priv->session_end = priv->session_start + len;
+
+			BRASERO_BURN_LOG ("Burning from %lld to %lld",
+					  priv->session_start,
+					  priv->session_end);
+		}
+
 		/* see if we reached a recording task: it's the last task */
 		if (!next) {
 			if (brasero_burn_session_is_dest_file (priv->session))
@@ -1813,8 +1900,10 @@ brasero_burn_check_real (BraseroBurn *self,
 		&& !brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL)) {
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_BUSY_DRIVE,
-				     _("the drive seems to be busy"));
+				     BRASERO_BURN_ERROR_DRIVE_BUSY,
+				     "%s. %s",
+				     _("The drive is busy"),
+				     _("Make sure another application is not using it"));
 			return BRASERO_BURN_ERR;
 		}
 
@@ -1834,7 +1923,7 @@ brasero_burn_check_real (BraseroBurn *self,
 		priv->task = NULL;
 	}
 	else {
-		BRASERO_BURN_LOG ("the track can't be checked");
+		BRASERO_BURN_LOG ("the track cannot be checked");
 		result = BRASERO_BURN_NOT_SUPPORTED;
 	}
 
@@ -1865,7 +1954,7 @@ brasero_burn_check_session_consistency (BraseroBurn *burn,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("there is no track to be burnt"));
+			     _("There is no track to be burnt"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -1875,19 +1964,14 @@ brasero_burn_check_session_consistency (BraseroBurn *burn,
 
 		burner = brasero_burn_session_get_burner (priv->session);
 		if (!burner) {
-			BRASERO_BURN_DEBUG (burn, "No drive set");
+			BRASERO_BURN_DEBUG (burn, "No burner specified.");
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
-				     _("there is no drive to burn to"));
+				     BRASERO_BURN_ERROR_OUTPUT_NONE,
+				     _("No burner specified"));
 			return BRASERO_BURN_ERR;	
 		}
 	}
-	else {
-		/* check number of copies must be 1 */
-		if (brasero_burn_session_get_num_copies (priv->session) != 1)
-			brasero_burn_session_set_num_copies (priv->session, 1);
-	}
 
 	media = brasero_burn_session_get_dest_media (priv->session);
 
@@ -1933,15 +2017,12 @@ brasero_burn_check_session_consistency (BraseroBurn *burn,
 				g_set_error (error,
 					     BRASERO_BURN_ERROR,
 					     BRASERO_BURN_ERROR_GENERAL,
-					     _("merging data is impossible with this disc"));
+					     _("Merging data is impossible with this disc"));
 				return BRASERO_BURN_ERR;
 			}
-			else if ((flag & BRASERO_BURN_FLAG_BURNPROOF)
-			     && !(BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
-			     ||   BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED))) {
-				 /* Warn the user BURNPROOF won't be possible */
-
-			}
+			/* No need to tell the user burnproof is not supported
+			 * as these drives handle errors differently which makes
+			 * burnproof useless for them. */
 		}
 	}
 
@@ -2127,13 +2208,17 @@ brasero_burn_record_session (BraseroBurn *burn,
 	/* this may be necessary for the drive to settle down and possibly be
 	 * mounted by gnome-volume-manager (just temporarily) */
 	result = brasero_burn_sleep (burn, 5000);
-	if (result != BRASERO_BURN_OK)
+	if (result != BRASERO_BURN_OK) {
+		brasero_burn_session_pop_tracks (priv->session);
 		return result;
+	}
 
 	/* reprobe the medium and wait for it to be probed */
 	result = brasero_burn_reprobe (burn);
-	if (result != BRASERO_BURN_OK)
+	if (result != BRASERO_BURN_OK) {
+		brasero_burn_session_pop_tracks (priv->session);
 		return result;
+	}
 
 	medium = brasero_drive_get_medium (priv->dest);
 
@@ -2147,8 +2232,7 @@ brasero_burn_record_session (BraseroBurn *burn,
 		 * number for these drives. */
 		media = brasero_medium_get_status (medium);
 
-		if (!BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
-		&&  !BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
+		if (!BRASERO_MEDIUM_RANDOM_WRITABLE (media)) {
 			guint track_num;
 
 			track_num = brasero_medium_get_track_num (medium);
@@ -2217,7 +2301,7 @@ brasero_burn_check (BraseroBurn *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("only one track at a time can be checked"));
+			     _("Only one track at a time can be checked"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -2260,6 +2344,7 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 	gchar *image = NULL;
 	BraseroTrack *track;
 	BraseroTrackType output;
+	GError *ret_error = NULL;
 	BraseroBurnResult result;
 	BraseroBurnPrivate *priv;
 	BraseroImageFormat format;
@@ -2285,7 +2370,7 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("impossible to find a format for the temporary image"));
+			     _("No format for the temporary image could be found"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -2295,9 +2380,48 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 						     format,
 						     &image,
 						     &toc,
-						     error);
-	if (result != BRASERO_BURN_OK)
-		return result;
+						     &ret_error);
+	while (result != BRASERO_BURN_OK) {
+		gboolean is_temp;
+
+		if (!ret_error
+		||  (ret_error->code != BRASERO_BURN_ERROR_DISK_SPACE
+		&&   ret_error->code != BRASERO_BURN_ERROR_PERMISSION)) {
+			g_propagate_error (error, ret_error);
+			return result;
+		}
+
+		/* That's an imager (outputs an image to the disc) so that means
+		 * that here the problem comes from the hard drive being too
+		 * small or we don't have the right permission. */
+
+		/* NOTE: Image file creation is always the last to take place 
+		 * when it's not temporary. Another job should not take place
+		 * afterwards */
+		if (!brasero_burn_session_is_dest_file (priv->session))
+			is_temp = TRUE;
+		else
+			is_temp = FALSE;
+
+		result = brasero_burn_ask_for_location (self,
+							ret_error,
+							is_temp,
+							error);
+
+		/* clean the error anyway since at worst the user will cancel */
+		g_error_free (ret_error);
+		ret_error = NULL;
+
+		if (result != BRASERO_BURN_OK)
+			return result;
+
+		/* retry */
+		result = brasero_burn_session_get_tmp_image (priv->session,
+							     format,
+							     &image,
+							     &toc,
+							     &ret_error);
+	}
 
 	/* some, like cdrdao, can't overwrite the files */
 	g_remove (image);
@@ -2399,7 +2523,7 @@ again:
 
 	if (result != BRASERO_BURN_OK) {
 		/* Tell the user his/her disc is not supported and reload */
-		berror = BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED;
+		berror = BRASERO_BURN_ERROR_MEDIUM_INVALID;
 		brasero_burn_session_set_flags (priv->session, session_flags);
 		goto again;
 	}
@@ -2451,7 +2575,7 @@ brasero_burn_record (BraseroBurn *burn,
 			goto end;
 	}
 	else if (!brasero_burn_session_is_dest_file (session)) {
-		BraseroBurnError berror;
+		BraseroBurnError berror = BRASERO_BURN_ERROR_NONE;
 
 		/* do some drive locking quite early to make sure we have a
 		 * media in the drive so that we'll have all the necessary
@@ -2469,7 +2593,7 @@ brasero_burn_record (BraseroBurn *burn,
 								  berror,
 								  required_media,
 								  error);
-			if (result == BRASERO_BURN_OK)
+			if (result != BRASERO_BURN_OK)
 				goto end;
 
 			result = brasero_burn_lock_dest_media (burn, &berror, error);
@@ -2485,34 +2609,8 @@ brasero_burn_record (BraseroBurn *burn,
 			goto end;
 	}
 
-	/* burn the session a first time whatever the number of copies required 
-	 * except if dummy session */
+	/* burn the session except if dummy session */
 	result = brasero_burn_record_session (burn, TRUE, error);
-	if (result == BRASERO_BURN_OK) {
-		gint num_copies;
-
-		/* burn all other required copies */
-		num_copies = brasero_burn_session_get_num_copies (session);
-		while (--num_copies > 0 && result == BRASERO_BURN_OK) {
-			BRASERO_BURN_LOG ("Burning additional copies (%i left)",
-					  num_copies);
-
-			/* we only need to reload and lock dest media */
-			result = brasero_burn_reload_dest_media (burn,
-								 BRASERO_BURN_WARNING_NEXT_COPY,
-								 error);
-			if (result != BRASERO_BURN_OK)
-				break;
-
-			/* see if we still need it to be locked */
-			if (brasero_burn_session_get_input_type (session, NULL) != BRASERO_TRACK_TYPE_DISC)
-				brasero_burn_unlock_src_media (burn, NULL);
-
-			result = brasero_burn_record_session (burn, TRUE, error);
-			if (result != BRASERO_BURN_OK)
-				break;
-		}
-	}
 
 end:
 
@@ -2525,12 +2623,13 @@ end:
 	&& (result == BRASERO_BURN_NOT_READY
 	||  result == BRASERO_BURN_NOT_SUPPORTED
 	||  result == BRASERO_BURN_RUNNING
-	||  result == BRASERO_BURN_NOT_RUNNING))
+	||  result == BRASERO_BURN_NOT_RUNNING)) {
+		BRASERO_BURN_LOG ("Internal error with result %i", result);
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("internal error (code %i)"),
-			     result);
+			     _("An internal error occured"));
+	}
 
 	if (result == BRASERO_BURN_CANCEL) {
 		BRASERO_BURN_DEBUG (burn, "Session cancelled by user");
@@ -2616,12 +2715,12 @@ brasero_burn_blank (BraseroBurn *burn,
 	result = brasero_burn_blank_real (burn, &ret_error);
 	while (result == BRASERO_BURN_ERR
 	&&     ret_error
-	&&     ret_error->code == BRASERO_BURN_ERROR_MEDIA_NOT_REWRITABLE) {
+	&&     ret_error->code == BRASERO_BURN_ERROR_MEDIUM_NOT_REWRITABLE) {
 		g_error_free (ret_error);
 		ret_error = NULL;
 
 		result = brasero_burn_ask_for_dest_media (burn,
-							  BRASERO_BURN_ERROR_MEDIA_NOT_REWRITABLE,
+							  BRASERO_BURN_ERROR_MEDIUM_NOT_REWRITABLE,
 							  BRASERO_MEDIUM_REWRITABLE|
 							  BRASERO_MEDIUM_HAS_DATA,
 							  error);
@@ -2768,7 +2867,7 @@ brasero_burn_class_init (BraseroBurnClass *klass)
 			      NULL, NULL,
 			      brasero_marshal_INT__VOID,
 			      G_TYPE_INT, 0);
-        brasero_burn_signals [INSERT_MEDIA_REQUEST_SIGNAL] =
+	brasero_burn_signals [INSERT_MEDIA_REQUEST_SIGNAL] =
 		g_signal_new ("insert_media",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
@@ -2781,7 +2880,19 @@ brasero_burn_class_init (BraseroBurnClass *klass)
 			      BRASERO_TYPE_DRIVE,
 			      G_TYPE_INT,
 			      G_TYPE_INT);
-        brasero_burn_signals [PROGRESS_CHANGED_SIGNAL] =
+	brasero_burn_signals [LOCATION_REQUEST_SIGNAL] =
+		g_signal_new ("location-request",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (BraseroBurnClass,
+					       location_request),
+			      NULL, NULL,
+			      brasero_marshal_INT__POINTER_BOOLEAN,
+			      G_TYPE_INT, 
+			      2,
+			      G_TYPE_POINTER,
+			      G_TYPE_INT);
+	brasero_burn_signals [PROGRESS_CHANGED_SIGNAL] =
 		g_signal_new ("progress_changed",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
