@@ -184,8 +184,9 @@ brasero_cdrdao_read_stderr_record (BraseroCdrdao *cdrdao, const gchar *line)
 
 		brasero_job_error (BRASERO_JOB (cdrdao),
 				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						_("the cue file (%s) seems to be invalid"),
+						BRASERO_BURN_ERROR_FILE_NOT_FOUND,
+						/* Translators: %s is a filename */
+						_("\"%s\" could not be found"),
 						name));
 		g_free (name);
 	}
@@ -216,20 +217,20 @@ brasero_cdrdao_read_stderr (BraseroProcess *process, const gchar *line)
 	if (strstr (line, "Cannot setup device")) {
 		brasero_job_error (BRASERO_JOB (cdrdao),
 				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_BUSY_DRIVE,
-						_("the drive seems to be busy")));
+						BRASERO_BURN_ERROR_DRIVE_BUSY,
+						_("The drive is busy")));
 	}
 	else if (strstr (line, "Illegal command")) {
 		brasero_job_error (BRASERO_JOB (cdrdao),
 				   g_error_new (BRASERO_BURN_ERROR,
 						BRASERO_BURN_ERROR_GENERAL,
-						_("your version of cdrdao doesn't seem to be supported by libbrasero")));
+						_("Your version of cdrdao does not seem to be supported by libbrasero")));
 	}
 	else if (strstr (line, "Operation not permitted. Cannot send SCSI")) {
 		brasero_job_error (BRASERO_JOB (cdrdao),
 				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_SCSI_IOCTL,
-						_("You don't seem to have the required permission to use this drive")));
+						BRASERO_BURN_ERROR_PERMISSION,
+						_("You do not have the required permissions to use this drive")));
 	}
 
 	return BRASERO_BURN_OK;
@@ -549,7 +550,7 @@ brasero_cdrdao_export_caps (BraseroPlugin *plugin, gchar **error)
 {
 	GSList *input;
 	GSList *output;
-	gchar *prog_name;
+	BraseroBurnResult result;
 	const BraseroMedia media_w = BRASERO_MEDIUM_CD|
 				     BRASERO_MEDIUM_WRITABLE|
 				     BRASERO_MEDIUM_REWRITABLE|
@@ -563,19 +564,15 @@ brasero_cdrdao_export_caps (BraseroPlugin *plugin, gchar **error)
 				      BRASERO_MEDIUM_BLANK;
 
 	brasero_plugin_define (plugin,
-			       "cdrdao",
-			       _("use cdrdao to image and burn CDs"),
+			       "Cdrdao",
+			       _("Use cdrdao to copy and burn CDs"),
 			       "Philippe Rouquier",
 			       20);
 
-	/* First see if this plugin can be used, i.e. if readcd is in
-	 * the path */
-	prog_name = g_find_program_in_path ("cdrdao");
-	if (!prog_name) {
-		*error = g_strdup (_("cdrdao could not be found in the path"));
-		return BRASERO_BURN_ERR;
-	}
-	g_free (prog_name);
+	/* First see if this plugin can be used */
+	result = brasero_process_check_path ("cdrdao", error);
+	if (result != BRASERO_BURN_OK)
+		return result;
 
 	/* that's for cdrdao images: CDs only as input */
 	input = brasero_caps_disc_new (BRASERO_MEDIUM_CD|
