@@ -109,7 +109,7 @@ brasero_checksum_files_get_file_checksum (BraseroChecksumFiles *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the file %s couldn't be read (%s)"),
+			     _("File \"%s\" could not be opened (%s)"),
 			     name,
 			     g_strerror (errsv));
 		g_free (name);
@@ -171,7 +171,7 @@ brasero_checksum_files_add_file_checksum (BraseroChecksumFiles *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the md5 file couldn't be written to (%s)"),
+			     _("Data could not be written (%s)"),
 			     g_strerror (errsv));
 			
 		return BRASERO_BURN_ERR;
@@ -194,7 +194,7 @@ brasero_checksum_files_add_file_checksum (BraseroChecksumFiles *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the md5 file couldn't be written to (%s)"),
+			     _("Data could not be written (%s)"),
 			     g_strerror (errsv));
 
 		return BRASERO_BURN_ERR;
@@ -261,6 +261,13 @@ brasero_checksum_files_explore_directory (BraseroChecksumFiles *self,
 			continue;
 		}
 
+		/* Only checksum regular files and avoid fifos, ... */
+		if (!g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
+			g_free (path);
+			g_free (graft_path);
+			continue;
+		}
+
 		result = brasero_checksum_files_add_file_checksum (self,
 								   path,
 								   checksum_type,
@@ -541,16 +548,18 @@ brasero_checksum_files_create_checksum (BraseroChecksumFiles *self,
 		break;
 	}
 
-	if (result != BRASERO_BURN_OK)
+	if (result != BRASERO_BURN_OK || !priv->sums_path)
 		return result;
 
 	priv->file = fopen (priv->sums_path, "w");
 	if (!priv->file) {
                 int errsv = errno;
+
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("md5 file couldn't be opened (%s)"),
+			     _("File \"%s\" could not be opened (%s)"),
+			     priv->sums_path,
 			     g_strerror (errsv));
 
 		return BRASERO_BURN_ERR;
@@ -731,7 +740,12 @@ brasero_checksum_files_check_files (BraseroChecksumFiles *self,
 
 	/* we need to get the number of files at this time and rewind */
 	file_nb = brasero_checksum_files_get_line_num (self, file, error);
-	if (file_nb < 1) {
+	if (file_nb == 0) {
+		fclose (file);
+		return BRASERO_BURN_OK;
+	}
+
+	if (file_nb < 0) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
@@ -847,6 +861,11 @@ brasero_checksum_files_check_files (BraseroChecksumFiles *self,
 
 		filename [i] = 0;
 		checksum_real = NULL;
+
+		/* we certainly don't want to checksum anything but regular file */
+		if (!g_file_test (filename, G_FILE_TEST_IS_REGULAR))
+			continue;
+
 		result = brasero_checksum_files_get_file_checksum (self,
 								   gchecksum_type,
 								   filename,
@@ -906,7 +925,7 @@ end:
 	g_set_error (error,
 		     BRASERO_BURN_ERROR,
 		     BRASERO_BURN_ERROR_BAD_CHECKSUM,
-		     _("some files may be corrupted on the disc"));
+		     _("Some files may be corrupted on the disc"));
 
 	return BRASERO_BURN_ERR;
 }
@@ -1024,7 +1043,7 @@ error:
 
 	error = g_error_new (BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_BAD_CHECKSUM,
-			     _("some files may be corrupted on the disc"));
+			     _("Some files may be corrupted on the disc"));
 	brasero_job_error (BRASERO_JOB (self), error);
 	return FALSE;
 }
@@ -1260,7 +1279,7 @@ brasero_checksum_files_export_caps (BraseroPlugin *plugin, gchar **error)
 
 	brasero_plugin_define (plugin,
 			       "File checksum",
-			       _("allows to check file integrities on a disc"),
+			       _("Allows to check file integrities on a disc"),
 			       "Philippe Rouquier",
 			       0);
 
