@@ -226,7 +226,7 @@ brasero_local_track_recursive_transfer (BraseroLocalTrack *self,
 				g_set_error (error,
 					     BRASERO_BURN_ERROR,
 					     BRASERO_BURN_ERROR_GENERAL,
-					     _("a directory couldn't be created (%s)"),
+					     _("Directory could not be created (%s)"),
 					     g_strerror (errsv));
 				result = BRASERO_BURN_ERR;
 			}
@@ -316,7 +316,7 @@ brasero_local_track_transfer (BraseroLocalTrack *self,
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
-				     _("a directory couldn't be created (%s)"),
+				     _("Directory could not be created (%s)"),
 				     g_strerror (errsv));
 			return BRASERO_BURN_ERR;
 		}
@@ -1066,7 +1066,7 @@ brasero_local_track_export_caps (BraseroPlugin *plugin, gchar **error)
 
 	brasero_plugin_define (plugin,
 			       "local-track",
-			       _("local-track allows to burn files not stored locally"),
+			       _("Local-track allows to burn files not stored locally"),
 			       "Philippe Rouquier",
 			       10);
 
@@ -1085,10 +1085,24 @@ brasero_local_track_export_caps (BraseroPlugin *plugin, gchar **error)
 				       BRASERO_AUDIO_FORMAT_AC3|
 				       BRASERO_AUDIO_FORMAT_MP2|
 				       BRASERO_AUDIO_FORMAT_44100|
-				       BRASERO_AUDIO_FORMAT_48000);
+				       BRASERO_AUDIO_FORMAT_48000|
+				       BRASERO_METADATA_INFO);
 	brasero_plugin_process_caps (plugin, caps);
 	g_slist_free (caps);
 
+	caps = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_FILE,
+				       BRASERO_AUDIO_FORMAT_UNDEFINED|
+				       BRASERO_AUDIO_FORMAT_4_CHANNEL|
+				       BRASERO_AUDIO_FORMAT_RAW|
+				       BRASERO_VIDEO_FORMAT_UNDEFINED|
+				       BRASERO_VIDEO_FORMAT_VCD|
+				       BRASERO_VIDEO_FORMAT_VIDEO_DVD|
+				       BRASERO_AUDIO_FORMAT_AC3|
+				       BRASERO_AUDIO_FORMAT_MP2|
+				       BRASERO_AUDIO_FORMAT_44100|
+				       BRASERO_AUDIO_FORMAT_48000);
+	brasero_plugin_process_caps (plugin, caps);
+	g_slist_free (caps);
 	caps = brasero_caps_data_new (BRASERO_IMAGE_FS_ANY);
 	brasero_plugin_process_caps (plugin, caps);
 	g_slist_free (caps);
