@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -243,11 +245,11 @@ brasero_transcode_create_volume (BraseroTranscode *transcode,
 	||  brasero_track_tag_lookup (track, BRASERO_TRACK_GAIN_VALUE, NULL) == BRASERO_BURN_OK) {
 		BRASERO_JOB_LOG (transcode, "Found audio levels tags");
 		volume = gst_element_factory_make ("rgvolume", NULL);
-		g_object_set (volume,
-			      "album-mode", FALSE,
-			      NULL);
-
-		if (!volume)
+		if (volume)
+			g_object_set (volume,
+				      "album-mode", FALSE,
+				      NULL);
+		else
 			BRASERO_JOB_LOG (transcode, "rgvolume object couldn't be created");
 	}
 
@@ -312,7 +314,10 @@ brasero_transcode_create_pipeline (BraseroTranscode *transcode,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("source can't be created"));
+			     /* Translators: %s is the name of the GstElement that 
+			      * could not be created */
+			     _("%s element could not be created"),
+			     "\"Source\"");
 		goto error;
 	}
 	gst_bin_add (GST_BIN (pipeline), source);
@@ -360,7 +365,8 @@ brasero_transcode_create_pipeline (BraseroTranscode *transcode,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("sink can't be created"));
+			     _("%s element could not be created"),
+			     "\"Sink\"");
 		goto error;
 	}
 
@@ -375,7 +381,8 @@ brasero_transcode_create_pipeline (BraseroTranscode *transcode,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("audioconvert can't be created"));
+			     _("%s element could not be created"),
+			     "\"Audioconvert\"");
 		goto error;
 	}
 	gst_bin_add (GST_BIN (pipeline), convert);
@@ -387,7 +394,8 @@ brasero_transcode_create_pipeline (BraseroTranscode *transcode,
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
-				     _("audioresample can't be created"));
+				     _("%s element could not be created"),
+				     "\"Audioresample\"");
 			goto error;
 		}
 		gst_bin_add (GST_BIN (pipeline), resample);
@@ -398,7 +406,8 @@ brasero_transcode_create_pipeline (BraseroTranscode *transcode,
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
-				     _("filter can't be created"));
+				     _("%s element could not be created"),
+				     "\"Filter\"");
 			goto error;
 		}
 		gst_bin_add (GST_BIN (pipeline), filter);
@@ -421,7 +430,8 @@ brasero_transcode_create_pipeline (BraseroTranscode *transcode,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("decode can't be created"));
+			     _("%s element could not be created"),
+			     "\"Decodebin\"");
 		goto error;
 	}
 	gst_bin_add (GST_BIN (pipeline), decode);
@@ -575,17 +585,23 @@ brasero_transcode_create_sibling_image (BraseroTranscode *transcode,
 	brasero_job_get_audio_output (BRASERO_JOB (transcode), &path_dest);
 
 	if (symlink (path_src, path_dest) == -1) {
+                int errsv = errno;
+
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("a symlink could not be created (%s)"),
-			     strerror (errno));
+			     /* Translators: the %s is the error message from errno */
+			     _("An internal error occured (%s)"),
+			     g_strerror (errsv));
 
 		goto error;
 	}
 
 	dest = brasero_track_new (BRASERO_TRACK_TYPE_AUDIO);
-	brasero_track_set_audio_source (dest, path_dest, BRASERO_AUDIO_FORMAT_RAW);
+	brasero_track_set_audio_source (dest,
+					path_dest,
+					BRASERO_AUDIO_FORMAT_RAW|
+					BRASERO_AUDIO_FORMAT_44100);
 
 	/* NOTE: there is no gap and start = 0 since these tracks are the result
 	 * of the transformation of previous ones */
@@ -828,11 +844,14 @@ brasero_transcode_pad_real (BraseroTranscode *transcode,
 		}
 
 		if (size != b_written) {
+                        int errsv = errno;
+
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
-				     _("error padding (%s)"),
-				     strerror (errno));
+				     /* Translators: %s is the string error from errno */
+				     _("Error while padding file (%s)"),
+				     g_strerror (errsv));
 			return -1;
 		}
 	}
@@ -860,7 +879,10 @@ brasero_transcode_push_track (BraseroTranscode *transcode)
 	brasero_job_get_output_type (BRASERO_JOB (transcode), &type);
 	track = brasero_track_new (BRASERO_TRACK_TYPE_AUDIO);
 
-	brasero_track_set_audio_source (track, output, BRASERO_AUDIO_FORMAT_RAW);
+	brasero_track_set_audio_source (track,
+					output,
+					BRASERO_AUDIO_FORMAT_RAW|
+					BRASERO_AUDIO_FORMAT_44100);
 	brasero_track_set_audio_boundaries (track, 0, length, 0);
 	brasero_track_set_audio_info (track, info);
 
@@ -1011,15 +1033,18 @@ brasero_transcode_pad_file (BraseroTranscode *transcode, GError **error)
 
 	output = NULL;
 	brasero_job_get_audio_output (BRASERO_JOB (transcode), &output);
-	fd = open (output, O_WRONLY | O_CREAT | O_APPEND);
+	fd = open (output, O_WRONLY | O_CREAT | O_APPEND, S_IRWXU | S_IRGRP | S_IROTH);
 	g_free (output);
 
 	if (fd == -1) {
+                int errsv = errno;
+
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("error opening file for padding : %s"),
-			     strerror (errno));
+			     /* Translators: %s is the string error from errno */
+			     _("Error while padding file (%s)"),
+			     g_strerror (errsv));
 		return FALSE;
 	}
 
@@ -1115,7 +1140,7 @@ brasero_transcode_get_duration (BraseroTranscode *transcode)
 	    brasero_job_error (BRASERO_JOB (transcode),
 			       g_error_new (BRASERO_BURN_ERROR,
 					    BRASERO_BURN_ERROR_GENERAL,
-					    _("error getting duration")));
+					    _("Error while getting duration")));
 	return duration;
 }
 
@@ -1178,18 +1203,18 @@ foreach_tag (const GstTagList *list,
 	brasero_job_get_current_track (BRASERO_JOB (transcode), &track);
 	info = brasero_track_get_audio_info (track);
 
-	if (!strcmp (tag, GST_TAG_TITLE)) {
+	if (info && !strcmp (tag, GST_TAG_TITLE)) {
 		if (!info->title)
 			gst_tag_list_get_string (list, tag, &(info->title));
 	}
-	else if (!strcmp (tag, GST_TAG_ARTIST)) {
+	else if (info && !strcmp (tag, GST_TAG_ARTIST)) {
 		if (!info->artist)
 			gst_tag_list_get_string (list, tag, &(info->artist));
 	}
-	else if (!strcmp (tag, GST_TAG_ISRC)) {
+	else if (info && !strcmp (tag, GST_TAG_ISRC)) {
 		gst_tag_list_get_int (list, tag, &(info->isrc));
 	}
-	else if (!strcmp (tag, GST_TAG_PERFORMER)) {
+	else if (info && !strcmp (tag, GST_TAG_PERFORMER)) {
 		if (!info->artist)
 			gst_tag_list_get_string (list, tag, &(info->artist));
 	}
@@ -1339,37 +1364,114 @@ brasero_transcode_bus_messages (GstBus *bus,
 	return TRUE;
 }
 
+static void
+brasero_transcode_error_on_pad_linking (BraseroTranscode *self)
+{
+	BraseroTranscodePrivate *priv;
+	GstMessage *message;
+	GstBus *bus;
+
+	priv = BRASERO_TRANSCODE_PRIVATE (self);
+
+	BRASERO_JOB_LOG (self, "Error on pad linking");
+	message = gst_message_new_error (GST_OBJECT (priv->pipeline),
+					 g_error_new (BRASERO_BURN_ERROR,
+						      BRASERO_BURN_ERROR_GENERAL,
+						      _("Impossible to link plugin pads")),
+					 "Sent by brasero_metadata_error_on_pad_linking");
+
+	bus = gst_pipeline_get_bus (GST_PIPELINE (priv->pipeline));
+	gst_bus_post (bus, message);
+	g_object_unref (bus);
+}
+
 static void
 brasero_transcode_new_decoded_pad_cb (GstElement *decode,
 				      GstPad *pad,
 				      gboolean arg2,
 				      BraseroTranscode *transcode)
 {
-	GstPad *sink;
 	GstCaps *caps;
 	GstStructure *structure;
 	BraseroTranscodePrivate *priv;
 
 	priv = BRASERO_TRANSCODE_PRIVATE (transcode);
 
-	sink = gst_element_get_pad (priv->link, "sink");
-	if (GST_PAD_IS_LINKED (sink))
-		return;
+	BRASERO_JOB_LOG (transcode, "New pad");
 
 	/* make sure we only have audio */
 	caps = gst_pad_get_caps (pad);
 	if (!caps)
 		return;
 
-	/* before linking pads (before any data reach grvolume), send tags */
-	brasero_transcode_send_volume_event (transcode);
-
 	structure = gst_caps_get_structure (caps, 0);
-	if (structure
-	&&  g_strrstr (gst_structure_get_name (structure), "audio"))
-		gst_pad_link (pad, sink);
+	if (structure) {
+		if (g_strrstr (gst_structure_get_name (structure), "audio")) {
+			GstPad *sink;
+			GstElement *queue;
+			GstPadLinkReturn res;
 
-	gst_object_unref (sink);
+			/* before linking pads (before any data reach grvolume), send tags */
+			brasero_transcode_send_volume_event (transcode);
+
+			/* This is necessary in case there is a video stream
+			 * (see brasero-metadata.c). we need to queue to avoid
+			 * a deadlock. */
+			queue = gst_element_factory_make ("queue", NULL);
+			gst_bin_add (GST_BIN (priv->pipeline), queue);
+			if (!gst_element_link (queue, priv->link)) {
+				brasero_transcode_error_on_pad_linking (transcode);
+				goto end;
+			}
+
+			sink = gst_element_get_pad (queue, "sink");
+			if (GST_PAD_IS_LINKED (sink)) {
+				brasero_transcode_error_on_pad_linking (transcode);
+				goto end;
+			}
+
+			res = gst_pad_link (pad, sink);
+			if (res == GST_PAD_LINK_OK)
+				gst_element_set_state (queue, GST_STATE_PLAYING);
+			else
+				brasero_transcode_error_on_pad_linking (transcode);
+
+			gst_object_unref (sink);
+		}
+		/* For video streams add a fakesink (see brasero-metadata.c) */
+		else if (g_strrstr (gst_structure_get_name (structure), "video")) {
+			GstPad *sink;
+			GstElement *fakesink;
+			GstPadLinkReturn res;
+
+			BRASERO_JOB_LOG (transcode, "Adding a fakesink for video stream");
+
+			fakesink = gst_element_factory_make ("fakesink", NULL);
+			if (!fakesink) {
+				brasero_transcode_error_on_pad_linking (transcode);
+				goto end;
+			}
+
+			sink = gst_element_get_static_pad (fakesink, "sink");
+			if (!sink) {
+				brasero_transcode_error_on_pad_linking (transcode);
+				gst_object_unref (fakesink);
+				goto end;
+			}
+
+			gst_bin_add (GST_BIN (priv->pipeline), fakesink);
+			res = gst_pad_link (pad, sink);
+
+			if (res == GST_PAD_LINK_OK)
+				gst_element_set_state (fakesink, GST_STATE_PLAYING);
+			else
+				brasero_transcode_error_on_pad_linking (transcode);
+
+			gst_object_unref (sink);
+		}
+	}
+
+end:
 	gst_caps_unref (caps);
 }
 
@@ -1438,7 +1540,22 @@ brasero_transcode_export_caps (BraseroPlugin *plugin, gchar **error)
 
 	output = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_FILE|
 					 BRASERO_PLUGIN_IO_ACCEPT_PIPE,
-					 BRASERO_AUDIO_FORMAT_RAW);
+					 BRASERO_AUDIO_FORMAT_RAW|
+					 BRASERO_AUDIO_FORMAT_44100|
+					 BRASERO_METADATA_INFO);
+
+	input = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_FILE,
+					BRASERO_AUDIO_FORMAT_UNDEFINED|
+					BRASERO_METADATA_INFO);
+
+	brasero_plugin_link_caps (plugin, output, input);
+	g_slist_free (output);
+	g_slist_free (input);
+
+	output = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_FILE|
+					 BRASERO_PLUGIN_IO_ACCEPT_PIPE,
+					 BRASERO_AUDIO_FORMAT_RAW|
+					 BRASERO_AUDIO_FORMAT_44100);
 
 	input = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_FILE,
 					BRASERO_AUDIO_FORMAT_UNDEFINED);
